@@ -57,6 +57,7 @@
  * Instead do sizeof(table) in calling procedure.
  *****************************************************************************
  */
+#if 0  // Unused with GDAL.
 static const char *Lookup(const char * const *table, size_t n, size_t index)
 {
    static const char * const def[] =
@@ -71,6 +72,7 @@ static const char *Lookup(const char * const *table, size_t n, size_t index)
       return def[2];
    }
 }
+#endif
 
 /*****************************************************************************
  * Print() --
@@ -116,7 +118,7 @@ static const char *Lookup(const char * const *table, size_t n, size_t index)
  * Using enumerated type instead of "ds" "dss" etc.  For speed considerations.
  *****************************************************************************
  */
-char *Print(const char *label, const char *varName, Prt_TYPE fmt, ...)
+char *Print(const char *label, const char *varName, int fmt, ...)
 {
    static char *buffer = NULL; /* Copy of message generated so far. */
    va_list ap;          /* pointer to variable argument list. */
@@ -197,6 +199,7 @@ char *Print(const char *label, const char *varName, Prt_TYPE fmt, ...)
    return NULL;
 }
 
+#if 0  // Unused with GDAL.
 /*****************************************************************************
  * PrintSect1() --
  *
@@ -828,7 +831,7 @@ static int PrintSect4 (grib_MetaData *meta, sChar f_unit)
                 sect4->upperLimit.value *
                 pow (10.0, -1 * sect4->upperLimit.factor), buffer);
          /* Intentionally fall through. */
-         // CPL_FALLTHROUGH
+         CPL_FALLTHROUGH
       case GS4_STATISTIC:
 /*         strftime (buffer, 100, "%m/%d/%Y %H:%M:%S UTC",
                    gmtime (&(sect4->validTime)));*/
@@ -1412,3 +1415,4 @@ int MetaPrint (grib_MetaData *meta, char **ans, sChar decimal, sChar f_unit)
    *ans = Print (NULL, NULL, Prt_NULL);
    return 0;
 }
+#endif  // Unused with GDAL.
