@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id: memdataset.cpp 33838 2016-03-31 20:42:32Z goatbar $
  *
  * Project:  Memory Array Translator
  * Purpose:  Complete implementation.
@@ -28,11 +27,26 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "cpl_string.h"
-#include "gdal_frmts.h"
+#include "cpl_port.h"
 #include "memdataset.h"
 
-CPL_CVSID("$Id: memdataset.cpp 33838 2016-03-31 20:42:32Z goatbar $");
+#include <climits>
+#include <cstdlib>
+#include <cstring>
+
+#include <vector>
+
+#include "cpl_config.h"
+#include "cpl_conv.h"
+#include "cpl_error.h"
+#include "cpl_minixml.h"
+#include "cpl_progress.h"
+#include "cpl_string.h"
+#include "cpl_vsi.h"
+#include "gdal.h"
+#include "gdal_frmts.h"
+
+CPL_CVSID("$Id: memdataset.cpp 37462 2017-02-26 02:13:03Z goatbar $");
 
 /************************************************************************/
 /*                        MEMCreateRasterBand()                         */
@@ -69,13 +83,45 @@ GDALRasterBandH MEMCreateRasterBandEx( GDALDataset *poDS, int nBand,
 /*                           MEMRasterBand()                            */
 /************************************************************************/
 
+MEMRasterBand::MEMRasterBand( GByte *pabyDataIn, GDALDataType eTypeIn,
+                              int nXSizeIn, int nYSizeIn ) :
+    GDALPamRasterBand(FALSE),
+    pabyData(pabyDataIn),
+    nPixelOffset(0),
+    nLineOffset(0),
+    bOwnData(true),
+    bNoDataSet(FALSE),
+    dfNoData(0.0),
+    poColorTable(NULL),
+    eColorInterp(GCI_Undefined),
+    pszUnitType(NULL),
+    papszCategoryNames(NULL),
+    dfOffset(0.0),
+    dfScale(1.0),
+    psSavedHistograms(NULL)
+{
+    eAccess = GA_Update;
+    eDataType = eTypeIn;
+    nRasterXSize = nXSizeIn;
+    nRasterYSize = nYSizeIn;
+    nBlockXSize = nXSizeIn;
+    nBlockYSize = 1;
+    nPixelOffset = GDALGetDataTypeSizeBytes(eTypeIn);
+    nLineOffset = nPixelOffset * static_cast<size_t>(nBlockXSize);
+}
+
+/************************************************************************/
+/*                           MEMRasterBand()                            */
+/************************************************************************/
+
 MEMRasterBand::MEMRasterBand( GDALDataset *poDSIn, int nBandIn,
                               GByte *pabyDataIn, GDALDataType eTypeIn,
                               GSpacing nPixelOffsetIn, GSpacing nLineOffsetIn,
-                              int bAssumeOwnership, const char * pszPixelType) :
+                              int bAssumeOwnership, const char * pszPixelType ) :
     GDALPamRasterBand(FALSE),
     pabyData(pabyDataIn),
-    // Skip nPixelOffset and nLineOffset.
+    nPixelOffset(nPixelOffsetIn),
+    nLineOffset(nLineOffsetIn),
     bOwnData(bAssumeOwnership),
     bNoDataSet(FALSE),
     dfNoData(0.0),
@@ -98,14 +144,10 @@ MEMRasterBand::MEMRasterBand( GDALDataset *poDSIn, int nBandIn,
     nBlockYSize = 1;
 
     if( nPixelOffsetIn == 0 )
-        nPixelOffsetIn = GDALGetDataTypeSizeBytes(eTypeIn);
+        nPixelOffset = GDALGetDataTypeSizeBytes(eTypeIn);
 
     if( nLineOffsetIn == 0 )
-        nLineOffsetIn = nPixelOffsetIn * static_cast<size_t>(nBlockXSize);
-
-    nPixelOffset = nPixelOffsetIn;
-    nLineOffset = nLineOffsetIn;
-    bOwnData = bAssumeOwnership;
+        nLineOffset = nPixelOffset * static_cast<size_t>(nBlockXSize);
 
     if( pszPixelType && EQUAL(pszPixelType,"SIGNEDBYTE") )
         SetMetadataItem( "PIXELTYPE", "SIGNEDBYTE", "IMAGE_STRUCTURE" );
@@ -133,7 +175,6 @@ MEMRasterBand::~MEMRasterBand()
         CPLDestroyXMLNode(psSavedHistograms);
 }
 
-
 /************************************************************************/
 /*                             IReadBlock()                             */
 /************************************************************************/
@@ -655,7 +696,8 @@ MEMRasterBand::GetDefaultHistogram( double *pdfMin, double *pdfMax,
     if( psSavedHistograms != NULL )
     {
         for( CPLXMLNode *psXMLHist = psSavedHistograms->psChild;
-             psXMLHist != NULL; psXMLHist = psXMLHist->psNext )
+             psXMLHist != NULL;
+             psXMLHist = psXMLHist->psNext )
         {
             if( psXMLHist->eType != CXT_Element
                 || !EQUAL(psXMLHist->pszValue,"HistItem") )
@@ -677,13 +719,80 @@ MEMRasterBand::GetDefaultHistogram( double *pdfMin, double *pdfMax,
                                                 pfnProgress,pProgressData);
 }
 
+/************************************************************************/
+/*                          GetOverviewCount()                          */
+/************************************************************************/
+
+int MEMRasterBand::GetOverviewCount()
+{
+    MEMDataset* poMemDS = dynamic_cast<MEMDataset*>(poDS);
+    if( poMemDS == NULL )
+        return 0;
+    return poMemDS->m_nOverviewDSCount;
+}
+
+/************************************************************************/
+/*                            GetOverview()                             */
+/************************************************************************/
+
+GDALRasterBand * MEMRasterBand::GetOverview( int i )
+
+{
+    MEMDataset* poMemDS = dynamic_cast<MEMDataset*>(poDS);
+    if( poMemDS == NULL )
+        return NULL;
+    if( i < 0 || i >= poMemDS->m_nOverviewDSCount )
+        return NULL;
+    return poMemDS->m_papoOverviewDS[i]->GetRasterBand(nBand);
+}
+
+/************************************************************************/
+/*                         CreateMaskBand()                             */
+/************************************************************************/
+
+CPLErr MEMRasterBand::CreateMaskBand( int nFlagsIn )
+{
+    InvalidateMaskBand();
+
+    MEMDataset* poMemDS = dynamic_cast<MEMDataset*>(poDS);
+    if( (nFlagsIn & GMF_PER_DATASET) != 0 && nBand != 1 && poMemDS != NULL )
+    {
+        MEMRasterBand* poFirstBand =
+            reinterpret_cast<MEMRasterBand*>(poMemDS->GetRasterBand(1));
+        if( poFirstBand != NULL)
+            return poFirstBand->CreateMaskBand( nFlagsIn );
+    }
+
+    GByte* pabyMaskData = static_cast<GByte*>(VSI_CALLOC_VERBOSE(nRasterXSize,
+                                                                 nRasterYSize));
+    if( pabyMaskData == NULL )
+        return CE_Failure;
+
+    nMaskFlags = nFlagsIn;
+    bOwnMask = true;
+    poMask = new MEMRasterBand( pabyMaskData, GDT_Byte,
+                                nRasterXSize, nRasterYSize );
+    if( (nFlagsIn & GMF_PER_DATASET) != 0 && nBand == 1 && poMemDS != NULL )
+    {
+        for( int i = 2; i <= poMemDS->GetRasterCount(); ++i )
+        {
+            MEMRasterBand* poOtherBand =
+                reinterpret_cast<MEMRasterBand*>(poMemDS->GetRasterBand(i));
+            poOtherBand->InvalidateMaskBand();
+            poOtherBand->nMaskFlags = nFlagsIn;
+            poOtherBand->bOwnMask = false;
+            poOtherBand->poMask = poMask;
+        }
+    }
+    return CE_None;
+}
+
 /************************************************************************/
 /* ==================================================================== */
 /*      MEMDataset                                                     */
 /* ==================================================================== */
 /************************************************************************/
 
-
 /************************************************************************/
 /*                            MEMDataset()                             */
 /************************************************************************/
@@ -693,7 +802,9 @@ MEMDataset::MEMDataset() :
     bGeoTransformSet(FALSE),
     pszProjection(NULL),
     nGCPCount(0),
-    pasGCPs(NULL)
+    pasGCPs(NULL),
+    m_nOverviewDSCount(0),
+    m_papoOverviewDS(NULL)
 {
     adfGeoTransform[0] = 0.0;
     adfGeoTransform[1] = 1.0;
@@ -701,6 +812,7 @@ MEMDataset::MEMDataset() :
     adfGeoTransform[3] = 0.0;
     adfGeoTransform[4] = 0.0;
     adfGeoTransform[5] = -1.0;
+    DisableReadWriteMutex();
 }
 
 /************************************************************************/
@@ -715,6 +827,10 @@ MEMDataset::~MEMDataset()
 
     GDALDeinitGCPs( nGCPCount, pasGCPs );
     CPLFree( pasGCPs );
+
+    for(int i=0;i<m_nOverviewDSCount;++i)
+        delete m_papoOverviewDS[i];
+    CPLFree(m_papoOverviewDS);
 }
 
 #if 0
@@ -811,7 +927,7 @@ void *MEMDataset::GetInternalHandle( const char * pszRequest )
             if( RequestedRasterBand != NULL )
             {
                 // return the internal band data pointer
-                return(RequestedRasterBand->GetData());
+                return RequestedRasterBand->GetData();
             }
         }
     }
@@ -941,6 +1057,278 @@ CPLErr MEMDataset::AddBand( GDALDataType eType, char **papszOptions )
     return CE_None;
 }
 
+/************************************************************************/
+/*                          IBuildOverviews()                           */
+/************************************************************************/
+
+CPLErr MEMDataset::IBuildOverviews( const char *pszResampling,
+                                     int nOverviews, int *panOverviewList,
+                                     int nListBands, int *panBandList,
+                                     GDALProgressFunc pfnProgress,
+                                     void * pProgressData )
+{
+    if( nBands == 0 )
+    {
+        CPLError( CE_Failure, CPLE_NotSupported,
+                  "Dataset has zero bands." );
+        return CE_Failure;
+    }
+
+    if( nListBands != nBands )
+    {
+        CPLError( CE_Failure, CPLE_NotSupported,
+                  "Generation of overviews in MEM only"
+                  "supported when operating on all bands." );
+        return CE_Failure;
+    }
+
+    if( nOverviews == 0 )
+    {
+        // Cleanup existing overviews
+        for(int i=0;i<m_nOverviewDSCount;++i)
+            delete m_papoOverviewDS[i];
+        CPLFree(m_papoOverviewDS);
+        m_nOverviewDSCount = 0;
+        m_papoOverviewDS = NULL;
+        return CE_None;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Force cascading. Help to get accurate results when masks are    */
+/*      involved.                                                       */
+/* -------------------------------------------------------------------- */
+    if( nOverviews > 1 &&
+        (STARTS_WITH_CI(pszResampling, "AVER") |
+         STARTS_WITH_CI(pszResampling, "GAUSS") ||
+         EQUAL(pszResampling, "CUBIC") ||
+         EQUAL(pszResampling, "CUBICSPLINE") ||
+         EQUAL(pszResampling, "LANCZOS") ||
+         EQUAL(pszResampling, "BILINEAR")) )
+    {
+        double dfTotalPixels = 0;
+        for( int i = 0; i < nOverviews; i++ )
+        {
+            dfTotalPixels +=
+                static_cast<double>(nRasterXSize) * nRasterYSize /
+                    (panOverviewList[i] * panOverviewList[i]);
+        }
+
+        double dfAccPixels = 0;
+        for( int i = 0; i < nOverviews; i++ )
+        {
+            double dfPixels =
+                static_cast<double>(nRasterXSize) * nRasterYSize /
+                    (panOverviewList[i] * panOverviewList[i]);
+            void* pScaledProgress = GDALCreateScaledProgress(
+                    dfAccPixels / dfTotalPixels,
+                    (dfAccPixels + dfPixels) / dfTotalPixels,
+                    pfnProgress, pProgressData );
+            CPLErr eErr = IBuildOverviews(
+                                    pszResampling,
+                                    1, &panOverviewList[i],
+                                    nListBands, panBandList,
+                                    GDALScaledProgress,
+                                    pScaledProgress );
+            GDALDestroyScaledProgress( pScaledProgress );
+            dfAccPixels += dfPixels;
+            if( eErr == CE_Failure )
+                return eErr;
+        }
+        return CE_None;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Establish which of the overview levels we already have, and     */
+/*      which are new.                                                  */
+/* -------------------------------------------------------------------- */
+    GDALRasterBand *poBand = GetRasterBand( 1 );
+
+    for( int i = 0; i < nOverviews; i++ )
+    {
+        bool bExisting = false;
+        for( int j = 0; j < poBand->GetOverviewCount(); j++ )
+        {
+            GDALRasterBand * poOverview = poBand->GetOverview( j );
+            if( poOverview == NULL )
+                continue;
+
+            int nOvFactor =
+                GDALComputeOvFactor(poOverview->GetXSize(),
+                                    poBand->GetXSize(),
+                                    poOverview->GetYSize(),
+                                    poBand->GetYSize());
+
+            if( nOvFactor == panOverviewList[i]
+                || nOvFactor == GDALOvLevelAdjust2( panOverviewList[i],
+                                                   poBand->GetXSize(),
+                                                   poBand->GetYSize() ) )
+            {
+                bExisting = true;
+                break;
+            }
+        }
+
+        // Create new overview dataset if needed.
+        if( !bExisting )
+        {
+            MEMDataset* poOvrDS = new MEMDataset();
+            poOvrDS->eAccess = GA_Update;
+            poOvrDS->nRasterXSize = (nRasterXSize + panOverviewList[i] - 1)
+                                            / panOverviewList[i];
+            poOvrDS->nRasterYSize = (nRasterYSize + panOverviewList[i] - 1)
+                                            / panOverviewList[i];
+            for( int iBand = 0; iBand < nBands; iBand ++ )
+            {
+                const GDALDataType eDT =
+                            GetRasterBand(iBand+1)->GetRasterDataType();
+                if( poOvrDS->AddBand( eDT, NULL ) != CE_None )
+                {
+                    delete poOvrDS;
+                    return CE_Failure;
+                }
+            }
+            m_nOverviewDSCount ++;
+            m_papoOverviewDS = (GDALDataset**) CPLRealloc(m_papoOverviewDS,
+                                    sizeof(GDALDataset*) * m_nOverviewDSCount );
+            m_papoOverviewDS[m_nOverviewDSCount-1] = poOvrDS;
+        }
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Build band list.                                                */
+/* -------------------------------------------------------------------- */
+    GDALRasterBand **pahBands = static_cast<GDALRasterBand **>(
+        CPLCalloc(sizeof(GDALRasterBand *), nBands) );
+    for( int i = 0; i < nBands; i++ )
+        pahBands[i] = GetRasterBand( panBandList[i] );
+
+/* -------------------------------------------------------------------- */
+/*      Refresh overviews that were listed.                             */
+/* -------------------------------------------------------------------- */
+    GDALRasterBand **papoOverviewBands = static_cast<GDALRasterBand **>(
+        CPLCalloc(sizeof(void*), nOverviews) );
+    GDALRasterBand **papoMaskOverviewBands = static_cast<GDALRasterBand **>(
+        CPLCalloc(sizeof(void*), nOverviews) );
+
+    CPLErr eErr = CE_None;
+    for( int iBand = 0; iBand < nBands && eErr == CE_None; iBand++ )
+    {
+        poBand = GetRasterBand( panBandList[iBand] );
+
+        int nNewOverviews = 0;
+        for( int i = 0; i < nOverviews; i++ )
+        {
+            for( int j = 0; j < poBand->GetOverviewCount(); j++ )
+            {
+                GDALRasterBand * poOverview = poBand->GetOverview( j );
+
+                int bHasNoData = FALSE;
+                double noDataValue = poBand->GetNoDataValue(&bHasNoData);
+
+                if( bHasNoData )
+                  poOverview->SetNoDataValue(noDataValue);
+
+                const int nOvFactor =
+                    GDALComputeOvFactor(poOverview->GetXSize(),
+                                        poBand->GetXSize(),
+                                        poOverview->GetYSize(),
+                                        poBand->GetYSize());
+
+                if( nOvFactor == panOverviewList[i]
+                    || nOvFactor == GDALOvLevelAdjust2(panOverviewList[i],
+                                                       poBand->GetXSize(),
+                                                       poBand->GetYSize() ) )
+                {
+                    papoOverviewBands[nNewOverviews++] = poOverview;
+                    break;
+                }
+            }
+        }
+
+        // If the band has an explicit mask, we need to create overviews
+        // for it
+        MEMRasterBand* poMEMBand = reinterpret_cast<MEMRasterBand*>(poBand);
+        const bool bMustGenerateMaskOvr =
+                ( (poMEMBand->bOwnMask && poMEMBand->poMask != NULL) ||
+        // Or if it is a per-dataset mask, in which case just do it for the
+        // first band
+                  ((poMEMBand->nMaskFlags & GMF_PER_DATASET) != 0 && iBand == 0) );
+
+        if( nNewOverviews > 0 && bMustGenerateMaskOvr )
+        {
+            for( int i = 0; i < nNewOverviews; i++ )
+            {
+                MEMRasterBand* poMEMOvrBand =
+                    reinterpret_cast<MEMRasterBand*>(papoOverviewBands[i]);
+                if( !(poMEMOvrBand->bOwnMask && poMEMOvrBand->poMask != NULL) &&
+                    (poMEMOvrBand->nMaskFlags & GMF_PER_DATASET) == 0 )
+                {
+                    poMEMOvrBand->CreateMaskBand( poMEMBand->nMaskFlags );
+                }
+                papoMaskOverviewBands[i] = poMEMOvrBand->GetMaskBand();
+            }
+
+            void* pScaledProgress = GDALCreateScaledProgress(
+                    1.0 * iBand / nBands,
+                    1.0 * (iBand+0.5) / nBands,
+                    pfnProgress, pProgressData );
+
+            MEMRasterBand* poMaskBand = reinterpret_cast<MEMRasterBand*>(
+                                                        poBand->GetMaskBand());
+            // Make the mask band to be its own mask, similarly to what is
+            // done for alpha bands in GDALRegenerateOverviews() (#5640)
+            poMaskBand->InvalidateMaskBand();
+            poMaskBand->bOwnMask = false;
+            poMaskBand->poMask = poMaskBand;
+            poMaskBand->nMaskFlags = 0;
+            eErr = GDALRegenerateOverviews(
+                                        (GDALRasterBandH) poMaskBand,
+                                        nNewOverviews,
+                                        (GDALRasterBandH*)papoMaskOverviewBands,
+                                        pszResampling,
+                                        GDALScaledProgress, pScaledProgress );
+            poMaskBand->InvalidateMaskBand();
+            GDALDestroyScaledProgress( pScaledProgress );
+        }
+
+        // Generate overview of bands *AFTER* mask overviews
+        if( nNewOverviews > 0 && eErr == CE_None  )
+        {
+            void* pScaledProgress = GDALCreateScaledProgress(
+                    1.0 * (iBand+(bMustGenerateMaskOvr ? 0.5 : 1)) / nBands,
+                    1.0 * (iBand+1)/ nBands,
+                    pfnProgress, pProgressData );
+            eErr = GDALRegenerateOverviews( (GDALRasterBandH) poBand,
+                                            nNewOverviews,
+                                            (GDALRasterBandH*)papoOverviewBands,
+                                            pszResampling,
+                                            GDALScaledProgress, pScaledProgress );
+            GDALDestroyScaledProgress( pScaledProgress );
+        }
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Cleanup                                                         */
+/* -------------------------------------------------------------------- */
+    CPLFree( papoOverviewBands );
+    CPLFree( papoMaskOverviewBands );
+    CPLFree( pahBands );
+
+    return eErr;
+}
+
+/************************************************************************/
+/*                         CreateMaskBand()                             */
+/************************************************************************/
+
+CPLErr MEMDataset::CreateMaskBand( int nFlagsIn )
+{
+    GDALRasterBand* poFirstBand = GetRasterBand(1);
+    if( poFirstBand == NULL )
+        return CE_Failure;
+    return poFirstBand->CreateMaskBand( nFlagsIn | GMF_PER_DATASET );
+}
+
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
