@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id $
  *
  * Project:  PDF driver
  * Purpose:  GDALDataset driver for PDF dataset.
@@ -35,8 +34,7 @@
 
 #include "cpl_vsi.h"
 
-CPL_CVSID("$Id: pdfio.cpp 31977 2015-12-03 11:42:22Z rouault $");
-
+CPL_CVSID("$Id: pdfio.cpp 35897 2016-10-24 11:54:24Z goatbar $");
 
 #ifdef POPPLER_BASE_STREAM_HAS_TWO_ARGS
 /* Poppler 0.31.0 is the first one that needs to know the file size */
@@ -53,49 +51,50 @@ static vsi_l_offset VSIPDFFileStreamGetSize(VSILFILE* f)
 /*                         VSIPDFFileStream()                           */
 /************************************************************************/
 
-VSIPDFFileStream::VSIPDFFileStream(VSILFILE* fIn, const char* pszFilename, Object *dictA):
+VSIPDFFileStream::VSIPDFFileStream(
+    VSILFILE* fIn, const char* pszFilename, Object *dictA) :
 #ifdef POPPLER_BASE_STREAM_HAS_TWO_ARGS
-                                                        BaseStream(dictA, (setPos_offset_type)VSIPDFFileStreamGetSize(fIn))
+    BaseStream(dictA, (setPos_offset_type)VSIPDFFileStreamGetSize(fIn)),
 #else
-                                                        BaseStream(dictA)
+    BaseStream(dictA),
 #endif
-{
-    poParent = NULL;
-    poFilename = new GooString(pszFilename);
-    this->f = fIn;
-    nStart = 0;
-    bLimited = gFalse;
-    nLength = 0;
-    nCurrentPos = VSI_L_OFFSET_MAX;
-    bHasSavedPos = FALSE;
-    nSavedPos = 0;
-    nPosInBuffer = nBufferLength = -1;
-}
+    poParent(NULL),
+    poFilename(new GooString(pszFilename)),
+    f(fIn),
+    nStart(0),
+    bLimited(gFalse),
+    nLength(0),
+    nCurrentPos(VSI_L_OFFSET_MAX),
+    bHasSavedPos(FALSE),
+    nSavedPos(0),
+    nPosInBuffer(-1),
+    nBufferLength(-1)
+{}
 
 /************************************************************************/
 /*                         VSIPDFFileStream()                           */
 /************************************************************************/
 
-VSIPDFFileStream::VSIPDFFileStream(VSIPDFFileStream* poParentIn,
-                                   vsi_l_offset startA, GBool limitedA,
-                                   vsi_l_offset lengthA, Object *dictA):
+VSIPDFFileStream::VSIPDFFileStream( VSIPDFFileStream* poParentIn,
+                                    vsi_l_offset startA, GBool limitedA,
+                                    vsi_l_offset lengthA, Object *dictA ) :
 #ifdef POPPLER_BASE_STREAM_HAS_TWO_ARGS
-                                                        BaseStream(dictA, (makeSubStream_offset_type)lengthA)
+    BaseStream(dictA, (makeSubStream_offset_type)lengthA),
 #else
-                                                        BaseStream(dictA)
+    BaseStream(dictA),
 #endif
-{
-    this->poParent = poParentIn;
-    poFilename = poParent->poFilename;
-    f = poParent->f;
-    nStart = startA;
-    bLimited = limitedA;
-    nLength = lengthA;
-    nCurrentPos = VSI_L_OFFSET_MAX;
-    bHasSavedPos = FALSE;
-    nSavedPos = 0;
-    nPosInBuffer = nBufferLength = -1;
-}
+    poParent(poParentIn),
+    poFilename(poParentIn->poFilename),
+    f(poParentIn->f),
+    nStart(startA),
+    bLimited(limitedA),
+    nLength(lengthA),
+    nCurrentPos(VSI_L_OFFSET_MAX),
+    bHasSavedPos(FALSE),
+    nSavedPos(0),
+    nPosInBuffer(-1),
+    nBufferLength(-1)
+{}
 
 /************************************************************************/
 /*                        ~VSIPDFFileStream()                           */
@@ -149,7 +148,6 @@ getPos_ret_type VSIPDFFileStream::getPos()
 /*                                getStart()                            */
 /************************************************************************/
 
-
 getStart_ret_type VSIPDFFileStream::getStart()
 {
     return (getStart_ret_type) nStart;
@@ -293,7 +291,8 @@ void VSIPDFFileStream::reset()
     nSavedPos = VSIFTellL(f);
     bHasSavedPos = TRUE;
     VSIFSeekL(f, nCurrentPos = nStart, SEEK_SET);
-    nPosInBuffer = nBufferLength = -1;
+    nPosInBuffer = -1;
+    nBufferLength = -1;
 }
 
 /************************************************************************/
@@ -343,7 +342,8 @@ void VSIPDFFileStream::setPos(setPos_offset_type pos, int dir)
             newpos = size;
         VSIFSeekL(f, nCurrentPos = size - newpos, SEEK_SET);
     }
-    nPosInBuffer = nBufferLength = -1;
+    nPosInBuffer = -1;
+    nBufferLength = -1;
 }
 
 /************************************************************************/
@@ -354,7 +354,8 @@ void VSIPDFFileStream::moveStart(moveStart_delta_type delta)
 {
     nStart += delta;
     VSIFSeekL(f, nCurrentPos = nStart, SEEK_SET);
-    nPosInBuffer = nBufferLength = -1;
+    nPosInBuffer = -1;
+    nBufferLength = -1;
 }
 
 /************************************************************************/
@@ -381,7 +382,8 @@ int VSIPDFFileStream::getChars(int nChars, Guchar *buffer)
             if (!bLimited && nToRead > BUFFER_SIZE)
             {
                 int nJustRead = (int) VSIFReadL(buffer + nRead, 1, nToRead, f);
-                nPosInBuffer = nBufferLength = -1;
+                nPosInBuffer = -1;
+                nBufferLength = -1;
                 nCurrentPos += nJustRead;
                 nRead += nJustRead;
                 break;
