@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id: cpgdataset.cpp 33928 2016-04-09 19:15:33Z goatbar $
  *
  * Project:  Polarimetric Workstation
  * Purpose:  Convair PolGASP data (.img/.hdr format).
@@ -33,7 +32,7 @@
 #include "ogr_spatialref.h"
 #include "rawdataset.h"
 
-CPL_CVSID("$Id: cpgdataset.cpp 33928 2016-04-09 19:15:33Z goatbar $");
+CPL_CVSID("$Id: cpgdataset.cpp 36763 2016-12-09 22:10:55Z rouault $");
 
 enum Interleave { BSQ, BIL, BIP };
 
@@ -67,7 +66,9 @@ class CPGDataset : public RawDataset
     static int  AdjustFilename( char **, const char *, const char * );
     static int FindType1( const char *pszWorkname );
     static int FindType2( const char *pszWorkname );
+#ifdef notdef
     static int FindType3( const char *pszWorkname );
+#endif
     static GDALDataset *InitializeType1Or2Dataset( const char *pszWorkname );
 #ifdef notdef
     static GDALDataset *InitializeType3Dataset( const char *pszWorkname );
@@ -78,12 +79,12 @@ class CPGDataset : public RawDataset
                 CPGDataset();
     virtual ~CPGDataset();
 
-    virtual int    GetGCPCount();
-    virtual const char *GetGCPProjection();
-    virtual const GDAL_GCP *GetGCPs();
+    virtual int    GetGCPCount() override;
+    virtual const char *GetGCPProjection() override;
+    virtual const GDAL_GCP *GetGCPs() override;
 
-    virtual const char *GetProjectionRef(void);
-    virtual CPLErr GetGeoTransform( double * );
+    virtual const char *GetProjectionRef(void) override;
+    virtual CPLErr GetGeoTransform( double * ) override;
 
     static GDALDataset *Open( GDALOpenInfo * );
 };
@@ -152,7 +153,7 @@ class SIRC_QSLCRasterBand : public GDALRasterBand
                    SIRC_QSLCRasterBand( CPGDataset *, int, GDALDataType );
     virtual ~SIRC_QSLCRasterBand() {}
 
-    virtual CPLErr IReadBlock( int, int, void * );
+    virtual CPLErr IReadBlock( int, int, void * ) override;
 };
 
 static const int M11 = 0;
@@ -182,16 +183,15 @@ class CPG_STOKESRasterBand : public GDALRasterBand
 {
     friend class CPGDataset;
 
-    int nBand;
     int bNativeOrder;
 
   public:
-                   CPG_STOKESRasterBand( GDALDataset *poDS, int nBand,
+                   CPG_STOKESRasterBand( GDALDataset *poDS,
                                          GDALDataType eType,
                                          int bNativeOrder );
     virtual ~CPG_STOKESRasterBand() {};
 
-    virtual CPLErr IReadBlock( int, int, void * );
+    virtual CPLErr IReadBlock( int, int, void * ) override;
 };
 
 /************************************************************************/
@@ -208,14 +208,12 @@ int CPGDataset::AdjustFilename( char **pszFilename,
                                 const char *pszExtension )
 
 {
-    const char *pszNewName;
-
-    /* eventually we should handle upper/lower case ... */
-
+    // TODO: Eventually we should handle upper/lower case.
     if ( EQUAL(pszPolarization,"stokes") )
     {
-        pszNewName = CPLResetExtension((const char *) *pszFilename,
-                                     (const char *) pszExtension);
+        const char *pszNewName =
+            CPLResetExtension((const char *) *pszFilename,
+                              (const char *) pszExtension);
         CPLFree(*pszFilename);
         *pszFilename = CPLStrdup(pszNewName);
     }
@@ -232,15 +230,17 @@ int CPGDataset::AdjustFilename( char **pszFilename,
           return FALSE;
 
         strncpy( subptr, pszPolarization, 2);
-        pszNewName = CPLResetExtension((const char *) *pszFilename,
-                                                (const char *) pszExtension);
+        const char *pszNewName =
+            CPLResetExtension((const char *) *pszFilename,
+                              (const char *) pszExtension);
         CPLFree(*pszFilename);
         *pszFilename = CPLStrdup(pszNewName);
     }
     else
     {
-        pszNewName = CPLResetExtension((const char *) *pszFilename,
-                                        (const char *) pszExtension);
+        const char *pszNewName =
+            CPLResetExtension((const char *) *pszFilename,
+                              (const char *) pszExtension);
         CPLFree(*pszFilename);
         *pszFilename = CPLStrdup(pszNewName);
     }
@@ -299,6 +299,7 @@ int CPGDataset::FindType2( const char *pszFilename )
   return !bNotFound;
 }
 
+#ifdef notdef
 int CPGDataset::FindType3( const char *pszFilename )
 {
   const int nNameLen = static_cast<int>(strlen( pszFilename ));
@@ -319,6 +320,7 @@ int CPGDataset::FindType3( const char *pszFilename )
 
   return !bNotFound;
 }
+#endif
 
 /************************************************************************/
 /*                        LoadStokesLine()                              */
@@ -385,7 +387,6 @@ CPLErr CPGDataset::LoadStokesLine( int iLine, int bNativeOrder )
                 padfStokesMatrix = NULL;
                 nLoadedStokesLine = -1;
                 return CE_Failure;
-
             }
         }
     }
@@ -411,7 +412,6 @@ CPLErr CPGDataset::LoadStokesLine( int iLine, int bNativeOrder )
                 padfStokesMatrix = NULL;
                 nLoadedStokesLine = -1;
                 return CE_Failure;
-
             }
         }
     }
@@ -443,13 +443,19 @@ GDALDataset* CPGDataset::InitializeType1Or2Dataset( const char *pszFilename )
 
     /* Parameters required for pseudo-geocoding.  GCPs map */
     /* slant range to ground range at 16 points.           */
-    int iGeoParamsFound = 0, itransposed = 0;
-    double dfaltitude = 0.0, dfnear_srd = 0.0;
-    double dfsample_size = 0.0, dfsample_size_az = 0.0;
+    int iGeoParamsFound = 0;
+    int itransposed = 0;
+    double dfaltitude = 0.0;
+    double dfnear_srd = 0.0;
+    double dfsample_size = 0.0;
+    double dfsample_size_az = 0.0;
 
     /* Parameters in geogratis geocoded images */
-    int iUTMParamsFound = 0, iUTMZone=0 /* , iCorner=0 */;
-    double dfnorth = 0.0, dfeast = 0.0;
+    int iUTMParamsFound = 0;
+    int iUTMZone = 0;
+    // int iCorner = 0;
+    double dfnorth = 0.0;
+    double dfeast = 0.0;
 
     char* pszWorkname = CPLStrdup(pszFilename);
     AdjustFilename( &pszWorkname, "hh", "hdr" );
@@ -603,9 +609,8 @@ GDALDataset* CPGDataset::InitializeType1Or2Dataset( const char *pszFilename )
         }
         for( int iBand = 0; iBand < 4; iBand++ )
         {
-            SIRC_QSLCRasterBand *poBand;
-
-            poBand = new SIRC_QSLCRasterBand( poDS, iBand+1, GDT_CFloat32 );
+            SIRC_QSLCRasterBand *poBand =
+                new SIRC_QSLCRasterBand( poDS, iBand+1, GDT_CFloat32 );
             poDS->SetBand( iBand+1, poBand );
             poBand->SetMetadataItem( "POLARIMETRIC_INTERP",
                                  apszPolarizations[iBand] );
@@ -657,7 +662,8 @@ GDALDataset* CPGDataset::InitializeType1Or2Dataset( const char *pszFilename )
         double dfnorth_center;
         if (itransposed == 1)
         {
-            printf( "Warning: did not have a convair SIRC-style test dataset\n"
+            CPLError(CE_Warning, CPLE_AppDefined,
+                     "Did not have a convair SIRC-style test dataset\n"
                     "with transposed=1 for testing.  Georeferencing may be "
                     "wrong.\n" );
             dfnorth_center = dfnorth - nSamples*dfsample_size/2.0;
@@ -718,7 +724,6 @@ GDALDataset* CPGDataset::InitializeType1Or2Dataset( const char *pszFilename )
                 /* -1 so that 0,0 maps to largest Y */
                 dfgcpY = -1*sqrt( dftemp*dftemp - dfaltitude*dfaltitude );
                 dfgcpX = dfgcpPixel*dfsample_size_az;
-
             }
             else
             {
@@ -736,7 +741,6 @@ GDALDataset* CPGDataset::InitializeType1Or2Dataset( const char *pszFilename )
                 dftemp = dfnear_srd + (dfsample_size*dfgcpPixel);
                 dfgcpX = sqrt( dftemp*dftemp - dfaltitude*dfaltitude );
                 dfgcpY = (nLines - dfgcpLine)*dfsample_size_az;
-
             }
             poDS->pasGCPList[ngcp].dfGCPX = dfgcpX;
             poDS->pasGCPList[ngcp].dfGCPY = dfgcpY;
@@ -747,14 +751,12 @@ GDALDataset* CPGDataset::InitializeType1Or2Dataset( const char *pszFilename )
 
             CPLFree(poDS->pasGCPList[ngcp].pszId);
             poDS->pasGCPList[ngcp].pszId = CPLStrdup( szID );
-
         }
 
         CPLFree(poDS->pszGCPProjection);
         poDS->pszGCPProjection = CPLStrdup(
             "LOCAL_CS[\"Ground range view / unreferenced meters\","
             "UNIT[\"Meter\",1.0]]");
-
     }
 
     CPLFree(pszWorkname);
@@ -765,14 +767,22 @@ GDALDataset* CPGDataset::InitializeType1Or2Dataset( const char *pszFilename )
 #ifdef notdef
 GDALDataset *CPGDataset::InitializeType3Dataset( const char *pszFilename )
 {
-    int iBytesPerPixel = 0, iInterleave=-1;
-    int nLines = 0, nSamples = 0, nBands = 0;
+    int iBytesPerPixel = 0;
+    int iInterleave = -1;
+    int nLines = 0;
+    int nSamples = 0;
+    int nBands = 0;
     int nError = 0;
 
     /* Parameters in geogratis geocoded images */
-    int iUTMParamsFound = 0, iUTMZone=0;
-    double dfnorth = 0.0, dfeast = 0.0, dfOffsetX = 0.0, dfOffsetY = 0.0;
-    double dfxsize = 0.0, dfysize = 0.0;
+    int iUTMParamsFound = 0;
+    int iUTMZone = 0;
+    double dfnorth = 0.0;
+    double dfeast = 0.0;
+    double dfOffsetX = 0.0;
+    double dfOffsetY = 0.0;
+    double dfxsize = 0.0;
+    double dfysize = 0.0;
 
     char* pszWorkname = CPLStrdup(pszFilename);
     AdjustFilename( &pszWorkname, "stokes", "img_def" );
@@ -976,7 +986,7 @@ GDALDataset *CPGDataset::InitializeType3Dataset( const char *pszFilename )
     for( int iBand = 0; iBand < 16; iBand++ )
     {
         CPG_STOKESRasterBand *poBand
-            = new CPG_STOKESRasterBand( poDS, iBand+1, GDT_CFloat32,
+            = new CPG_STOKESRasterBand( poDS, GDT_CFloat32,
                                         !CPL_IS_LSB );
         poDS->SetBand( iBand+1, poBand );
     }
@@ -1096,7 +1106,7 @@ GDALDataset *CPGDataset::Open( GDALOpenInfo * poOpenInfo )
     }
 
     /* Read the header info and create the dataset */
-    CPGDataset *poDS;
+    CPGDataset *poDS = NULL;
 
 #ifdef notdef
     if ( CPGType < 3 )
@@ -1107,11 +1117,10 @@ GDALDataset *CPGDataset::Open( GDALOpenInfo * poOpenInfo )
     else
       poDS = reinterpret_cast<CPGDataset *>(
           InitializeType3Dataset( poOpenInfo->pszFilename ) );
-    if (poDS == NULL)
+    if( poDS == NULL )
         return NULL;
 #endif
 
-
 /* -------------------------------------------------------------------- */
 /*      Check for overviews.                                            */
 /* -------------------------------------------------------------------- */
@@ -1325,28 +1334,28 @@ CPLErr SIRC_QSLCRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
 /*                        CPG_STOKESRasterBand()                        */
 /************************************************************************/
 
-CPG_STOKESRasterBand::CPG_STOKESRasterBand( GDALDataset *poDSIn, int nBandIn,
+CPG_STOKESRasterBand::CPG_STOKESRasterBand( GDALDataset *poDSIn,
                                             GDALDataType eType,
                                             int bNativeOrderIn ) :
-    nBand(nBandIn),
     bNativeOrder(bNativeOrderIn)
 {
-    static const char * const apszPolarizations[16] = { "Covariance_11",
-                                                 "Covariance_12",
-                                                 "Covariance_13",
-                                                 "Covariance_14",
-                                                 "Covariance_21",
-                                                 "Covariance_22",
-                                                 "Covariance_23",
-                                                 "Covariance_24",
-                                                 "Covariance_31",
-                                                 "Covariance_32",
-                                                 "Covariance_33",
-                                                 "Covariance_34",
-                                                 "Covariance_41",
-                                                 "Covariance_42",
-                                                 "Covariance_43",
-                                                 "Covariance_44" };
+    static const char * const apszPolarizations[16] = {
+        "Covariance_11",
+        "Covariance_12",
+        "Covariance_13",
+        "Covariance_14",
+        "Covariance_21",
+        "Covariance_22",
+        "Covariance_23",
+        "Covariance_24",
+        "Covariance_31",
+        "Covariance_32",
+        "Covariance_33",
+        "Covariance_34",
+        "Covariance_41",
+        "Covariance_42",
+        "Covariance_43",
+        "Covariance_44" };
 
     poDS = poDSIn;
     eDataType = eType;
@@ -1553,7 +1562,6 @@ CPLErr CPG_STOKESRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
             m33 += step;
             m44 += step;
         }
-
     }
     else if ( nBand == 12 ) /* C34 */
     {
