@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id: fastdataset.cpp 33935 2016-04-10 06:47:56Z goatbar $
  *
  * Project:  EOSAT FAST Format reader
  * Purpose:  Reads Landsat FAST-L7A, IRS 1C/1D
@@ -33,7 +32,7 @@
 #include "ogr_spatialref.h"
 #include "rawdataset.h"
 
-CPL_CVSID("$Id: fastdataset.cpp 33935 2016-04-10 06:47:56Z goatbar $");
+CPL_CVSID("$Id: fastdataset.cpp 39148 2017-06-17 07:09:58Z rouault $");
 
 // static const int ADM_STD_HEADER_SIZE = 4608;  // Format specification says it
 static const int ADM_HEADER_SIZE = 5000;  // Should be 4608, but some vendors
@@ -124,12 +123,12 @@ class FASTDataset : public GDALPamDataset
 
     static GDALDataset *Open( GDALOpenInfo * );
 
-    CPLErr      GetGeoTransform( double * );
-    const char  *GetProjectionRef();
+    CPLErr      GetGeoTransform( double * ) override;
+    const char  *GetProjectionRef() override;
     VSILFILE    *FOpenChannel( const char *, int iBand, int iFASTBand );
     void        TryEuromap_IRS_1C_1D_ChannelNameConvention();
 
-    virtual  char** GetFileList();
+    virtual  char** GetFileList() override;
 };
 
 /************************************************************************/
@@ -147,7 +146,6 @@ class FASTRasterBand : public RawRasterBand
                                 int, int, GDALDataType, int );
 };
 
-
 /************************************************************************/
 /*                           FASTRasterBand()                           */
 /************************************************************************/
@@ -243,7 +241,7 @@ char** FASTDataset::GetFileList()
 
     for( int i = 0; i < 6; i++ )
     {
-        if (apoChannelFilenames[i].size() > 0)
+        if (!apoChannelFilenames[i].empty())
             papszFileList =
                 CSLAddString(papszFileList, apoChannelFilenames[i].c_str());
     }
@@ -267,7 +265,6 @@ int FASTDataset::OpenChannel( const char *pszFilenameIn, int iBand )
 /*                             FOpenChannel()                           */
 /************************************************************************/
 
-
 VSILFILE *FASTDataset::FOpenChannel( const char *pszBandname,
                                      int iBand, int iFASTBand )
 {
@@ -904,6 +901,12 @@ GDALDataset *FASTDataset::Open( GDALOpenInfo * poOpenInfo )
             poDS->SetMetadataItem( CPLSPrintf(pszFirst, i ), pszValue );
             CPLFree( pszValue );
         }
+        else
+        {
+            CPLFree(pszHeader);
+            delete poDS;
+            return NULL;
+        }
         pszTemp += nValueLen;
         pszTemp = strpbrk( pszTemp, "-.0123456789" );
         if ( pszTemp )
@@ -915,6 +918,12 @@ GDALDataset *FASTDataset::Open( GDALOpenInfo * poOpenInfo )
             poDS->SetMetadataItem( CPLSPrintf(pszSecond, i ), pszValue );
             CPLFree( pszValue );
         }
+        else
+        {
+            CPLFree(pszHeader);
+            delete poDS;
+            return NULL;
+        }
         pszTemp += nValueLen;
     }
 
@@ -965,49 +974,60 @@ GDALDataset *FASTDataset::Open( GDALOpenInfo * poOpenInfo )
         {
             pszTemp = strpbrk( pszTemp, "-.0123456789" );
             if ( pszTemp )
+            {
                 adfProjParms[i] = CPLScanDouble( pszTemp, VALUE_SIZE );
-            pszTemp = strpbrk( pszTemp, " \t" );
+                pszTemp = strpbrk( pszTemp, " \t" );
+            }
+            if (pszTemp == NULL )
+            {
+                CPLFree(pszHeader);
+                delete poDS;
+                return NULL;
+            }
         }
     }
 
     // Coordinates should follow the word "PROJECTION", otherwise we can
     // be confused by other occurrences of the corner keywords.
     char *pszGeomRecord = strstr( pszHeader, "PROJECTION" );
-    // Read corner coordinates
-    pszTemp = strstr( pszGeomRecord, CORNER_UPPER_LEFT );
-    if ( pszTemp && !EQUAL( pszTemp, "" ) )
+    if( pszGeomRecord )
     {
-        pszTemp += strlen( CORNER_UPPER_LEFT ) + 28;
-        dfULX = CPLScanDouble( pszTemp, CORNER_VALUE_SIZE );
-        pszTemp += CORNER_VALUE_SIZE + 1;
-        dfULY = CPLScanDouble( pszTemp, CORNER_VALUE_SIZE );
-    }
+        // Read corner coordinates
+        pszTemp = strstr( pszGeomRecord, CORNER_UPPER_LEFT );
+        if ( pszTemp && !EQUAL( pszTemp, "" ) )
+        {
+            pszTemp += strlen( CORNER_UPPER_LEFT ) + 28;
+            dfULX = CPLScanDouble( pszTemp, CORNER_VALUE_SIZE );
+            pszTemp += CORNER_VALUE_SIZE + 1;
+            dfULY = CPLScanDouble( pszTemp, CORNER_VALUE_SIZE );
+        }
 
-    pszTemp = strstr( pszGeomRecord, CORNER_UPPER_RIGHT );
-    if ( pszTemp && !EQUAL( pszTemp, "" ) )
-    {
-        pszTemp += strlen( CORNER_UPPER_RIGHT ) + 28;
-        dfURX = CPLScanDouble( pszTemp, CORNER_VALUE_SIZE );
-        pszTemp += CORNER_VALUE_SIZE + 1;
-        dfURY = CPLScanDouble( pszTemp, CORNER_VALUE_SIZE );
-    }
+        pszTemp = strstr( pszGeomRecord, CORNER_UPPER_RIGHT );
+        if ( pszTemp && !EQUAL( pszTemp, "" ) )
+        {
+            pszTemp += strlen( CORNER_UPPER_RIGHT ) + 28;
+            dfURX = CPLScanDouble( pszTemp, CORNER_VALUE_SIZE );
+            pszTemp += CORNER_VALUE_SIZE + 1;
+            dfURY = CPLScanDouble( pszTemp, CORNER_VALUE_SIZE );
+        }
 
-    pszTemp = strstr( pszGeomRecord, CORNER_LOWER_LEFT );
-    if ( pszTemp && !EQUAL( pszTemp, "" ) )
-    {
-        pszTemp += strlen( CORNER_LOWER_LEFT ) + 28;
-        dfLLX = CPLScanDouble( pszTemp, CORNER_VALUE_SIZE );
-        pszTemp += CORNER_VALUE_SIZE + 1;
-        dfLLY = CPLScanDouble( pszTemp, CORNER_VALUE_SIZE );
-    }
+        pszTemp = strstr( pszGeomRecord, CORNER_LOWER_LEFT );
+        if ( pszTemp && !EQUAL( pszTemp, "" ) )
+        {
+            pszTemp += strlen( CORNER_LOWER_LEFT ) + 28;
+            dfLLX = CPLScanDouble( pszTemp, CORNER_VALUE_SIZE );
+            pszTemp += CORNER_VALUE_SIZE + 1;
+            dfLLY = CPLScanDouble( pszTemp, CORNER_VALUE_SIZE );
+        }
 
-    pszTemp = strstr( pszGeomRecord, CORNER_LOWER_RIGHT );
-    if ( pszTemp && !EQUAL( pszTemp, "" ) )
-    {
-        pszTemp += strlen( CORNER_LOWER_RIGHT ) + 28;
-        dfLRX = CPLScanDouble( pszTemp, CORNER_VALUE_SIZE );
-        pszTemp += CORNER_VALUE_SIZE + 1;
-        dfLRY = CPLScanDouble( pszTemp, CORNER_VALUE_SIZE );
+        pszTemp = strstr( pszGeomRecord, CORNER_LOWER_RIGHT );
+        if ( pszTemp && !EQUAL( pszTemp, "" ) )
+        {
+            pszTemp += strlen( CORNER_LOWER_RIGHT ) + 28;
+            dfLRX = CPLScanDouble( pszTemp, CORNER_VALUE_SIZE );
+            pszTemp += CORNER_VALUE_SIZE + 1;
+            dfLRY = CPLScanDouble( pszTemp, CORNER_VALUE_SIZE );
+        }
     }
 
     if ( dfULX != 0.0 && dfULY != 0.0
