@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id: vrtrawrasterband.cpp 33720 2016-03-15 00:39:53Z goatbar $
  *
  * Project:  Virtual GDAL Datasets
  * Purpose:  Implementation of VRTRawRasterBand
@@ -28,12 +27,27 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
+#include "cpl_port.h"
+#include "rawdataset.h"
 #include "vrtdataset.h"
+
+#include <cerrno>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
+
+#include "cpl_conv.h"
+#include "cpl_error.h"
+#include "cpl_hash_set.h"
 #include "cpl_minixml.h"
 #include "cpl_string.h"
-#include "rawdataset.h"
+#include "cpl_vsi.h"
+#include "gdal.h"
+#include "gdal_priv.h"
 
-CPL_CVSID("$Id: vrtrawrasterband.cpp 33720 2016-03-15 00:39:53Z goatbar $");
+CPL_CVSID("$Id: vrtrawrasterband.cpp 37310 2017-02-06 11:42:54Z goatbar $");
+
+/*! @cond Doxygen_Suppress */
 
 /************************************************************************/
 /* ==================================================================== */
@@ -47,12 +61,15 @@ CPL_CVSID("$Id: vrtrawrasterband.cpp 33720 2016-03-15 00:39:53Z goatbar $");
 
 VRTRawRasterBand::VRTRawRasterBand( GDALDataset *poDSIn, int nBandIn,
                                     GDALDataType eType ) :
-    m_poRawRaster(NULL), m_pszSourceFilename(NULL), m_bRelativeToVRT(FALSE)
+    m_poRawRaster(NULL),
+    m_pszSourceFilename(NULL),
+    m_bRelativeToVRT(FALSE)
 {
     Initialize( poDSIn->GetRasterXSize(), poDSIn->GetRasterYSize() );
 
-    this->poDS = poDSIn;
-    this->nBand = nBandIn;
+    // Declared in GDALRasterBand.
+    poDS = poDSIn;
+    nBand = nBandIn;
 
     if( eType != GDT_Unknown )
         eDataType = eType;
@@ -74,12 +91,13 @@ VRTRawRasterBand::~VRTRawRasterBand()
 /************************************************************************/
 
 CPLErr VRTRawRasterBand::IRasterIO( GDALRWFlag eRWFlag,
-                                 int nXOff, int nYOff, int nXSize, int nYSize,
-                                 void * pData, int nBufXSize, int nBufYSize,
-                                 GDALDataType eBufType,
-                                 GSpacing nPixelSpace,
-                                 GSpacing nLineSpace,
-                                 GDALRasterIOExtraArg* psExtraArg)
+                                    int nXOff, int nYOff,
+                                    int nXSize, int nYSize,
+                                    void * pData, int nBufXSize, int nBufYSize,
+                                    GDALDataType eBufType,
+                                    GSpacing nPixelSpace,
+                                    GSpacing nLineSpace,
+                                    GDALRasterIOExtraArg* psExtraArg )
 {
     if( m_poRawRaster == NULL )
     {
@@ -92,9 +110,9 @@ CPLErr VRTRawRasterBand::IRasterIO( GDALRWFlag eRWFlag,
     {
         CPLError( CE_Failure, CPLE_NoWriteAccess,
                   "Attempt to write to read only dataset in"
-                  "VRTRawRasterBand::IRasterIO().\n" );
+                  "VRTRawRasterBand::IRasterIO()." );
 
-        return( CE_Failure );
+        return CE_Failure;
     }
 
 /* -------------------------------------------------------------------- */
@@ -106,15 +124,17 @@ CPLErr VRTRawRasterBand::IRasterIO( GDALRWFlag eRWFlag,
     {
         if( OverviewRasterIO( eRWFlag, nXOff, nYOff, nXSize, nYSize,
                               pData, nBufXSize, nBufYSize,
-                              eBufType, nPixelSpace, nLineSpace, psExtraArg ) == CE_None )
+                              eBufType, nPixelSpace,
+                              nLineSpace, psExtraArg ) == CE_None )
             return CE_None;
     }
 
     m_poRawRaster->SetAccess(eAccess);
 
     return m_poRawRaster->RasterIO( eRWFlag, nXOff, nYOff, nXSize, nYSize,
-                                  pData, nBufXSize, nBufYSize,
-                                  eBufType, nPixelSpace, nLineSpace, psExtraArg );
+                                    pData, nBufXSize, nBufYSize,
+                                    eBufType, nPixelSpace,
+                                    nLineSpace, psExtraArg );
 }
 
 /************************************************************************/
@@ -122,7 +142,7 @@ CPLErr VRTRawRasterBand::IRasterIO( GDALRWFlag eRWFlag,
 /************************************************************************/
 
 CPLErr VRTRawRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
-                                   void * pImage )
+                                     void * pImage )
 
 {
     if( m_poRawRaster == NULL )
@@ -188,7 +208,9 @@ CPLErr VRTRawRasterBand::SetRawLink( const char *pszFilename,
             CPLProjectRelativeFilename( pszVRTPath, pszFilename ) );
     }
     else
+    {
         pszExpandedFilename = CPLStrdup( pszFilename );
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Try and open the file.  We always use the large file API.       */
@@ -207,9 +229,8 @@ CPLErr VRTRawRasterBand::SetRawLink( const char *pszFilename,
     if( fp == NULL )
     {
         CPLError( CE_Failure, CPLE_OpenFailed,
-                  "Unable to open %s.\n%s",
-                  pszExpandedFilename,
-                  VSIStrerror( errno ) );
+                  "Unable to open %s.%s",
+                  pszExpandedFilename, VSIStrerror( errno ) );
 
         CPLFree( pszExpandedFilename );
         return CE_Failure;
@@ -223,12 +244,12 @@ CPLErr VRTRawRasterBand::SetRawLink( const char *pszFilename,
 /* -------------------------------------------------------------------- */
 /*      Work out if we are in native mode or not.                       */
 /* -------------------------------------------------------------------- */
-    int bNative = TRUE;
+    bool bNative = true;
 
     if( pszByteOrder != NULL )
     {
         if( EQUAL(pszByteOrder,"LSB") )
-            bNative = CPL_IS_LSB;
+            bNative = CPL_TO_BOOL(CPL_IS_LSB);
         else if( EQUAL(pszByteOrder,"MSB") )
             bNative = !CPL_IS_LSB;
         else
@@ -244,8 +265,8 @@ CPLErr VRTRawRasterBand::SetRawLink( const char *pszFilename,
 /*      Create a corresponding RawRasterBand.                           */
 /* -------------------------------------------------------------------- */
     m_poRawRaster = new RawRasterBand( fp, nImageOffset, nPixelOffset,
-                                     nLineOffset, GetRasterDataType(),
-                                     bNative, GetXSize(), GetYSize(), TRUE );
+                                       nLineOffset, GetRasterDataType(),
+                                       bNative, GetXSize(), GetYSize(), TRUE );
 
 /* -------------------------------------------------------------------- */
 /*      Reset block size to match the raw raster.                       */
@@ -267,8 +288,8 @@ void VRTRawRasterBand::ClearRawLink()
         VSILFILE* fp = m_poRawRaster->GetFPL();
         delete m_poRawRaster;
         m_poRawRaster = NULL;
-        /* We close the file after deleting the raster band */
-        /* since data can be flushed in the destructor */
+        // We close the file after deleting the raster band
+        // since data can be flushed in the destructor.
         if( fp != NULL )
         {
             CPLCloseShared( reinterpret_cast<FILE*>( fp ) );
@@ -283,10 +304,10 @@ void VRTRawRasterBand::ClearRawLink()
 /************************************************************************/
 
 CPLErr VRTRawRasterBand::XMLInit( CPLXMLNode * psTree,
-                                      const char *pszVRTPath )
+                                  const char *pszVRTPath )
 
 {
-    CPLErr eErr = VRTRasterBand::XMLInit( psTree, pszVRTPath );
+    const CPLErr eErr = VRTRasterBand::XMLInit( psTree, pszVRTPath );
     if( eErr != CE_None )
         return eErr;
 
@@ -294,8 +315,8 @@ CPLErr VRTRawRasterBand::XMLInit( CPLXMLNode * psTree,
 /*      Validate a bit.                                                 */
 /* -------------------------------------------------------------------- */
     if( psTree == NULL || psTree->eType != CXT_Element
-        || !EQUAL(psTree->pszValue,"VRTRasterBand")
-        || !EQUAL(CPLGetXMLValue(psTree,"subClass",""),"VRTRawRasterBand") )
+        || !EQUAL(psTree->pszValue, "VRTRasterBand")
+        || !EQUAL(CPLGetXMLValue(psTree,"subClass",""), "VRTRawRasterBand") )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "Invalid node passed to VRTRawRasterBand::XMLInit()." );
@@ -315,31 +336,33 @@ CPLErr VRTRawRasterBand::XMLInit( CPLXMLNode * psTree,
         return CE_Failure;
     }
 
-    const int l_bRelativeToVRT
-        = atoi(CPLGetXMLValue( psTree, "SourceFilename.relativeToVRT", "1" ) );
+    // TODO(schwehr): Should this be a bool?
+    const int l_bRelativeToVRT =
+        atoi(CPLGetXMLValue( psTree, "SourceFilename.relativeToVRT", "1" ) );
 
 /* -------------------------------------------------------------------- */
 /*      Collect layout information.                                     */
 /* -------------------------------------------------------------------- */
-    int nPixelOffset;
-    int nWordDataSize = GDALGetDataTypeSize( GetRasterDataType() ) / 8;
+    int nWordDataSize = GDALGetDataTypeSizeBytes( GetRasterDataType() );
 
     const char* pszImageOffset = CPLGetXMLValue( psTree, "ImageOffset", "0");
     const vsi_l_offset nImageOffset = CPLScanUIntBig(
-                        pszImageOffset, static_cast<int>(strlen(pszImageOffset)));
+        pszImageOffset, static_cast<int>(strlen(pszImageOffset)) );
 
-    if( CPLGetXMLValue( psTree, "PixelOffset", NULL ) == NULL )
-        nPixelOffset = nWordDataSize;
-    else
+    int nPixelOffset = nWordDataSize;
+    if( CPLGetXMLValue( psTree, "PixelOffset", NULL ) != NULL )
+    {
         nPixelOffset = atoi(CPLGetXMLValue( psTree, "PixelOffset", "0") );
+    }
     if (nPixelOffset <= 0)
     {
         CPLError( CE_Failure, CPLE_AppDefined,
-                  "Invalid value for <PixelOffset> element : %d", nPixelOffset );
+                  "Invalid value for <PixelOffset> element : %d",
+                  nPixelOffset );
         return CE_Failure;
     }
 
-    int nLineOffset;
+    int nLineOffset = 0;
     if( CPLGetXMLValue( psTree, "LineOffset", NULL ) == NULL )
         nLineOffset = nWordDataSize * GetXSize();
     else
@@ -355,17 +378,6 @@ CPLErr VRTRawRasterBand::XMLInit( CPLXMLNode * psTree,
                        pszByteOrder );
 }
 
-/************************************************************************/
-/*                           VRTRawStripSpace()                         */
-/************************************************************************/
-
-static const char* VRTRawStripSpace(const char* pszStr)
-{
-    while( *pszStr == ' ' )
-        pszStr ++;
-    return pszStr;
-}
-
 /************************************************************************/
 /*                           SerializeToXML()                           */
 /************************************************************************/
@@ -380,7 +392,8 @@ CPLXMLNode *VRTRawRasterBand::SerializeToXML( const char *pszVRTPath )
     if( m_poRawRaster == NULL )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
-                  "VRTRawRasterBand::SerializeToXML() fails because m_poRawRaster is NULL." );
+                  "VRTRawRasterBand::SerializeToXML() fails because "
+                  "m_poRawRaster is NULL." );
         return NULL;
     }
 
@@ -407,19 +420,18 @@ CPLXMLNode *VRTRawRasterBand::SerializeToXML( const char *pszVRTPath )
 /* -------------------------------------------------------------------- */
 /*      Set other layout information.                                   */
 /* -------------------------------------------------------------------- */
-    char szOffset[22];
 
-    CPLPrintUIntBig(szOffset, m_poRawRaster->GetImgOffset(), sizeof(szOffset)-1);
-    szOffset[sizeof(szOffset)-1] = '\0';
-    CPLCreateXMLElementAndValue(psTree, "ImageOffset", VRTRawStripSpace(szOffset));
+    CPLCreateXMLElementAndValue( psTree, "ImageOffset",
+                                 CPLSPrintf( CPL_FRMT_GUIB,
+                                             m_poRawRaster->GetImgOffset()) );
 
-    CPLPrintUIntBig(szOffset, m_poRawRaster->GetPixelOffset(),sizeof(szOffset)-1);
-    szOffset[sizeof(szOffset)-1] = '\0';
-    CPLCreateXMLElementAndValue(psTree, "PixelOffset", VRTRawStripSpace(szOffset));
+    CPLCreateXMLElementAndValue( psTree, "PixelOffset",
+                                 CPLSPrintf( "%d",
+                                             m_poRawRaster->GetPixelOffset()) );
 
-    CPLPrintUIntBig(szOffset, m_poRawRaster->GetLineOffset(), sizeof(szOffset)-1);
-    szOffset[sizeof(szOffset)-1] = '\0';
-    CPLCreateXMLElementAndValue(psTree, "LineOffset", VRTRawStripSpace(szOffset));
+    CPLCreateXMLElementAndValue( psTree, "LineOffset",
+                                 CPLSPrintf( "%d",
+                                             m_poRawRaster->GetLineOffset()) );
 
 #if CPL_IS_LSB == 1
     if( m_poRawRaster->GetNativeOrder() )
@@ -437,8 +449,8 @@ CPLXMLNode *VRTRawRasterBand::SerializeToXML( const char *pszVRTPath )
 /*                             GetFileList()                            */
 /************************************************************************/
 
-void VRTRawRasterBand::GetFileList(char*** ppapszFileList, int *pnSize,
-                                int *pnMaxSize, CPLHashSet* hSetFiles)
+void VRTRawRasterBand::GetFileList( char*** ppapszFileList, int *pnSize,
+                                    int *pnMaxSize, CPLHashSet* hSetFiles )
 {
     if (m_pszSourceFilename == NULL)
         return;
@@ -446,7 +458,14 @@ void VRTRawRasterBand::GetFileList(char*** ppapszFileList, int *pnSize,
 /* -------------------------------------------------------------------- */
 /*      Is it already in the list ?                                     */
 /* -------------------------------------------------------------------- */
-    if( CPLHashSetLookup(hSetFiles, m_pszSourceFilename) != NULL )
+    CPLString osSourceFilename;
+    if( m_bRelativeToVRT && strlen(poDS->GetDescription()) > 0 )
+        osSourceFilename = CPLFormFilename(
+              CPLGetDirname(poDS->GetDescription()), m_pszSourceFilename, NULL );
+    else
+        osSourceFilename = m_pszSourceFilename;
+
+    if( CPLHashSetLookup(hSetFiles, osSourceFilename) != NULL )
         return;
 
 /* -------------------------------------------------------------------- */
@@ -462,12 +481,14 @@ void VRTRawRasterBand::GetFileList(char*** ppapszFileList, int *pnSize,
 /* -------------------------------------------------------------------- */
 /*      Add the string to the list                                      */
 /* -------------------------------------------------------------------- */
-    (*ppapszFileList)[*pnSize] = CPLStrdup(m_pszSourceFilename);
+    (*ppapszFileList)[*pnSize] = CPLStrdup(osSourceFilename);
     (*ppapszFileList)[(*pnSize + 1)] = NULL;
     CPLHashSetInsert(hSetFiles, (*ppapszFileList)[*pnSize]);
 
-    (*pnSize) ++;
+    (*pnSize)++;
 
     VRTRasterBand::GetFileList( ppapszFileList, pnSize,
                                 pnMaxSize, hSetFiles);
 }
+
+/*! @endcond */
