@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id: ograpispy.cpp 33680 2016-03-08 09:59:03Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  OGR C API "Spy"
@@ -28,60 +27,68 @@
  ****************************************************************************/
 
 #include "cpl_port.h"
+#include "ograpispy.h"
 
-#include <stdio.h>
+#include <cstdio>
 #include <map>
 #include <set>
 
-#include "ograpispy.h"
-
+#include "cpl_string.h"
 #include "gdal.h"
-#include "ogr_spatialref.h"
 #include "ogr_geometry.h"
 #include "ogr_feature.h"
+#include "ogr_spatialref.h"
 #include "ogrsf_frmts.h"
-#include "cpl_string.h"
+
+CPL_CVSID("$Id: ograpispy.cpp 37371 2017-02-13 11:41:59Z rouault $");
 
 #ifdef OGRAPISPY_ENABLED
 
 int bOGRAPISpyEnabled = FALSE;
-static CPLString osSnapshotPath, osSpyFile;
+static CPLString osSnapshotPath;
+static CPLString osSpyFile;
 static FILE* fpSpyFile = NULL;
 extern "C" int CPL_DLL GDALIsInGlobalDestructor(void);
 
+namespace
+{
+
 class LayerDescription
 {
-    public:
-        int iLayer;
+  public:
+    int iLayer;
 
-        LayerDescription(): iLayer(-1) {}
-        LayerDescription(int iLayerIn): iLayer(iLayerIn) {}
+    LayerDescription(): iLayer(-1) {}
+    explicit LayerDescription( int iLayerIn ): iLayer(iLayerIn) {}
 };
 
 class DatasetDescription
 {
-    public:
-        int iDS;
-        std::map<OGRLayerH, LayerDescription> oMapLayer;
+  public:
+    int iDS;
+    std::map<OGRLayerH, LayerDescription> oMapLayer;
 
-        DatasetDescription() : iDS(-1) {}
-        DatasetDescription(int iDSIn) : iDS(iDSIn) {}
-        ~DatasetDescription();
+    DatasetDescription() : iDS(-1) {}
+    explicit DatasetDescription( int iDSIn ) : iDS(iDSIn) {}
+    ~DatasetDescription();
 };
 
 class FeatureDefnDescription
 {
-    public:
-        OGRFeatureDefnH hFDefn;
-        int iUniqueNumber;
-        std::map<OGRFieldDefnH, int> oMapFieldDefn;
-        std::map<OGRGeomFieldDefnH, int> oMapGeomFieldDefn;
+  public:
+    OGRFeatureDefnH hFDefn;
+    int iUniqueNumber;
+    std::map<OGRFieldDefnH, int> oMapFieldDefn;
+    std::map<OGRGeomFieldDefnH, int> oMapGeomFieldDefn;
 
-        FeatureDefnDescription(): hFDefn(NULL), iUniqueNumber(-1) {}
-        FeatureDefnDescription(OGRFeatureDefnH hFDefnIn, int iUniqueNumberIn): hFDefn(hFDefnIn), iUniqueNumber(iUniqueNumberIn) {}
-        void Free();
+    FeatureDefnDescription(): hFDefn(NULL), iUniqueNumber(-1) {}
+    FeatureDefnDescription( OGRFeatureDefnH hFDefnIn, int iUniqueNumberIn ):
+        hFDefn(hFDefnIn), iUniqueNumber(iUniqueNumberIn) {}
+    void Free();
 };
 
+}  // namespace
+
 static std::map<OGRDataSourceH, DatasetDescription> oMapDS;
 static std::map<OGRLayerH, CPLString> oGlobalMapLayer;
 static OGRLayerH hLayerGetNextFeature = NULL;
@@ -96,21 +103,24 @@ static std::map<OGRFieldDefnH, CPLString> oGlobalMapFieldDefn;
 void FeatureDefnDescription::Free()
 {
     {
-        std::map<OGRGeomFieldDefnH, int>::iterator oIter = oMapGeomFieldDefn.begin();
-        for(; oIter != oMapGeomFieldDefn.end(); ++oIter)
+        std::map<OGRGeomFieldDefnH, int>::iterator oIter =
+            oMapGeomFieldDefn.begin();
+        for( ; oIter != oMapGeomFieldDefn.end(); ++oIter )
             oGlobalMapGeomFieldDefn.erase(oIter->first);
     }
     {
-        std::map<OGRFieldDefnH, int>::iterator oIter = oMapFieldDefn.begin();
-        for(; oIter != oMapFieldDefn.end(); ++oIter)
+        std::map<OGRFieldDefnH, int>::iterator oIter =
+            oMapFieldDefn.begin();
+        for( ; oIter != oMapFieldDefn.end(); ++oIter )
             oGlobalMapFieldDefn.erase(oIter->first);
     }
 }
 
 DatasetDescription::~DatasetDescription()
 {
-    std::map<OGRLayerH, LayerDescription>::iterator oIter = oMapLayer.begin();
-    for(; oIter != oMapLayer.end(); ++oIter)
+    std::map<OGRLayerH, LayerDescription>::iterator oIter =
+        oMapLayer.begin();
+    for( ; oIter != oMapLayer.end(); ++oIter )
         oGlobalMapLayer.erase(oIter->first);
 }
 
@@ -136,19 +146,20 @@ static void OGRAPISpyFileClose()
 static bool OGRAPISpyEnabled()
 {
     const char* pszSpyFile = CPLGetConfigOption("OGR_API_SPY_FILE", NULL);
-    bOGRAPISpyEnabled = (pszSpyFile != NULL);
+    bOGRAPISpyEnabled = pszSpyFile != NULL;
     if( !bOGRAPISpyEnabled )
     {
         osSpyFile.resize(0);
         aoSetCreatedDS.clear();
         return false;
     }
-    if( osSpyFile.size() )
+    if( !osSpyFile.empty() )
         return true;
 
     osSpyFile = pszSpyFile;
 
-    const char* pszSnapshotPath = CPLGetConfigOption("OGR_API_SPY_SNAPSHOT_PATH", ".");
+    const char* pszSnapshotPath =
+        CPLGetConfigOption("OGR_API_SPY_SNAPSHOT_PATH", ".");
     if( EQUAL(pszSnapshotPath, "NO") )
         osSnapshotPath = "";
     else
@@ -163,43 +174,42 @@ static bool OGRAPISpyEnabled()
     if( fpSpyFile == NULL )
         fpSpyFile = stderr;
 
-    fprintf(fpSpyFile, "# This file is generated by the OGR_API_SPY mechanism.\n");
+    fprintf(fpSpyFile,
+            "# This file is generated by the OGR_API_SPY mechanism.\n");
     fprintf(fpSpyFile, "from osgeo import ogr\n");
     fprintf(fpSpyFile, "from osgeo import osr\n");
     fprintf(fpSpyFile, "import os\n");
     fprintf(fpSpyFile, "import shutil\n");
-    fprintf(fpSpyFile, "os.access\n"); // to make pyflakes happy in case it's unused later
-    fprintf(fpSpyFile, "shutil.copy\n"); // same here
+    // To make pyflakes happy in case it's unused later.
+    fprintf(fpSpyFile, "os.access\n");
+    fprintf(fpSpyFile, "shutil.copy\n");  // Same here.
     fprintf(fpSpyFile, "\n");
 
     return true;
 }
 
-static CPLString OGRAPISpyGetOptions(char** papszOptions)
+static CPLString OGRAPISpyGetOptions( char** papszOptions )
 {
-    CPLString options;
     if( papszOptions == NULL )
     {
-        options = "[]";
+        return "[]";
     }
-    else
+
+    CPLString options = "[";
+    for( char** papszIter = papszOptions; *papszIter != NULL; papszIter++ )
     {
-        options = "[";
-        for(char** papszIter = papszOptions; *papszIter != NULL; papszIter++)
-        {
-            if( papszIter != papszOptions )
-                options += ", ";
-            options += "'";
-            options += *papszIter;
-            options += "'";
-        }
-        options += "]";
+        if( papszIter != papszOptions )
+            options += ", ";
+        options += "'";
+        options += *papszIter;
+        options += "'";
     }
+    options += "]";
 
     return options;
 }
 
-static CPLString OGRAPISpyGetString(const char* pszStr)
+static CPLString OGRAPISpyGetString( const char* pszStr )
 {
     if( pszStr == NULL )
         return "None";
@@ -212,60 +222,63 @@ static CPLString OGRAPISpyGetString(const char* pszStr)
             osRet += "\\\\";
         else
             osRet += *pszStr;
-        pszStr ++;
+        pszStr++;
     }
     osRet += "'";
     return osRet;
 }
 
-static CPLString OGRAPISpyGetDSVar(OGRDataSourceH hDS)
+static CPLString OGRAPISpyGetDSVar( OGRDataSourceH hDS )
 {
     if( hDS && oMapDS.find(hDS) == oMapDS.end() )
     {
-        int i = (int)oMapDS.size() + 1;
+        int i = static_cast<int>(oMapDS.size()) + 1;
         oMapDS[hDS] = DatasetDescription(i);
     }
-    return CPLSPrintf("ds%d", (hDS) ? oMapDS[hDS].iDS : 0);
+    return CPLSPrintf("ds%d", hDS ? oMapDS[hDS].iDS : 0);
 }
 
-static CPLString OGRAPISpyGetLayerVar(OGRLayerH hLayer)
+static CPLString OGRAPISpyGetLayerVar( OGRLayerH hLayer )
 {
     return oGlobalMapLayer[hLayer];
 }
 
-static CPLString OGRAPISpyGetAndRegisterLayerVar(OGRDataSourceH hDS,
-                                                 OGRLayerH hLayer)
+static CPLString OGRAPISpyGetAndRegisterLayerVar( OGRDataSourceH hDS,
+                                                  OGRLayerH hLayer )
 {
     DatasetDescription& dd = oMapDS[hDS];
     if( hLayer && dd.oMapLayer.find(hLayer) == dd.oMapLayer.end() )
     {
-        int i = (int)dd.oMapLayer.size() + 1;
-        dd.oMapLayer[hLayer] = i;
-        oGlobalMapLayer[hLayer] = OGRAPISpyGetDSVar(hDS) + "_" + CPLSPrintf("lyr%d", i);
+        const int i = static_cast<int>(dd.oMapLayer.size()) + 1;
+        dd.oMapLayer[hLayer] = LayerDescription(i);
+        oGlobalMapLayer[hLayer] =
+            OGRAPISpyGetDSVar(hDS) + "_" + CPLSPrintf("lyr%d", i);
     }
+
     return OGRAPISpyGetDSVar(hDS) + "_" +
-           CPLSPrintf("lyr%d", (hLayer) ? dd.oMapLayer[hLayer].iLayer : 0);
+           CPLSPrintf("lyr%d", hLayer ? dd.oMapLayer[hLayer].iLayer : 0);
 }
 
-static CPLString OGRAPISpyGetSRS(OGRSpatialReferenceH hSpatialRef)
+static CPLString OGRAPISpyGetSRS( OGRSpatialReferenceH hSpatialRef )
 {
-    if (hSpatialRef == NULL)
+    if( hSpatialRef == NULL )
         return "None";
 
     char* pszWKT = NULL;
-    ((OGRSpatialReference*)hSpatialRef)->exportToWkt(&pszWKT);
-    const char* pszRet = CPLSPrintf("osr.SpatialReference(\"\"\"%s\"\"\")", pszWKT);
+    reinterpret_cast<OGRSpatialReference*>(hSpatialRef)->exportToWkt(&pszWKT);
+    const char* pszRet =
+        CPLSPrintf("osr.SpatialReference(\"\"\"%s\"\"\")", pszWKT);
     CPLFree(pszWKT);
     return pszRet;
 }
 
-static CPLString OGRAPISpyGetGeom(OGRGeometryH hGeom)
+static CPLString OGRAPISpyGetGeom( OGRGeometryH hGeom )
 {
-    if (hGeom == NULL)
+    if( hGeom == NULL )
         return "None";
 
     char* pszWKT = NULL;
-    ((OGRGeometry*)hGeom)->exportToWkt(&pszWKT);
+    reinterpret_cast<OGRGeometry *>(hGeom)->exportToWkt(&pszWKT);
     const char* pszRet = CPLSPrintf("ogr.CreateGeometryFromWkt('%s')", pszWKT);
     CPLFree(pszWKT);
     return pszRet;
@@ -273,9 +286,9 @@ static CPLString OGRAPISpyGetGeom(OGRGeometryH hGeom)
 
 #define casePrefixOgrDot(x)  case x: return "ogr." #x;
 
-static CPLString OGRAPISpyGetGeomType(OGRwkbGeometryType eType)
+static CPLString OGRAPISpyGetGeomType( OGRwkbGeometryType eType )
 {
-    switch(eType)
+    switch( eType )
     {
         casePrefixOgrDot(wkbUnknown)
         casePrefixOgrDot(wkbPoint)
@@ -355,7 +368,7 @@ static CPLString OGRAPISpyGetGeomType(OGRwkbGeometryType eType)
 
 static CPLString OGRAPISpyGetFieldType(OGRFieldType eType)
 {
-    switch(eType)
+    switch( eType )
     {
         casePrefixOgrDot(OFTInteger)
         casePrefixOgrDot(OFTInteger64)
@@ -375,18 +388,25 @@ static CPLString OGRAPISpyGetFieldType(OGRFieldType eType)
     return "error";
 }
 
-static CPLString OGRAPISpyGetFeatureDefnVar(OGRFeatureDefnH hFDefn)
+#undef casePrefixOgrDot
+
+static CPLString OGRAPISpyGetFeatureDefnVar( OGRFeatureDefnH hFDefn )
 {
-    std::map<OGRFeatureDefnH, FeatureDefnDescription>::iterator oIter = oMapFDefn.find(hFDefn);
-    int i;
+    std::map<OGRFeatureDefnH, FeatureDefnDescription>::iterator oIter =
+        oMapFDefn.find(hFDefn);
+    int i = 0;
     if( oIter == oMapFDefn.end() )
     {
-        i = (int)oMapFDefn.size() + 1;
+        i = static_cast<int>(oMapFDefn.size()) + 1;
         oMapFDefn[hFDefn] = FeatureDefnDescription(hFDefn, i);
-        ((OGRFeatureDefn*)hFDefn)->Reference(); // so that we can check when they are no longer used
+
+        // So that we can check when they are no longer used.
+        reinterpret_cast<OGRFeatureDefn *>(hFDefn)->Reference();
     }
     else
+    {
         i = oIter->second.iUniqueNumber;
+    }
     return CPLSPrintf("fdefn%d", i);
 }
 
@@ -395,7 +415,10 @@ static void OGRAPISpyFlushDefered()
     OGRAPISpyFileReopen();
     if( hLayerGetLayerDefn != NULL )
     {
-        OGRFeatureDefnH hDefn = (OGRFeatureDefnH)(((OGRLayer*)hLayerGetLayerDefn)->GetLayerDefn());
+        OGRFeatureDefnH hDefn =
+            reinterpret_cast<OGRFeatureDefnH>(
+                reinterpret_cast<OGRLayer *>(hLayerGetLayerDefn)->
+                    GetLayerDefn());
         fprintf(fpSpyFile, "%s = %s.GetLayerDefn()\n",
             OGRAPISpyGetFeatureDefnVar(hDefn).c_str(),
             OGRAPISpyGetLayerVar(hLayerGetLayerDefn).c_str());
@@ -427,9 +450,9 @@ static void OGRAPISpyFlushDefered()
     }
 }
 
-int OGRAPISpyOpenTakeSnapshot(const char* pszName, int bUpdate)
+int OGRAPISpyOpenTakeSnapshot( const char* pszName, int bUpdate )
 {
-    if( !OGRAPISpyEnabled() || !bUpdate || osSnapshotPath.size() == 0 ||
+    if( !OGRAPISpyEnabled() || !bUpdate || osSnapshotPath.empty() ||
         aoSetCreatedDS.find(pszName) != aoSetCreatedDS.end() )
         return -1;
     OGRAPISpyFlushDefered();
@@ -437,10 +460,12 @@ int OGRAPISpyOpenTakeSnapshot(const char* pszName, int bUpdate)
     VSIStatBufL sStat;
     if( VSIStatL( pszName, &sStat ) == 0 )
     {
-        GDALDatasetH hDS = GDALOpenEx(pszName, GDAL_OF_VECTOR, NULL, NULL, NULL);
+        GDALDatasetH hDS =
+            GDALOpenEx(pszName, GDAL_OF_VECTOR, NULL, NULL, NULL);
         if( hDS )
         {
-            char** papszFileList = ((GDALDataset*)hDS)->GetFileList();
+            char** papszFileList =
+                reinterpret_cast<GDALDataset *>(hDS)->GetFileList();
             GDALClose(hDS);
             if( papszFileList )
             {
@@ -450,7 +475,8 @@ int OGRAPISpyOpenTakeSnapshot(const char* pszName, int bUpdate)
                 CPLString osWorkingDir;
                 while( true )
                 {
-                    osBaseDir = CPLFormFilename(osSnapshotPath,
+                    osBaseDir =
+                        CPLFormFilename(osSnapshotPath,
                                         CPLSPrintf("snapshot_%d", i), NULL );
                     if( VSIStatL( osBaseDir, &sStat ) != 0 )
                         break;
@@ -463,11 +489,12 @@ int OGRAPISpyOpenTakeSnapshot(const char* pszName, int bUpdate)
                 VSIMkdir( osWorkingDir, 0777 );
                 fprintf(fpSpyFile, "# Take snapshot of %s\n", pszName);
                 fprintf(fpSpyFile, "try:\n");
-                fprintf(fpSpyFile, "    shutil.rmtree('%s')\n", osWorkingDir.c_str());
+                fprintf(fpSpyFile, "    shutil.rmtree('%s')\n",
+                        osWorkingDir.c_str());
                 fprintf(fpSpyFile, "except:\n");
                 fprintf(fpSpyFile, "    pass\n");
                 fprintf(fpSpyFile, "os.mkdir('%s')\n", osWorkingDir.c_str());
-                for(char** papszIter = papszFileList; *papszIter; papszIter++)
+                for( char** papszIter = papszFileList; *papszIter; papszIter++ )
                 {
                     CPLString osSnapshotSrcFile = CPLFormFilename(
                             osSrcDir, CPLGetFilename(*papszIter), NULL);
@@ -487,7 +514,8 @@ int OGRAPISpyOpenTakeSnapshot(const char* pszName, int bUpdate)
     return -1;
 }
 
-void OGRAPISpyOpen(const char* pszName, int bUpdate, int iSnapshot, GDALDatasetH* phDS)
+void OGRAPISpyOpen( const char* pszName, int bUpdate, int iSnapshot,
+                    GDALDatasetH* phDS )
 {
     if( !OGRAPISpyEnabled() ) return;
     OGRAPISpyFlushDefered();
@@ -495,27 +523,33 @@ void OGRAPISpyOpen(const char* pszName, int bUpdate, int iSnapshot, GDALDatasetH
     CPLString osName;
     if( iSnapshot > 0 )
     {
-        CPLString osBaseDir = CPLFormFilename(osSnapshotPath,
-                                   CPLSPrintf("snapshot_%d", iSnapshot), NULL );
+        CPLString osBaseDir =
+            CPLFormFilename(osSnapshotPath,
+                            CPLSPrintf("snapshot_%d", iSnapshot), NULL );
         CPLString osWorkingDir = CPLFormFilename( osBaseDir, "working", NULL );
         osName = CPLFormFilename(osWorkingDir, CPLGetFilename(pszName), NULL);
         pszName = osName.c_str();
 
         if( *phDS != NULL )
         {
-            GDALClose( (GDALDatasetH) *phDS );
-            *phDS = GDALOpenEx(pszName, GDAL_OF_VECTOR | GDAL_OF_UPDATE, NULL, NULL, NULL);
+            GDALClose( reinterpret_cast<GDALDatasetH>(*phDS) );
+            *phDS = GDALOpenEx(pszName,
+                               GDAL_OF_VECTOR | GDAL_OF_UPDATE,
+                               NULL, NULL, NULL);
         }
     }
 
     if( *phDS != NULL )
-        fprintf(fpSpyFile, "%s = ", OGRAPISpyGetDSVar((OGRDataSourceH) *phDS).c_str());
+        fprintf(fpSpyFile,
+                "%s = ",
+                OGRAPISpyGetDSVar(
+                    reinterpret_cast<OGRDataSourceH>(*phDS)).c_str());
     fprintf(fpSpyFile, "ogr.Open(%s, update = %d)\n",
             OGRAPISpyGetString(pszName).c_str(), bUpdate);
     OGRAPISpyFileClose();
 }
 
-void OGRAPISpyPreClose(OGRDataSourceH hDS)
+void OGRAPISpyPreClose( OGRDataSourceH hDS )
 {
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "ds%d = None\n", oMapDS[hDS].iDS);
@@ -528,19 +562,21 @@ void OGRAPISpyPostClose()
     if( !GDALIsInGlobalDestructor() )
     {
         std::map<OGRFeatureDefnH, FeatureDefnDescription>::iterator oIter =
-                                                                oMapFDefn.begin();
+            oMapFDefn.begin();
         std::vector<OGRFeatureDefnH> oArray;
-        for(; oIter != oMapFDefn.end(); ++oIter)
+        for( ; oIter != oMapFDefn.end(); ++oIter )
         {
             FeatureDefnDescription& featureDefnDescription = oIter->second;
-            if( ((OGRFeatureDefn*)featureDefnDescription.hFDefn)->GetReferenceCount() == 1 )
+            if( ((OGRFeatureDefn*)featureDefnDescription.hFDefn)->
+                GetReferenceCount() == 1 )
             {
                 oArray.push_back(featureDefnDescription.hFDefn);
             }
         }
-        for(size_t i = 0; i < oArray.size(); i++)
+        for( size_t i = 0; i < oArray.size(); i++ )
         {
-            FeatureDefnDescription& featureDefnDescription = oMapFDefn[oArray[i]];
+            FeatureDefnDescription& featureDefnDescription =
+                oMapFDefn[oArray[i]];
             ((OGRFeatureDefn*)featureDefnDescription.hFDefn)->Release();
             featureDefnDescription.Free();
             oMapFDefn.erase(oArray[i]);
@@ -548,15 +584,16 @@ void OGRAPISpyPostClose()
     }
 }
 
-void OGRAPISpyCreateDataSource(OGRSFDriverH hDriver, const char* pszName,
-                               char** papszOptions, OGRDataSourceH hDS)
+void OGRAPISpyCreateDataSource( OGRSFDriverH hDriver, const char* pszName,
+                                char** papszOptions, OGRDataSourceH hDS )
 {
     if( !OGRAPISpyEnabled() ) return;
     OGRAPISpyFlushDefered();
     if( hDS != NULL )
         fprintf(fpSpyFile, "%s = ", OGRAPISpyGetDSVar(hDS).c_str());
-    fprintf(fpSpyFile, "ogr.GetDriverByName('%s').CreateDataSource(%s, options = %s)\n",
-            GDALGetDriverShortName((GDALDriverH)hDriver),
+    fprintf(fpSpyFile,
+            "ogr.GetDriverByName('%s').CreateDataSource(%s, options = %s)\n",
+            GDALGetDriverShortName(reinterpret_cast<GDALDriverH>(hDriver)),
             OGRAPISpyGetString(pszName).c_str(),
             OGRAPISpyGetOptions(papszOptions).c_str());
     if( hDS != NULL )
@@ -566,12 +603,12 @@ void OGRAPISpyCreateDataSource(OGRSFDriverH hDriver, const char* pszName,
     OGRAPISpyFileClose();
 }
 
-void OGRAPISpyDeleteDataSource(OGRSFDriverH hDriver, const char* pszName)
+void OGRAPISpyDeleteDataSource( OGRSFDriverH hDriver, const char* pszName )
 {
     if( !OGRAPISpyEnabled() ) return;
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "ogr.GetDriverByName('%s').DeleteDataSource(%s)\n",
-            GDALGetDriverShortName((GDALDriverH)hDriver),
+            GDALGetDriverShortName(reinterpret_cast<GDALDriverH>(hDriver)),
             OGRAPISpyGetString(pszName).c_str());
     aoSetCreatedDS.erase(pszName);
     OGRAPISpyFileClose();
@@ -613,7 +650,7 @@ void OGRAPISpy_DS_ExecuteSQL( OGRDataSourceH hDS,
                               const char *pszStatement,
                               OGRGeometryH hSpatialFilter,
                               const char *pszDialect,
-                              OGRLayerH hLayer)
+                              OGRLayerH hLayer )
 {
     OGRAPISpyFlushDefered();
     if( hLayer != NULL )
@@ -627,7 +664,7 @@ void OGRAPISpy_DS_ExecuteSQL( OGRDataSourceH hDS,
     OGRAPISpyFileClose();
 }
 
-void OGRAPISpy_DS_ReleaseResultSet( OGRDataSourceH hDS, OGRLayerH hLayer)
+void OGRAPISpy_DS_ReleaseResultSet( OGRDataSourceH hDS, OGRLayerH hLayer )
 {
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "%s.ReleaseResultSet(%s)\n",
@@ -646,13 +683,14 @@ void OGRAPISpy_DS_CreateLayer( OGRDataSourceH hDS,
                                OGRSpatialReferenceH hSpatialRef,
                                OGRwkbGeometryType eType,
                                char ** papszOptions,
-                               OGRLayerH hLayer)
+                               OGRLayerH hLayer )
 {
     OGRAPISpyFlushDefered();
     if( hLayer != NULL )
         fprintf(fpSpyFile, "%s = ",
             OGRAPISpyGetAndRegisterLayerVar(hDS, hLayer).c_str());
-    fprintf(fpSpyFile, "%s.CreateLayer(%s, srs = %s, geom_type = %s, options = %s)\n",
+    fprintf(fpSpyFile,
+            "%s.CreateLayer(%s, srs = %s, geom_type = %s, options = %s)\n",
             OGRAPISpyGetDSVar(hDS).c_str(),
             OGRAPISpyGetString(pszName).c_str(),
             OGRAPISpyGetSRS(hSpatialRef).c_str(),
@@ -666,7 +704,7 @@ void OGRAPISpy_DS_DeleteLayer( OGRDataSourceH hDS, int iLayer )
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "%s.DeleteLayer(%d)\n",
             OGRAPISpyGetDSVar(hDS).c_str(), iLayer);
-    // Should perhaps remove from the maps
+    // Should perhaps remove from the maps.
     OGRAPISpyFileClose();
 }
 
@@ -674,7 +712,8 @@ void OGRAPISpy_Dataset_StartTransaction( GDALDatasetH hDS, int bForce )
 {
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "%s.StartTransaction(%d)\n",
-            OGRAPISpyGetDSVar((OGRDataSourceH)hDS).c_str(), bForce);
+            OGRAPISpyGetDSVar(reinterpret_cast<OGRDataSourceH>(hDS)).c_str(),
+                              bForce);
     OGRAPISpyFileClose();
 }
 
@@ -682,7 +721,7 @@ void OGRAPISpy_Dataset_CommitTransaction( GDALDatasetH hDS )
 {
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "%s.CommitTransaction()\n",
-            OGRAPISpyGetDSVar((OGRDataSourceH)hDS).c_str());
+            OGRAPISpyGetDSVar(reinterpret_cast<OGRDataSourceH>(hDS)).c_str());
     OGRAPISpyFileClose();
 }
 
@@ -690,7 +729,7 @@ void OGRAPISpy_Dataset_RollbackTransaction( GDALDatasetH hDS )
 {
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "%s.RollbackTransaction()\n",
-            OGRAPISpyGetDSVar((OGRDataSourceH)hDS).c_str());
+            OGRAPISpyGetDSVar(reinterpret_cast<OGRDataSourceH>(hDS)).c_str());
     OGRAPISpyFileClose();
 }
 
@@ -756,37 +795,53 @@ void OGRAPISpy_L_GetNextFeature( OGRLayerH hLayer )
 
 static void OGRAPISpyDumpFeature( OGRFeatureH hFeat )
 {
-    OGRFeature* poFeature = (OGRFeature*) hFeat;
+    OGRFeature* poFeature = reinterpret_cast<OGRFeature *>(hFeat);
 
     fprintf(fpSpyFile, "f = ogr.Feature(%s)\n",
-            OGRAPISpyGetFeatureDefnVar((OGRFeatureDefnH)(poFeature->GetDefnRef())).c_str());
+            OGRAPISpyGetFeatureDefnVar(
+               reinterpret_cast<OGRFeatureDefnH>(poFeature->
+                   GetDefnRef())).c_str());
     if( poFeature->GetFID() != -1 )
         fprintf(fpSpyFile, "f.SetFID(" CPL_FRMT_GIB ")\n", poFeature->GetFID());
-    int i;
-    for(i = 0; i < poFeature->GetFieldCount(); i++)
+    for( int i = 0; i < poFeature->GetFieldCount(); i++ )
     {
-        if( poFeature->IsFieldSet(i) )
+        if( poFeature->IsFieldNull(i) )
+        {
+            fprintf(fpSpyFile, "f.SetFieldNull(%d)\n", i);
+        }
+        else if( poFeature->IsFieldSet(i) )
         {
             switch( poFeature->GetFieldDefnRef(i)->GetType())
             {
-                case OFTInteger: fprintf(fpSpyFile, "f.SetField(%d, %d)\n", i,
-                    poFeature->GetFieldAsInteger(i)); break;
-                case OFTReal: fprintf(fpSpyFile, "%s", CPLSPrintf("f.SetField(%d, %.16g)\n", i,
-                    poFeature->GetFieldAsDouble(i))); break;
-                case OFTString: fprintf(fpSpyFile, "f.SetField(%d, %s)\n", i,
-                    OGRAPISpyGetString(poFeature->GetFieldAsString(i)).c_str()); break;
-                default: fprintf(fpSpyFile, "f.SetField(%d, %s) #FIXME\n", i,
-                    OGRAPISpyGetString(poFeature->GetFieldAsString(i)).c_str()); break;
+                case OFTInteger:
+                    fprintf(fpSpyFile, "f.SetField(%d, %d)\n", i,
+                    poFeature->GetFieldAsInteger(i));
+                    break;
+                case OFTReal:
+                    fprintf(fpSpyFile,
+                            "%s", CPLSPrintf("f.SetField(%d, %.16g)\n", i,
+                    poFeature->GetFieldAsDouble(i)));
+                    break;
+                case OFTString:
+                    fprintf(fpSpyFile, "f.SetField(%d, %s)\n", i,
+                    OGRAPISpyGetString(poFeature->GetFieldAsString(i)).c_str());
+                    break;
+                default:
+                    fprintf(fpSpyFile, "f.SetField(%d, %s) #FIXME\n", i,
+                    OGRAPISpyGetString(poFeature->GetFieldAsString(i)).c_str());
+                    break;
             }
         }
     }
-    for(i = 0; i < poFeature->GetGeomFieldCount(); i++)
+    for( int i = 0; i < poFeature->GetGeomFieldCount(); i++ )
     {
         OGRGeometry* poGeom = poFeature->GetGeomFieldRef(i);
         if( poGeom != NULL )
         {
-            fprintf(fpSpyFile, "f.SetGeomField(%d, %s)\n", i, OGRAPISpyGetGeom(
-                (OGRGeometryH)poGeom ).c_str() );
+            fprintf(fpSpyFile, "f.SetGeomField(%d, %s)\n",
+                    i,
+                    OGRAPISpyGetGeom(
+                       reinterpret_cast<OGRGeometryH>(poGeom)).c_str());
         }
     }
     const char* pszStyleString = poFeature->GetStyleString();
@@ -801,7 +856,8 @@ void OGRAPISpy_L_SetFeature( OGRLayerH hLayer, OGRFeatureH hFeat )
     OGRAPISpyDumpFeature(hFeat);
     fprintf(fpSpyFile, "%s.SetFeature(f)\n",
             OGRAPISpyGetLayerVar(hLayer).c_str());
-    fprintf(fpSpyFile, "f = None\n"); /* in case layer defn is changed afterwards */
+    // In case layer defn is changed afterwards.
+    fprintf(fpSpyFile, "f = None\n");
     OGRAPISpyFileClose();
 }
 
@@ -811,7 +867,8 @@ void OGRAPISpy_L_CreateFeature( OGRLayerH hLayer, OGRFeatureH hFeat )
     OGRAPISpyDumpFeature(hFeat);
     fprintf(fpSpyFile, "%s.CreateFeature(f)\n",
             OGRAPISpyGetLayerVar(hLayer).c_str());
-    fprintf(fpSpyFile, "f = None\n"); /* in case layer defn is changed afterwards */
+    // In case layer defn is changed afterwards.
+    fprintf(fpSpyFile, "f = None\n");
     OGRAPISpyFileClose();
 }
 
@@ -823,7 +880,8 @@ static void OGRAPISpyDumpFieldDefn( OGRFieldDefn* poFieldDefn )
     if( poFieldDefn->GetWidth() > 0 )
         fprintf(fpSpyFile, "fd.SetWidth(%d)\n", poFieldDefn->GetWidth() );
     if( poFieldDefn->GetPrecision() > 0 )
-        fprintf(fpSpyFile, "fd.SetPrecision(%d)\n", poFieldDefn->GetPrecision() );
+        fprintf(fpSpyFile, "fd.SetPrecision(%d)\n",
+                poFieldDefn->GetPrecision() );
     if( !poFieldDefn->IsNullable() )
         fprintf(fpSpyFile, "fd.SetNullable(0)\n");
     if( poFieldDefn->GetDefault() != NULL )
@@ -853,7 +911,7 @@ void OGRAPISpy_L_DeleteField( OGRLayerH hLayer, int iField )
 void OGRAPISpy_L_ReorderFields( OGRLayerH hLayer, int* panMap )
 {
     OGRAPISpyFlushDefered();
-    OGRLayer* poLayer = (OGRLayer*) hLayer;
+    OGRLayer* poLayer = reinterpret_cast<OGRLayer *>(hLayer);
     fprintf(fpSpyFile, "%s.ReorderFields([",
             OGRAPISpyGetLayerVar(hLayer).c_str());
     for( int i = 0; i < poLayer->GetLayerDefn()->GetFieldCount(); i++ )
@@ -889,13 +947,18 @@ void OGRAPISpy_L_CreateGeomField( OGRLayerH hLayer, OGRGeomFieldDefnH hField,
                                   int bApproxOK )
 {
     OGRAPISpyFlushDefered();
-    OGRGeomFieldDefn* poGeomFieldDefn = (OGRGeomFieldDefn*) hField;
+    OGRGeomFieldDefn* poGeomFieldDefn =
+        reinterpret_cast<OGRGeomFieldDefn *>(hField);
+
     fprintf(fpSpyFile, "geom_fd = ogr.GeomFieldDefn(%s, %s)\n",
             OGRAPISpyGetString(poGeomFieldDefn->GetNameRef()).c_str(),
             OGRAPISpyGetGeomType(poGeomFieldDefn->GetType()).c_str());
     if( poGeomFieldDefn->GetSpatialRef() != NULL )
-        fprintf(fpSpyFile, "geom_fd.SetSpatialRef(%s)\n", OGRAPISpyGetSRS(
-            (OGRSpatialReferenceH)poGeomFieldDefn->GetSpatialRef()).c_str() );
+        fprintf(
+            fpSpyFile, "geom_fd.SetSpatialRef(%s)\n",
+            OGRAPISpyGetSRS(
+                reinterpret_cast<OGRSpatialReferenceH>
+                    (poGeomFieldDefn->GetSpatialRef())).c_str() );
     if( !poGeomFieldDefn->IsNullable() )
         fprintf(fpSpyFile, "geom_fd.SetNullable(0)\n");
     fprintf(fpSpyFile, "%s.CreateGeomField(geom_fd, approx_ok = %d)\n",
@@ -906,13 +969,17 @@ void OGRAPISpy_L_CreateGeomField( OGRLayerH hLayer, OGRGeomFieldDefnH hField,
 static void OGRAPISpy_L_Op( OGRLayerH hLayer, const char* pszMethod )
 {
     OGRAPISpyFlushDefered();
-    fprintf(fpSpyFile, "%s.%s()\n", OGRAPISpyGetLayerVar(hLayer).c_str(), pszMethod);
+    fprintf(fpSpyFile, "%s.%s()\n",
+            OGRAPISpyGetLayerVar(hLayer).c_str(), pszMethod);
     OGRAPISpyFileClose();
 }
 
-void OGRAPISpy_L_StartTransaction( OGRLayerH hLayer ) { OGRAPISpy_L_Op(hLayer, "StartTransaction"); }
-void OGRAPISpy_L_CommitTransaction( OGRLayerH hLayer ) { OGRAPISpy_L_Op(hLayer, "CommitTransaction"); }
-void OGRAPISpy_L_RollbackTransaction( OGRLayerH hLayer ) { OGRAPISpy_L_Op(hLayer, "RollbackTransaction"); }
+void OGRAPISpy_L_StartTransaction( OGRLayerH hLayer )
+    { OGRAPISpy_L_Op(hLayer, "StartTransaction"); }
+void OGRAPISpy_L_CommitTransaction( OGRLayerH hLayer )
+    { OGRAPISpy_L_Op(hLayer, "CommitTransaction"); }
+void OGRAPISpy_L_RollbackTransaction( OGRLayerH hLayer )
+    { OGRAPISpy_L_Op(hLayer, "RollbackTransaction"); }
 
 void OGRAPISpy_L_GetLayerDefn( OGRLayerH hLayer )
 {
@@ -924,14 +991,22 @@ void OGRAPISpy_L_GetLayerDefn( OGRLayerH hLayer )
     }
 }
 
-void OGRAPISpy_L_GetSpatialRef( OGRLayerH hLayer ) { OGRAPISpy_L_Op(hLayer, "GetSpatialRef"); }
-void OGRAPISpy_L_GetSpatialFilter( OGRLayerH hLayer ) { OGRAPISpy_L_Op(hLayer, "GetSpatialFilter"); }
-void OGRAPISpy_L_ResetReading( OGRLayerH hLayer ) { OGRAPISpy_L_Op(hLayer, "ResetReading"); }
-void OGRAPISpy_L_SyncToDisk( OGRLayerH hLayer ) { OGRAPISpy_L_Op(hLayer, "SyncToDisk"); }
-void OGRAPISpy_L_GetFIDColumn( OGRLayerH hLayer ) { OGRAPISpy_L_Op(hLayer, "GetFIDColumn"); }
-void OGRAPISpy_L_GetGeometryColumn( OGRLayerH hLayer ) { OGRAPISpy_L_Op(hLayer, "GetGeometryColumn"); }
-void OGRAPISpy_L_GetName( OGRLayerH hLayer ) { OGRAPISpy_L_Op(hLayer, "GetName"); }
-void OGRAPISpy_L_GetGeomType( OGRLayerH hLayer ) { OGRAPISpy_L_Op(hLayer, "GetGeomType"); }
+void OGRAPISpy_L_GetSpatialRef( OGRLayerH hLayer )
+    { OGRAPISpy_L_Op(hLayer, "GetSpatialRef"); }
+void OGRAPISpy_L_GetSpatialFilter( OGRLayerH hLayer )
+    { OGRAPISpy_L_Op(hLayer, "GetSpatialFilter"); }
+void OGRAPISpy_L_ResetReading( OGRLayerH hLayer )
+    { OGRAPISpy_L_Op(hLayer, "ResetReading"); }
+void OGRAPISpy_L_SyncToDisk( OGRLayerH hLayer )
+    { OGRAPISpy_L_Op(hLayer, "SyncToDisk"); }
+void OGRAPISpy_L_GetFIDColumn( OGRLayerH hLayer )
+    { OGRAPISpy_L_Op(hLayer, "GetFIDColumn"); }
+void OGRAPISpy_L_GetGeometryColumn( OGRLayerH hLayer )
+    { OGRAPISpy_L_Op(hLayer, "GetGeometryColumn"); }
+void OGRAPISpy_L_GetName( OGRLayerH hLayer )
+    { OGRAPISpy_L_Op(hLayer, "GetName"); }
+void OGRAPISpy_L_GetGeomType( OGRLayerH hLayer )
+    { OGRAPISpy_L_Op(hLayer, "GetGeomType"); }
 
 void OGRAPISpy_L_FindFieldIndex( OGRLayerH hLayer, const char *pszFieldName,
                                  int bExactMatch )
@@ -974,10 +1049,11 @@ void OGRAPISpy_L_SetSpatialFilterEx( OGRLayerH hLayer, int iGeomField,
 
 void OGRAPISpy_L_SetSpatialFilterRect( OGRLayerH hLayer,
                                        double dfMinX, double dfMinY,
-                                       double dfMaxX, double dfMaxY)
+                                       double dfMaxX, double dfMaxY )
 {
     OGRAPISpyFlushDefered();
-    fprintf(fpSpyFile, "%s", CPLSPrintf("%s.SetSpatialFilterRect(%.16g, %.16g, %.16g, %.16g)\n",
+    fprintf(fpSpyFile, "%s",
+            CPLSPrintf("%s.SetSpatialFilterRect(%.16g, %.16g, %.16g, %.16g)\n",
             OGRAPISpyGetLayerVar(hLayer).c_str(),
             dfMinX, dfMinY, dfMaxX, dfMaxY));
     OGRAPISpyFileClose();
@@ -985,7 +1061,7 @@ void OGRAPISpy_L_SetSpatialFilterRect( OGRLayerH hLayer,
 
 void OGRAPISpy_L_SetSpatialFilterRectEx( OGRLayerH hLayer, int iGeomField,
                                          double dfMinX, double dfMinY,
-                                         double dfMaxX, double dfMaxY)
+                                         double dfMaxX, double dfMaxY )
 {
 
     OGRAPISpyFlushDefered();
@@ -1011,11 +1087,12 @@ void OGRAPISpy_L_SetIgnoredFields( OGRLayerH hLayer,
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "%s.SetIgnoredFields(%s)\n",
             OGRAPISpyGetLayerVar(hLayer).c_str(),
-            OGRAPISpyGetOptions((char**)papszIgnoredFields).c_str());
+            OGRAPISpyGetOptions(
+                const_cast<char **>(papszIgnoredFields)).c_str());
     OGRAPISpyFileClose();
 }
 
-void OGRAPISpy_FD_GetGeomType(OGRFeatureDefnH hDefn)
+void OGRAPISpy_FD_GetGeomType( OGRFeatureDefnH hDefn )
 {
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "%s.GetGeomType()\n",
@@ -1023,10 +1100,13 @@ void OGRAPISpy_FD_GetGeomType(OGRFeatureDefnH hDefn)
     OGRAPISpyFileClose();
 }
 
-void OGRAPISpy_FD_GetFieldCount(OGRFeatureDefnH hDefn)
+void OGRAPISpy_FD_GetFieldCount( OGRFeatureDefnH hDefn )
 {
     if( hLayerGetLayerDefn != NULL &&
-        (OGRFeatureDefnH)(((OGRLayer*)hLayerGetLayerDefn)->GetLayerDefn()) == hDefn )
+        reinterpret_cast<OGRFeatureDefnH>(
+            reinterpret_cast<OGRLayer *>(
+                hLayerGetLayerDefn)->GetLayerDefn()) ==
+        hDefn )
     {
         bDeferGetFieldCount = true;
     }
@@ -1039,8 +1119,8 @@ void OGRAPISpy_FD_GetFieldCount(OGRFeatureDefnH hDefn)
     }
 }
 
-void OGRAPISpy_FD_GetFieldDefn(OGRFeatureDefnH hDefn, int iField,
-                               OGRFieldDefnH hField)
+void OGRAPISpy_FD_GetFieldDefn( OGRFeatureDefnH hDefn, int iField,
+                                OGRFieldDefnH hField )
 {
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "%s_fielddefn%d = %s.GetFieldDefn(%d)\n",
@@ -1054,15 +1134,17 @@ void OGRAPISpy_FD_GetFieldDefn(OGRFeatureDefnH hDefn, int iField,
     if( oIter == oGlobalMapFieldDefn.end() )
     {
         oMapFDefn[hDefn].oMapFieldDefn[hField] = iField;
-        oGlobalMapFieldDefn[hField] = CPLSPrintf("%s_fielddefn%d",
-                                                     OGRAPISpyGetFeatureDefnVar(hDefn).c_str(),
-                                                     iField);
+        oGlobalMapFieldDefn[hField] =
+            CPLSPrintf("%s_fielddefn%d",
+                       OGRAPISpyGetFeatureDefnVar(hDefn).c_str(),
+                       iField);
     }
 
     OGRAPISpyFileClose();
 }
 
-void OGRAPISpy_FD_GetFieldIndex(OGRFeatureDefnH hDefn, const char* pszFieldName)
+void OGRAPISpy_FD_GetFieldIndex( OGRFeatureDefnH hDefn,
+                                 const char* pszFieldName )
 {
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "%s.GetFieldIndex(%s)\n",
@@ -1071,7 +1153,7 @@ void OGRAPISpy_FD_GetFieldIndex(OGRFeatureDefnH hDefn, const char* pszFieldName)
     OGRAPISpyFileClose();
 }
 
-void OGRAPISpy_Fld_GetXXXX(OGRFieldDefnH hField, const char* pszOp)
+void OGRAPISpy_Fld_GetXXXX( OGRFieldDefnH hField, const char* pszOp )
 {
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "%s.%s()\n",
@@ -1079,7 +1161,7 @@ void OGRAPISpy_Fld_GetXXXX(OGRFieldDefnH hField, const char* pszOp)
     OGRAPISpyFileClose();
 }
 
-void OGRAPISpy_FD_GetGeomFieldCount(OGRFeatureDefnH hDefn)
+void OGRAPISpy_FD_GetGeomFieldCount( OGRFeatureDefnH hDefn )
 {
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "%s.GetGeomFieldCount()\n",
@@ -1087,8 +1169,8 @@ void OGRAPISpy_FD_GetGeomFieldCount(OGRFeatureDefnH hDefn)
     OGRAPISpyFileClose();
 }
 
-void OGRAPISpy_FD_GetGeomFieldDefn(OGRFeatureDefnH hDefn, int iGeomField,
-                                   OGRGeomFieldDefnH hGeomField)
+void OGRAPISpy_FD_GetGeomFieldDefn( OGRFeatureDefnH hDefn, int iGeomField,
+                                    OGRGeomFieldDefnH hGeomField )
 {
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "%s_geomfielddefn%d = %s.GetGeomFieldDefn(%d)\n",
@@ -1102,15 +1184,17 @@ void OGRAPISpy_FD_GetGeomFieldDefn(OGRFeatureDefnH hDefn, int iGeomField,
     if( oIter == oGlobalMapGeomFieldDefn.end() )
     {
         oMapFDefn[hDefn].oMapGeomFieldDefn[hGeomField] = iGeomField;
-        oGlobalMapGeomFieldDefn[hGeomField] = CPLSPrintf("%s_geomfielddefn%d",
-                                                     OGRAPISpyGetFeatureDefnVar(hDefn).c_str(),
-                                                     iGeomField);
+        oGlobalMapGeomFieldDefn[hGeomField] =
+            CPLSPrintf("%s_geomfielddefn%d",
+                       OGRAPISpyGetFeatureDefnVar(hDefn).c_str(),
+                       iGeomField);
     }
 
     OGRAPISpyFileClose();
 }
 
-void OGRAPISpy_FD_GetGeomFieldIndex(OGRFeatureDefnH hDefn, const char* pszFieldName)
+void OGRAPISpy_FD_GetGeomFieldIndex( OGRFeatureDefnH hDefn,
+                                     const char* pszFieldName )
 {
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "%s.GetGeomFieldIndex(%s)\n",
@@ -1119,7 +1203,7 @@ void OGRAPISpy_FD_GetGeomFieldIndex(OGRFeatureDefnH hDefn, const char* pszFieldN
     OGRAPISpyFileClose();
 }
 
-void OGRAPISpy_GFld_GetXXXX(OGRGeomFieldDefnH hGeomField, const char* pszOp)
+void OGRAPISpy_GFld_GetXXXX( OGRGeomFieldDefnH hGeomField, const char* pszOp )
 {
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "%s.%s()\n",
@@ -1127,4 +1211,4 @@ void OGRAPISpy_GFld_GetXXXX(OGRGeomFieldDefnH hGeomField, const char* pszOp)
     OGRAPISpyFileClose();
 }
 
-#endif /* OGRAPISPY_ENABLED */
+#endif  // OGRAPISPY_ENABLED
