@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id: ogrcurve.cpp 33631 2016-03-04 06:28:09Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  The OGRCurve geometry class.
@@ -30,23 +29,21 @@
 #include "ogr_geometry.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogrcurve.cpp 33631 2016-03-04 06:28:09Z goatbar $");
+CPL_CVSID("$Id: ogrcurve.cpp 36883 2016-12-15 13:31:12Z rouault $");
+
+//! @cond Doxygen_Suppress
 
 /************************************************************************/
 /*                                OGRCurve()                            */
 /************************************************************************/
 
-OGRCurve::OGRCurve()
-{
-}
+OGRCurve::OGRCurve() {}
 
 /************************************************************************/
 /*                               ~OGRCurve()                            */
 /************************************************************************/
 
-OGRCurve::~OGRCurve()
-{
-}
+OGRCurve::~OGRCurve() {}
 
 /************************************************************************/
 /*                       OGRCurve( const OGRCurve& )                    */
@@ -54,8 +51,7 @@ OGRCurve::~OGRCurve()
 
 OGRCurve::OGRCurve( const OGRCurve& other ) :
     OGRGeometry( other )
-{
-}
+{}
 
 /************************************************************************/
 /*                       operator=( const OGRCurve& )                   */
@@ -69,6 +65,7 @@ OGRCurve& OGRCurve::operator=( const OGRCurve& other )
     }
     return *this;
 }
+//! @endcond
 
 /************************************************************************/
 /*                            getDimension()                            */
@@ -98,20 +95,72 @@ int OGRCurve::getDimension() const
 int OGRCurve::get_IsClosed() const
 
 {
-    OGRPoint            oStartPoint, oEndPoint;
-
+    OGRPoint oStartPoint;
     StartPoint( &oStartPoint );
+
+    OGRPoint oEndPoint;
     EndPoint( &oEndPoint );
 
-    if( oStartPoint.getX() == oEndPoint.getX()
-        && oStartPoint.getY() == oEndPoint.getY() )
+    if (oStartPoint.IsMeasured() && oEndPoint.IsMeasured())
     {
-        return TRUE;
+        // XYZM type
+        if (oStartPoint.Is3D() && oEndPoint.Is3D())
+        {
+            if( oStartPoint.getX() == oEndPoint.getX() && oStartPoint.getY() == oEndPoint.getY()
+                && oStartPoint.getZ() == oEndPoint.getZ() && oStartPoint.getM() == oEndPoint.getM())
+            {
+                return TRUE;
+            }
+            else
+                return FALSE;
+        }
+        // XYM type
+        else
+        {
+            if( oStartPoint.getX() == oEndPoint.getX() && oStartPoint.getY() == oEndPoint.getY()
+                && oStartPoint.getM() == oEndPoint.getM())
+            {
+                return TRUE;
+            }
+            else
+                return FALSE;
+        }
     }
-    else
+
+    else if (oStartPoint.Is3D() && oEndPoint.Is3D())
+    {
+        // XYZ type
+        if( oStartPoint.getX() == oEndPoint.getX() && oStartPoint.getY() == oEndPoint.getY()
+            && oStartPoint.getZ() == oEndPoint.getZ())
+        {
+            return TRUE;
+        }
+        else
+            return FALSE;
+    }
+
+    // one of the points is 3D
+    else if (((oStartPoint.Is3D() & oEndPoint.Is3D()) == 0) &&
+             ((oStartPoint.Is3D() | oEndPoint.Is3D()) == 1))
     {
         return FALSE;
     }
+
+    // one of the points is Measured
+    else if (((oStartPoint.IsMeasured() & oEndPoint.IsMeasured()) == 0) &&
+             ((oStartPoint.IsMeasured() | oEndPoint.IsMeasured()) == 1))
+    {
+        return FALSE;
+    }
+
+    else
+    {
+        // XY type
+        if( oStartPoint.getX() == oEndPoint.getX() && oStartPoint.getY() == oEndPoint.getY() )
+            return TRUE;
+        else
+            return FALSE;
+    }
 }
 
 /**
@@ -161,7 +210,8 @@ int OGRCurve::get_IsClosed() const
  */
 
 /**
- * \fn OGRLineString* OGRCurve::CurveToLine( double dfMaxAngleStepSizeDegrees, const char* const* papszOptions ) const;
+ * \fn OGRLineString* OGRCurve::CurveToLine( double dfMaxAngleStepSizeDegrees,
+ *     const char* const* papszOptions ) const;
  *
  * \brief Return a linestring from a curve geometry.
  *
@@ -178,7 +228,8 @@ int OGRCurve::get_IsClosed() const
  * @param dfMaxAngleStepSizeDegrees the largest step in degrees along the
  * arc, zero to use the default setting.
  * @param papszOptions options as a null-terminated list of strings or NULL.
- *                     See OGRGeometryFactory::curveToLineString() for valid options.
+ *                     See OGRGeometryFactory::curveToLineString() for valid
+ *                     options.
  *
  * @return a line string approximating the curve
  *
@@ -257,19 +308,22 @@ int OGRCurve::get_IsClosed() const
 
 OGRBoolean OGRCurve::IsConvex() const
 {
-    OGRBoolean bRet = TRUE;
+    bool bRet = true;
     OGRPointIterator* poPointIter = getPointIterator();
-    OGRPoint p1, p2, p3;
+    OGRPoint p1;
+    OGRPoint p2;
     if( poPointIter->getNextPoint(&p1) &&
         poPointIter->getNextPoint(&p2) )
     {
+        OGRPoint p3;
         while( poPointIter->getNextPoint(&p3) )
         {
-            double crossproduct = (p2.getX() - p1.getX()) * (p3.getY() - p2.getY()) -
-                                  (p2.getY() - p1.getY()) * (p3.getX() - p2.getX());
+            const double crossproduct =
+                (p2.getX() - p1.getX()) * (p3.getY() - p2.getY()) -
+                (p2.getY() - p1.getY()) * (p3.getX() - p2.getX());
             if( crossproduct > 0 )
             {
-                bRet = FALSE;
+                bRet = false;
                 break;
             }
             p1.setX(p2.getX());
@@ -298,7 +352,7 @@ OGRBoolean OGRCurve::IsConvex() const
  * @since GDAL 2.0
  */
 
-OGRCompoundCurve* OGRCurve::CastToCompoundCurve(OGRCurve* poCurve)
+OGRCompoundCurve* OGRCurve::CastToCompoundCurve( OGRCurve* poCurve )
 {
     OGRCompoundCurve* poCC = new OGRCompoundCurve();
     if( poCurve->getGeometryType() == wkbLineString )
@@ -329,7 +383,7 @@ OGRCompoundCurve* OGRCurve::CastToCompoundCurve(OGRCurve* poCurve)
  * @since GDAL 2.0
  */
 
-OGRLineString* OGRCurve::CastToLineString(OGRCurve* poCurve)
+OGRLineString* OGRCurve::CastToLineString( OGRCurve* poCurve )
 {
     OGRCurveCasterToLineString pfn = poCurve->GetCasterToLineString();
     return pfn(poCurve);
@@ -351,7 +405,7 @@ OGRLineString* OGRCurve::CastToLineString(OGRCurve* poCurve)
  * @since GDAL 2.0
  */
 
-OGRLinearRing* OGRCurve::CastToLinearRing(OGRCurve* poCurve)
+OGRLinearRing* OGRCurve::CastToLinearRing( OGRCurve* poCurve )
 {
     OGRCurveCasterToLinearRing pfn = poCurve->GetCasterToLinearRing();
     return pfn(poCurve);
@@ -372,7 +426,7 @@ OGRLinearRing* OGRCurve::CastToLinearRing(OGRCurve* poCurve)
  * @since GDAL 2.0
  */
 
-int OGRCurve::ContainsPoint( CPL_UNUSED const OGRPoint* p ) const
+int OGRCurve::ContainsPoint( const OGRPoint* /* p */ ) const
 {
     return -1;
 }
@@ -381,9 +435,7 @@ int OGRCurve::ContainsPoint( CPL_UNUSED const OGRPoint* p ) const
 /*                          ~OGRPointIterator()                         */
 /************************************************************************/
 
-OGRPointIterator::~OGRPointIterator()
-{
-}
+OGRPointIterator::~OGRPointIterator() {}
 
 /**
  * \fn OGRBoolean OGRPointIterator::getNextPoint(OGRPoint* p);
@@ -406,7 +458,7 @@ OGRPointIterator::~OGRPointIterator()
  *
  * @since GDAL 2.0
  */
-void OGRPointIterator::destroy(OGRPointIterator* poIter)
+void OGRPointIterator::destroy( OGRPointIterator* poIter )
 {
     delete poIter;
 }
