@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id: FGdbDatasource.cpp 33563 2016-02-26 14:57:06Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements FileGDB OGR Datasource.
@@ -37,7 +36,7 @@
 #include "FGdbUtils.h"
 #include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: FGdbDatasource.cpp 33563 2016-02-26 14:57:06Z rouault $");
+CPL_CVSID("$Id: FGdbDatasource.cpp 36682 2016-12-04 20:34:45Z rouault $");
 
 using std::vector;
 using std::wstring;
@@ -46,7 +45,7 @@ using std::wstring;
 /*                          FGdbDataSource()                           */
 /************************************************************************/
 
-FGdbDataSource::FGdbDataSource(FGdbDriver* poDriverIn, 
+FGdbDataSource::FGdbDataSource(FGdbDriver* poDriverIn,
                                FGdbDatabaseConnection* pConnection):
 OGRDataSource(),
 m_poDriver(poDriverIn), m_pConnection(pConnection), m_pGeodatabase(NULL), m_bUpdate(false),
@@ -62,7 +61,7 @@ m_poOpenFileGDBDrv(NULL)
 FGdbDataSource::~FGdbDataSource()
 {
     CPLMutexHolderOptionalLockD(m_poDriver ? m_poDriver->GetMutex() : NULL);
-    
+
     if( m_pConnection && m_pConnection->IsLocked() )
         CommitTransaction();
 
@@ -117,7 +116,7 @@ int FGdbDataSource::FixIndexes()
             size_t count = m_layers.size();
             for(size_t i = 0; i < count; ++i )
             {
-                if( m_layers[i]->m_oMapOGRFIDToFGDBFID.size() == 0)
+                if( m_layers[i]->m_oMapOGRFIDToFGDBFID.empty())
                     continue;
                 CPLString osFilter = "name = '";
                 osFilter += m_layers[i]->GetName();
@@ -165,11 +164,11 @@ int FGdbDataSource::Open(const char * pszNewName, int bUpdate,
 
     std::vector<std::wstring> typesRequested;
 
-	// We're only interested in Tables, Feature Datasets and Feature Classes
-	typesRequested.push_back(L"Feature Class");
-	typesRequested.push_back(L"Table");
-	typesRequested.push_back(L"Feature Dataset");
-	
+    // We're only interested in Tables, Feature Datasets and Feature Classes
+    typesRequested.push_back(L"Feature Class");
+    typesRequested.push_back(L"Table");
+    typesRequested.push_back(L"Feature Dataset");
+
     bool rv = LoadLayers(L"\\");
 
     return rv;
@@ -220,7 +219,7 @@ int FGdbDataSource::ReOpen()
                  m_osFSName.c_str());
         return FALSE;
     }
-    
+
     int bRet = TRUE;
     size_t count = m_layers.size();
     for(size_t i = 0; i < count; ++i )
@@ -243,14 +242,14 @@ int FGdbDataSource::ReOpen()
         m_layers[i]->m_oMapOGRFIDToFGDBFID.clear();
         m_layers[i]->m_oMapFGDBFIDToOGRFID.clear();
     }
-    
+
     m_pGeodatabase = pDS->m_pGeodatabase;
     pDS->m_pGeodatabase = NULL;
 
     pDS->m_poDriver = NULL;
     pDS->m_pConnection = NULL;
     delete pDS;
-    
+
     return bRet;
 }
 
@@ -269,7 +268,7 @@ bool FGdbDataSource::OpenFGDBTables(const std::wstring &type,
         if (FAILED(hr = m_pGeodatabase->OpenTable(layers[i], *pTable)))
         {
             delete pTable;
-            
+
             std::wstring fgdb_error_desc_w;
             fgdbError er;
             er = FileGDBAPI::ErrorInfo::GetErrorDescription(hr, fgdb_error_desc_w);
@@ -304,7 +303,7 @@ bool FGdbDataSource::OpenFGDBTables(const std::wstring &type,
 /*                            LoadLayers()                             */
 /************************************************************************/
 
-bool FGdbDataSource::LoadLayers(const std::wstring &root) 
+bool FGdbDataSource::LoadLayers(const std::wstring &root)
 {
     std::vector<wstring> tables;
     std::vector<wstring> featureclasses;
@@ -317,7 +316,7 @@ bool FGdbDataSource::LoadLayers(const std::wstring &root)
         return GDBErr(hr, "Error reading Tables in " + WStringToString(root));
     }
     /* Open the tables we found */
-    if ( tables.size() > 0 && ! OpenFGDBTables(L"Table", tables) )
+    if ( !tables.empty() && ! OpenFGDBTables(L"Table", tables) )
         return false;
 
     /* Find all the Feature Classes in the root */
@@ -326,7 +325,7 @@ bool FGdbDataSource::LoadLayers(const std::wstring &root)
         return GDBErr(hr, "Error reading Feature Classes in " + WStringToString(root));
     }
     /* Open the tables we found */
-    if ( featureclasses.size() > 0 && ! OpenFGDBTables(L"Feature Class", featureclasses) )
+    if ( !featureclasses.empty() && ! OpenFGDBTables(L"Feature Class", featureclasses) )
         return false;
 
     /* Find all the Feature Datasets in the root */
@@ -341,13 +340,12 @@ bool FGdbDataSource::LoadLayers(const std::wstring &root)
         {
             return GDBErr(hr, "Error reading Feature Classes in " + WStringToString(featuredatasets[i]));
         }
-        if ( featureclasses.size() > 0 && ! OpenFGDBTables(L"Feature Class", featureclasses) )
+        if ( !featureclasses.empty() && ! OpenFGDBTables(L"Feature Class", featureclasses) )
             return false;
     }
     return true;
 }
 
-
 #if 0
 /************************************************************************/
 /*                            LoadLayersOld()                              */
@@ -375,7 +373,7 @@ bool FGdbDataSource::LoadLayersOld(const std::vector<wstring> & datasetTypes,
         std::vector<wstring> childDatasets;
         m_pGeodatabase->GetChildDatasets( parent, datasetTypes[dsTypeIndex], childDatasets);
 
-        if (childDatasets.size() > 0)
+        if (!childDatasets.empty())
         {
             //it is a container of other datasets
 
@@ -425,7 +423,6 @@ bool FGdbDataSource::LoadLayersOld(const std::vector<wstring> & datasetTypes,
 }
 #endif
 
-
 /************************************************************************/
 /*                            DeleteLayer()                             */
 /************************************************************************/
@@ -437,7 +434,7 @@ OGRErr FGdbDataSource::DeleteLayer( int iLayer )
 
     if( iLayer < 0 || iLayer >= static_cast<int>(m_layers.size()) )
         return OGRERR_FAILURE;
-    
+
     FGdbLayer* poBaseLayer = m_layers[iLayer];
 
     // Fetch FGDBAPI Table before deleting OGR layer object
@@ -456,7 +453,7 @@ OGRErr FGdbDataSource::DeleteLayer( int iLayer )
     m_layers.erase(m_layers.begin() + iLayer);
 
     long hr;
-  
+
     if (FAILED(hr = m_pGeodatabase->Delete(strPath, strType)))
     {
         CPLError( CE_Warning, CPLE_AppDefined,
@@ -479,18 +476,20 @@ int FGdbDataSource::TestCapability( const char * pszCap )
 
     else if( EQUAL(pszCap,ODsCDeleteLayer) )
         return m_bUpdate;
-    else if EQUAL(pszCap,ODsCCreateGeomFieldAfterCreateLayer)
+    else if( EQUAL(pszCap,ODsCCreateGeomFieldAfterCreateLayer) )
         return TRUE;
+    else if( EQUAL(pszCap,ODsCRandomLayerWrite) )
+        return m_bUpdate;
+
     return FALSE;
 }
 
-
 /************************************************************************/
 /*                              GetLayer()                              */
 /************************************************************************/
 
 OGRLayer *FGdbDataSource::GetLayer( int iLayer )
-{ 
+{
     int count = static_cast<int>(m_layers.size());
 
     if( iLayer < 0 || iLayer >= count )
@@ -523,10 +522,9 @@ FGdbDataSource::ICreateLayer( const char * pszLayerName,
 
     m_layers.push_back(pLayer);
 
-    return pLayer;  
+    return pLayer;
 }
 
-
 /************************************************************************/
 /*                   OGRFGdbSingleFeatureLayer                          */
 /************************************************************************/
@@ -541,12 +539,12 @@ class OGRFGdbSingleFeatureLayer : public OGRLayer
   public:
                         OGRFGdbSingleFeatureLayer( const char* pszLayerName,
                                                    const char *pszVal );
-                        ~OGRFGdbSingleFeatureLayer();
+               virtual ~OGRFGdbSingleFeatureLayer();
 
-    virtual void        ResetReading() { iNextShapeId = 0; }
-    virtual OGRFeature *GetNextFeature();
-    virtual OGRFeatureDefn *GetLayerDefn() { return poFeatureDefn; }
-    virtual int         TestCapability( const char * ) { return FALSE; }
+    virtual void        ResetReading() override { iNextShapeId = 0; }
+    virtual OGRFeature *GetNextFeature() override;
+    virtual OGRFeatureDefn *GetLayerDefn() override { return poFeatureDefn; }
+    virtual int         TestCapability( const char * ) override { return FALSE; }
 };
 
 /************************************************************************/
@@ -563,7 +561,7 @@ OGRFGdbSingleFeatureLayer::OGRFGdbSingleFeatureLayer(const char* pszLayerName,
     poFeatureDefn->AddFieldDefn( &oField );
 
     iNextShapeId = 0;
-    this->pszVal = pszValIn ? CPLStrdup(pszValIn) : NULL;
+    pszVal = pszValIn ? CPLStrdup(pszValIn) : NULL;
 }
 
 /************************************************************************/
@@ -577,7 +575,6 @@ OGRFGdbSingleFeatureLayer::~OGRFGdbSingleFeatureLayer()
     CPLFree(pszVal);
 }
 
-
 /************************************************************************/
 /*                           GetNextFeature()                           */
 /************************************************************************/
@@ -616,7 +613,7 @@ OGRLayer * FGdbDataSource::ExecuteSQL( const char *pszSQLCommand,
     {
         m_layers[i]->EndBulkLoad();
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Use generic implementation for recognized dialects              */
 /* -------------------------------------------------------------------- */
