@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id: gtm.cpp 32760 2016-01-05 17:50:33Z rouault $
  *
  * Project:  GTM Driver
  * Purpose:  Class for reading, parsing and handling a gtmfile.
@@ -30,6 +29,7 @@
 
 #include "gtm.h"
 
+CPL_CVSID("$Id: gtm.cpp 38444 2017-05-17 08:54:05Z rouault $");
 
 /************************************************************************/
 /*        Methods for dealing with write on files and buffers           */
@@ -105,15 +105,15 @@ static unsigned char readUChar(VSILFILE* fp)
     return val;
 }
 
-static unsigned short readUShort(VSILFILE* fp, int *pbSuccess = NULL)
+static unsigned short readUShort( VSILFILE* fp, bool *pbSuccess = NULL )
 {
     unsigned short val;
     if (VSIFReadL( &val, 1, 2, fp ) != 2)
     {
-        if (pbSuccess) *pbSuccess = FALSE;
+        if (pbSuccess) *pbSuccess = false;
         return 0;
     }
-    if (pbSuccess) *pbSuccess = TRUE;
+    if (pbSuccess) *pbSuccess = true;
     CPL_LSBPTR16(&val);
     return val;
 }
@@ -226,7 +226,6 @@ unsigned char Track::getType() const
     return type;
 }
 
-
 int Track::getColor() const
 {
     return color;
@@ -256,7 +255,6 @@ const TrackPoint* Track::getPoint(int pointNum) const
     return NULL;
 }
 
-
 /************************************************************************/
 /*                Implementation of GTM Function Members                */
 /************************************************************************/
@@ -307,7 +305,6 @@ bool GTM::Open(const char* pszFilenameIn)
     return true;
 }
 
-
 bool GTM::isValid()
 {
 /* -------------------------------------------------------------------- */
@@ -342,7 +339,11 @@ bool GTM::isValid()
         {
             VSILFILE* pGTMFileOri = pGTMFile;
             pGTMFile = fp;
-            if (isValid())
+            char* pszFilenameOri = pszFilename;
+            pszFilename = pszGZIPFileName;
+            const bool bRet = isValid();
+            pszFilename = pszFilenameOri;
+            if (bRet)
             {
                 VSIFCloseL(pGTMFileOri);
                 CPLFree(pszGZIPFileName);
@@ -358,7 +359,7 @@ bool GTM::isValid()
         CPLFree(pszGZIPFileName);
     }
 
-    const short version = CPL_LSBINT16PTR(buffer);
+    const short version = CPL_LSBSINT16PTR(buffer);
     /*Skip string length */
     const char* szHeader = buffer + 2;
     if (version == 211 && strcmp(szHeader, "TrackMaker") == 0 )
@@ -430,7 +431,6 @@ bool GTM::readHeaderNumbers()
     stringSize = readUShort(pGTMFile);
     headerSize += stringSize + 2; // String + size field
 
-
     /* Read userfont string size */
     if ( VSIFSeekL(pGTMFile, stringSize, SEEK_CUR) != 0)
         return false;
@@ -513,8 +513,8 @@ Waypoint* GTM::fetchNextWaypoint()
 
     /* Trim string name */
     {
-    int i;
-        for (i = 9; i >= 0; --i)
+        int i = 9;  // Used after for.
+        for( ; i >= 0; --i)
         {
             if (name[i] != ' ')
             {
@@ -561,7 +561,6 @@ Waypoint* GTM::fetchNextWaypoint()
     Waypoint* poWaypoint = new Waypoint(latitude, longitude, altitude,
                                         name, comment, (int) icon, wptdate);
 
-
     /* Set actual waypoint offset to the next it there is one */
     ++waypointFetched;
     if (waypointFetched < nwpts)
@@ -574,7 +573,6 @@ Waypoint* GTM::fetchNextWaypoint()
     return poWaypoint;
 }
 
-
 /************************************************************************/
 /*                        Track control functions                    */
 /************************************************************************/
@@ -598,22 +596,14 @@ void GTM::rewindTrack()
 
 Track* GTM::fetchNextTrack()
 {
-    unsigned short stringSize;
-
-    char* pszName;
-    unsigned char type;
-    int color;
-
-
     /* Point to the actual track offset */
     if ( VSIFSeekL(pGTMFile, actualTrackOffset, SEEK_SET) != 0)
         return NULL;
 
-
     /* Read string length */
-    stringSize = readUShort(pGTMFile);
+    const unsigned short stringSize = readUShort(pGTMFile);
     /* Read name string */
-    pszName = (char*) VSI_MALLOC2_VERBOSE(sizeof(char), stringSize+1);
+    char* pszName = (char*) VSI_MALLOC2_VERBOSE(sizeof(char), stringSize+1);
     if( pszName == NULL )
         return NULL;
     if ( stringSize != 0 && !readFile( pszName, 1, sizeof(char) * stringSize ) )
@@ -624,10 +614,10 @@ Track* GTM::fetchNextTrack()
     pszName[stringSize] = '\0';
 
     /* Read type */
-    type = readUChar(pGTMFile);
+    const unsigned char type = readUChar(pGTMFile);
 
     /* Read color */
-    color = readInt(pGTMFile);
+    const int color = readInt(pGTMFile);
 
     Track* poTrack = new Track(pszName, type, color);
 
@@ -637,10 +627,11 @@ Track* GTM::fetchNextTrack()
     ++trackFetched;
 
     /* Now, We read all trackpoints for this track */
-    double latitude, longitude;
-    GIntBig datetime;
-    unsigned char start;
-    float altitude;
+    double latitude = 0.0;
+    double longitude = 0.0;
+    GIntBig datetime = 0;
+    unsigned char start = 0;
+    float altitude = 0.0f;
     /* NOTE: Parameters are passed by reference */
     if ( !readTrackPoints(latitude, longitude, datetime, start, altitude) )
     {
@@ -678,8 +669,6 @@ Track* GTM::fetchNextTrack()
     return poTrack;
 }
 
-
-
 /************************************************************************/
 /*                        Private Methods Implementation                */
 /************************************************************************/
@@ -713,7 +702,6 @@ vsi_l_offset GTM::findFirstWaypointOffset()
     return VSIFTellL(pGTMFile);
 }
 
-
 vsi_l_offset GTM::findFirstTrackpointOffset()
 {
     if (firstWaypointOffset == 0)
@@ -730,8 +718,6 @@ vsi_l_offset GTM::findFirstTrackpointOffset()
     if (VSIFSeekL(pGTMFile, firstWaypointOffset, SEEK_SET) != 0)
         return 0;
 
-    unsigned short stringSize;
-    int bSuccess;
     /* Skip waypoints */
     for (int i = 0; i < nwpts; ++i)
     {
@@ -739,10 +725,11 @@ vsi_l_offset GTM::findFirstTrackpointOffset()
         if (VSIFSeekL(pGTMFile, 26, SEEK_CUR) != 0)
             return 0;
         /* Read string comment size */
-        stringSize = readUShort(pGTMFile, &bSuccess);
+        bool bSuccess = false;
+        const unsigned short stringSize = readUShort(pGTMFile, &bSuccess);
 
         /* Skip to the next Waypoint */
-        if (bSuccess == FALSE || VSIFSeekL(pGTMFile, stringSize + 15, SEEK_CUR) != 0)
+        if( !bSuccess || VSIFSeekL(pGTMFile, stringSize + 15, SEEK_CUR) != 0 )
             return 0;
     }
 
@@ -758,10 +745,12 @@ vsi_l_offset GTM::findFirstTrackpointOffset()
                 return 0;
 
             /* Read string facename size */
-            stringSize = readUShort(pGTMFile, &bSuccess);
+            bool bSuccess = false;
+            const unsigned short stringSize = readUShort(pGTMFile, &bSuccess);
 
             /* Skip to the next Waypoint Style*/
-            if (bSuccess == FALSE || VSIFSeekL(pGTMFile, stringSize + 24, SEEK_CUR) != 0)
+            if( !bSuccess ||
+                VSIFSeekL(pGTMFile, stringSize + 24, SEEK_CUR) != 0 )
                 return 0;
         }
     }
@@ -827,6 +816,3 @@ bool GTM::readFile(void* pBuffer, size_t nSize, size_t nCount)
     }
     return true;
 }
-
-
-
