@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id: kml.cpp 27729 2014-09-24 00:40:16Z goatbar $
  *
  * Project:  KML Driver
  * Purpose:  Class for reading, parsing and handling a kmlfile.
@@ -29,25 +28,29 @@
  ****************************************************************************/
 #include "kmlnode.h"
 #include "kml.h"
-#include "cpl_error.h"
-#include "cpl_conv.h"
-// std
-#include <cstdio>
-#include <cerrno>
-#include <string>
-#include <iostream>
 
-KML::KML()
-{
-	nDepth_ = 0;
-	validity = KML_VALIDITY_UNKNOWN;
-	pKMLFile_ = NULL;
-	sError_ = "";
-	poTrunk_ = NULL;
-	poCurrent_ = NULL;
-	nNumLayers_ = -1;
-        papoLayers_ = NULL;
-}
+#include "cpl_conv.h"
+#include "cpl_error.h"
+
+#include <cerrno>
+#include <cstdio>
+#include <iostream>
+#include <string>
+
+CPL_CVSID("$Id: kml.cpp 38409 2017-05-16 09:58:58Z rouault $");
+
+KML::KML() :
+    poTrunk_(NULL),
+    nNumLayers_(-1),
+    papoLayers_(NULL),
+    nDepth_(0),
+    validity(KML_VALIDITY_UNKNOWN),
+    pKMLFile_(NULL),
+    poCurrent_(NULL),
+    oCurrentParser(NULL),
+    nDataHandlerCounter(0),
+    nWithoutEventCounter(0)
+{}
 
 KML::~KML()
 {
@@ -64,20 +67,11 @@ bool KML::open(const char * pszFilename)
         VSIFCloseL( pKMLFile_ );
 
     pKMLFile_ = VSIFOpenL( pszFilename, "r" );
-    if( NULL == pKMLFile_ )
-    {
-        return FALSE;
-    }
-
-    return TRUE;
+    return pKMLFile_ != NULL;
 }
 
 void KML::parse()
 {
-    std::size_t nDone = 0;
-    std::size_t nLen = 0;
-    char aBuf[BUFSIZ] = { 0 };
-
     if( NULL == pKMLFile_ )
     {
         sError_ = "No file given";
@@ -102,6 +96,10 @@ void KML::parse()
     oCurrentParser = oParser;
     nWithoutEventCounter = 0;
 
+    int nDone = 0;
+    int nLen = 0;
+    char aBuf[BUFSIZ] = { 0 };
+
     do
     {
         nDataHandlerCounter = 0;
@@ -109,11 +107,12 @@ void KML::parse()
         nDone = VSIFEofL(pKMLFile_);
         if (XML_Parse(oParser, aBuf, nLen, nDone) == XML_STATUS_ERROR)
         {
-            CPLError(CE_Failure, CPLE_AppDefined,
-                        "XML parsing of KML file failed : %s at line %d, column %d",
-                        XML_ErrorString(XML_GetErrorCode(oParser)),
-                        (int)XML_GetCurrentLineNumber(oParser),
-                        (int)XML_GetCurrentColumnNumber(oParser));
+            CPLError( CE_Failure, CPLE_AppDefined,
+                      "XML parsing of KML file failed : %s at line %d, "
+                      "column %d",
+                      XML_ErrorString(XML_GetErrorCode(oParser)),
+                      static_cast<int>(XML_GetCurrentLineNumber(oParser)),
+                      static_cast<int>(XML_GetCurrentColumnNumber(oParser)));
             XML_ParserFree(oParser);
             VSIRewindL(pKMLFile_);
             return;
@@ -134,10 +133,6 @@ void KML::parse()
 
 void KML::checkValidity()
 {
-    std::size_t nDone = 0;
-    std::size_t nLen = 0;
-    char aBuf[BUFSIZ] = { 0 };
-
     if(poTrunk_ != NULL)
     {
         delete poTrunk_;
@@ -152,7 +147,7 @@ void KML::checkValidity()
 
     if(pKMLFile_ == NULL)
     {
-        this->sError_ = "No file given";
+        sError_ = "No file given";
         return;
     }
 
@@ -164,11 +159,15 @@ void KML::checkValidity()
 
     oCurrentParser = oParser;
 
-    /* Parses the file until we find the first element */
+    int nDone = 0;
+    int nLen = 0;
+    char aBuf[BUFSIZ] = { 0 };
+
+    // Parses the file until we find the first element.
     do
     {
         nDataHandlerCounter = 0;
-        nLen = (int)VSIFReadL( aBuf, 1, sizeof(aBuf), pKMLFile_ );
+        nLen = static_cast<int>(VSIFReadL( aBuf, 1, sizeof(aBuf), pKMLFile_ ));
         nDone = VSIFEofL(pKMLFile_);
         if (XML_Parse(oParser, aBuf, nLen, nDone) == XML_STATUS_ERROR)
         {
@@ -196,25 +195,26 @@ void KML::checkValidity()
         nCount ++;
         /* After reading 50 * BUFSIZE bytes, and not finding whether the file */
         /* is KML or not, we give up and fail silently */
-    } while (!nDone && nLen > 0 && validity == KML_VALIDITY_UNKNOWN && nCount < 50);
+    } while ( !nDone && nLen > 0 &&
+              validity == KML_VALIDITY_UNKNOWN && nCount < 50 );
 
     XML_ParserFree(oParser);
     VSIRewindL(pKMLFile_);
     poCurrent_ = NULL;
 }
 
-void XMLCALL KML::startElement(void* pUserData, const char* pszName, const char** ppszAttr)
+void XMLCALL KML::startElement( void* pUserData, const char* pszName,
+                                const char** ppszAttr )
 {
-    int i = 0;
     KMLNode* poMynew = NULL;
-    Attribute* poAtt = NULL;
 
-    KML* poKML = (KML*) pUserData;
+    KML* poKML = static_cast<KML*>(pUserData);
 
     poKML->nWithoutEventCounter = 0;
 
-    if(poKML->poTrunk_ == NULL 
-    || (poKML->poCurrent_->getName()).compare("description") != 0)
+    if(poKML->poTrunk_ == NULL
+    || (poKML->poCurrent_ != NULL &&
+        poKML->poCurrent_->getName().compare("description") != 0))
     {
         if (poKML->nDepth_ == 1024)
         {
@@ -229,9 +229,9 @@ void XMLCALL KML::startElement(void* pUserData, const char* pszName, const char*
             poMynew->setName(pszName);
         poMynew->setLevel(poKML->nDepth_);
 
-        for (i = 0; ppszAttr[i]; i += 2)
+        for( int i = 0; ppszAttr[i]; i += 2 )
         {
-            poAtt = new Attribute();
+            Attribute* poAtt = new Attribute();
             poAtt->sName = ppszAttr[i];
             poAtt->sValue = ppszAttr[i + 1];
             poMynew->addAttribute(poAtt);
@@ -245,11 +245,11 @@ void XMLCALL KML::startElement(void* pUserData, const char* pszName, const char*
 
         poKML->nDepth_++;
     }
-    else
+    else if( poKML->poCurrent_ != NULL )
     {
         std::string sNewContent = "<";
         sNewContent += pszName;
-        for (i = 0; ppszAttr[i]; i += 2)
+        for( int i = 0; ppszAttr[i]; i += 2 )
         {
             sNewContent += " ";
             sNewContent += ppszAttr[i];
@@ -265,11 +265,10 @@ void XMLCALL KML::startElement(void* pUserData, const char* pszName, const char*
     }
 }
 
-void XMLCALL KML::startElementValidate(void* pUserData, const char* pszName, const char** ppszAttr)
+void XMLCALL KML::startElementValidate( void* pUserData, const char* pszName,
+                                        const char** ppszAttr )
 {
-    int i = 0;
-
-    KML* poKML = (KML*) pUserData;
+    KML* poKML = static_cast<KML *>(pUserData);
 
     if (poKML->validity != KML_VALIDITY_UNKNOWN)
         return;
@@ -279,13 +278,13 @@ void XMLCALL KML::startElementValidate(void* pUserData, const char* pszName, con
     if(strcmp(pszName, "kml") == 0 || strcmp(pszName, "Document") == 0)
     {
         // Check all Attributes
-        for (i = 0; ppszAttr[i]; i += 2)
+        for( int i = 0; ppszAttr[i]; i += 2 )
         {
             // Find the namespace and determine the KML version
             if(strcmp(ppszAttr[i], "xmlns") == 0)
             {
                 // Is it KML 2.2?
-                if((strcmp(ppszAttr[i + 1], "http://earth.google.com/kml/2.2") == 0) || 
+                if((strcmp(ppszAttr[i + 1], "http://earth.google.com/kml/2.2") == 0) ||
                    (strcmp(ppszAttr[i + 1], "http://www.opengis.net/kml/2.2") == 0))
                 {
                     poKML->validity = KML_VALIDITY_VALID;
@@ -303,7 +302,9 @@ void XMLCALL KML::startElementValidate(void* pUserData, const char* pszName, con
                 }
                 else
                 {
-                    CPLDebug("KML", "Unhandled xmlns value : %s. Going on though...", ppszAttr[i]);
+                    CPLDebug("KML",
+                             "Unhandled xmlns value : %s. Going on though...",
+                             ppszAttr[i]);
                     poKML->validity = KML_VALIDITY_VALID;
                     poKML->sVersion_ = "?";
                 }
@@ -312,32 +313,33 @@ void XMLCALL KML::startElementValidate(void* pUserData, const char* pszName, con
 
         if (poKML->validity == KML_VALIDITY_INVALID)
         {
-            CPLDebug("KML", "Did not find xmlns attribute in <kml> element. Going on though...");
+            CPLDebug( "KML",
+                      "Did not find xmlns attribute in <kml> element. "
+                      "Going on though..." );
             poKML->validity = KML_VALIDITY_VALID;
             poKML->sVersion_ = "?";
         }
     }
 }
 
-void XMLCALL KML::dataHandlerValidate(void * pUserData,
-                                      CPL_UNUSED const char * pszData,
-                                      CPL_UNUSED int nLen)
+void XMLCALL KML::dataHandlerValidate( void * pUserData,
+                                       const char * /* pszData */,
+                                       int /* nLen */ )
 {
-    KML* poKML = (KML*) pUserData;
+    KML* poKML = static_cast<KML *>(pUserData);
 
     poKML->nDataHandlerCounter ++;
     if (poKML->nDataHandlerCounter >= BUFSIZ)
     {
-        CPLError(CE_Failure, CPLE_AppDefined, "File probably corrupted (million laugh pattern)");
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "File probably corrupted (million laugh pattern)" );
         XML_StopParser(poKML->oCurrentParser, XML_FALSE);
     }
 }
 
 void XMLCALL KML::endElement(void* pUserData, const char* pszName)
 {
-    KMLNode* poTmp = NULL;
-
-    KML* poKML = (KML*) pUserData;
+    KML* poKML = static_cast<KML *>(pUserData);
 
     poKML->nWithoutEventCounter = 0;
 
@@ -345,31 +347,32 @@ void XMLCALL KML::endElement(void* pUserData, const char* pszName)
        poKML->poCurrent_->getName().compare(pszName) == 0)
     {
         poKML->nDepth_--;
-        poTmp = poKML->poCurrent_;
+        KMLNode* poTmp = poKML->poCurrent_;
         // Split the coordinates
         if(poKML->poCurrent_->getName().compare("coordinates") == 0 &&
            poKML->poCurrent_->numContent() == 1)
         {
-            std::string sData = poKML->poCurrent_->getContent(0);
+            const std::string sData = poKML->poCurrent_->getContent(0);
             std::size_t nPos = 0;
-            std::size_t nLength = sData.length();
+            const std::size_t nLength = sData.length();
             const char* pszData = sData.c_str();
-            while(TRUE)
+            while( true )
             {
                 // Cut off whitespaces
-                while(nPos < nLength &&
-                      (pszData[nPos] == ' ' || pszData[nPos] == '\n'
-                       || pszData[nPos] == '\r' || pszData[nPos] == '\t' ))
-                    nPos ++;
+                while( nPos < nLength &&
+                       (pszData[nPos] == ' ' || pszData[nPos] == '\n'
+                        || pszData[nPos] == '\r' || pszData[nPos] == '\t' ) )
+                    nPos++;
 
                 if (nPos == nLength)
                     break;
 
-                std::size_t nPosBegin = nPos;
+                const std::size_t nPosBegin = nPos;
 
                 // Get content
                 while(nPos < nLength &&
-                      pszData[nPos] != ' ' && pszData[nPos] != '\n' && pszData[nPos] != '\r' && 
+                      pszData[nPos] != ' ' && pszData[nPos] != '\n' &&
+                      pszData[nPos] != '\r' &&
                       pszData[nPos] != '\t')
                     nPos++;
 
@@ -384,46 +387,50 @@ void XMLCALL KML::endElement(void* pUserData, const char* pszName)
         }
         else if (poKML->poCurrent_->numContent() == 1)
         {
-            std::string sData = poKML->poCurrent_->getContent(0);
+            const std::string sData = poKML->poCurrent_->getContent(0);
             std::string sDataWithoutNL;
             std::size_t nPos = 0;
-            std::size_t nLength = sData.length();
+            const std::size_t nLength = sData.length();
             const char* pszData = sData.c_str();
             std::size_t nLineStartPos = 0;
-            int bLineStart = TRUE;
+            bool bLineStart = true;
 
-            /* Re-assemble multi-line content by removing leading spaces for each line */
-            /* I'm not sure why we do that. Shouldn't we preserve content as such ? */
+            // Re-assemble multi-line content by removing leading spaces for
+            // each line.  I am not sure why we do that. Should we preserve
+            // content as such?
             while(nPos < nLength)
             {
-                char ch = pszData[nPos];
-                if (bLineStart && (ch == ' ' || ch == '\t' || ch == '\n' || ch == '\r'))
+                const char ch = pszData[nPos];
+                if( bLineStart && (ch == ' ' || ch == '\t' || ch == '\n' ||
+                                   ch == '\r') )
                     nLineStartPos ++;
-                else if (ch == '\n' || ch == '\r')
+                else if( ch == '\n' || ch == '\r' )
                 {
-                    if (!bLineStart)
+                    if( !bLineStart )
                     {
-                        std::string sTmp(pszData + nLineStartPos, nPos - nLineStartPos);
-                        if (sDataWithoutNL.size() > 0)
+                        std::string sTmp( pszData + nLineStartPos,
+                                          nPos - nLineStartPos);
+                        if( !sDataWithoutNL.empty() )
                             sDataWithoutNL += " ";
                         sDataWithoutNL += sTmp;
-                        bLineStart = TRUE;
+                        bLineStart = true;
                     }
                     nLineStartPos = nPos + 1;
                 }
                 else
                 {
-                    bLineStart = FALSE;
+                    bLineStart = false;
                 }
-                nPos ++;
+                nPos++;
             }
 
-            if (nLineStartPos > 0)
+            if( nLineStartPos > 0 )
             {
                 if (nLineStartPos < nPos)
                 {
-                    std::string sTmp(pszData + nLineStartPos, nPos - nLineStartPos);
-                    if (sDataWithoutNL.size() > 0)
+                    std::string sTmp( pszData + nLineStartPos,
+                                      nPos - nLineStartPos);
+                    if( !sDataWithoutNL.empty() )
                         sDataWithoutNL += " ";
                     sDataWithoutNL += sTmp;
                 }
@@ -463,7 +470,7 @@ void XMLCALL KML::endElement(void* pUserData, const char* pszName)
 
 void XMLCALL KML::dataHandler(void* pUserData, const char* pszData, int nLen)
 {
-    KML* poKML = (KML*) pUserData;
+    KML* poKML = static_cast<KML *>(pUserData);
 
     poKML->nWithoutEventCounter = 0;
 
@@ -473,7 +480,8 @@ void XMLCALL KML::dataHandler(void* pUserData, const char* pszData, int nLen)
     poKML->nDataHandlerCounter ++;
     if (poKML->nDataHandlerCounter >= BUFSIZ)
     {
-        CPLError(CE_Failure, CPLE_AppDefined, "File probably corrupted (million laugh pattern)");
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "File probably corrupted (million laugh pattern)" );
         XML_StopParser(poKML->oCurrentParser, XML_FALSE);
     }
 
@@ -498,15 +506,15 @@ bool KML::isValid()
     checkValidity();
 
     if( validity == KML_VALIDITY_VALID )
-        CPLDebug("KML", "Valid: %d Version: %s", 
-                 validity == KML_VALIDITY_VALID, sVersion_.c_str());
+        CPLDebug( "KML", "Valid: %d Version: %s",
+                  validity == KML_VALIDITY_VALID, sVersion_.c_str());
 
     return validity == KML_VALIDITY_VALID;
 }
 
 std::string KML::getError() const
 {
-	return sError_;
+    return sError_;
 }
 
 int KML::classifyNodes()
@@ -527,40 +535,36 @@ void KML::print(unsigned short nNum)
 
 bool KML::isHandled(std::string const& elem) const
 {
-    if( isLeaf(elem) || isFeature(elem) || isFeatureContainer(elem)
-        || isContainer(elem) || isRest(elem) )
-    {
-        return true;
-    }
-    return false;
+    return isLeaf(elem) || isFeature(elem) || isFeatureContainer(elem)
+        || isContainer(elem) || isRest(elem);
 }
 
-bool KML::isLeaf(CPL_UNUSED std::string const& elem) const
+bool KML::isLeaf( std::string const& /* elem */ ) const
 {
     return false;
 };
 
-bool KML::isFeature(CPL_UNUSED std::string const& elem) const
+bool KML::isFeature( std::string const& /* elem */ ) const
 {
     return false;
 };
 
-bool KML::isFeatureContainer(CPL_UNUSED std::string const& elem) const
+bool KML::isFeatureContainer( std::string const& /* elem */ ) const
 {
     return false;
 };
 
-bool KML::isContainer(CPL_UNUSED std::string const& elem) const
+bool KML::isContainer( std::string const& /* elem */ ) const
 {
     return false;
 };
 
-bool KML::isRest(CPL_UNUSED std::string const& elem) const
+bool KML::isRest( std::string const& /* elem */ ) const
 {
     return false;
 };
 
-void KML::findLayers(CPL_UNUSED KMLNode* poNode, CPL_UNUSED int bKeepEmptyContainers)
+void KML::findLayers( KMLNode* /* poNode */, int /* bKeepEmptyContainers */ )
 {
     // idle
 };
@@ -576,10 +580,10 @@ int KML::getNumLayers() const
 }
 
 bool KML::selectLayer(int nNum) {
-    if(this->nNumLayers_ < 1 || nNum >= this->nNumLayers_)
-        return FALSE;
+    if( nNumLayers_ < 1 || nNum >= nNumLayers_ )
+        return false;
     poCurrent_ = papoLayers_[nNum];
-    return TRUE;
+    return true;
 }
 
 std::string KML::getCurrentName() const
@@ -596,30 +600,48 @@ Nodetype KML::getCurrentType() const
 {
     if(poCurrent_ != NULL)
         return poCurrent_->getType();
-    else
-        return Unknown;
+
+    return Unknown;
 }
 
 int KML::is25D() const
 {
     if(poCurrent_ != NULL)
         return poCurrent_->is25D();
-    else
-        return Unknown;
+
+    return Unknown;
 }
 
 int KML::getNumFeatures()
 {
-    if(poCurrent_ != NULL)
-        return static_cast<int>(poCurrent_->getNumFeatures());
-    else
+    if(poCurrent_ == NULL)
         return -1;
+
+    return static_cast<int>(poCurrent_->getNumFeatures());
 }
 
 Feature* KML::getFeature(std::size_t nNum, int& nLastAsked, int &nLastCount)
 {
-    if(poCurrent_ != NULL)
-        return poCurrent_->getFeature(nNum, nLastAsked, nLastCount);
-    else
+    if(poCurrent_ == NULL)
         return NULL;
+
+    return poCurrent_->getFeature(nNum, nLastAsked, nLastCount);
+}
+
+void KML::unregisterLayerIfMatchingThisNode(KMLNode* poNode)
+{
+    for(int i=0;i<nNumLayers_;)
+    {
+        if( papoLayers_[i] == poNode )
+        {
+            if( i < nNumLayers_ - 1 )
+            {
+                memmove( papoLayers_ + i, papoLayers_ + i + 1,
+                        (nNumLayers_ - 1 - i) * sizeof(KMLNode*) );
+            }
+            nNumLayers_ --;
+            break;
+        }
+        i++;
+    }
 }
