@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id: ogrwalkdatasource.cpp
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRWalkDatasource class.
@@ -30,6 +29,8 @@
 #include "ogrwalk.h"
 #include <vector>
 
+CPL_CVSID("$Id: ogrwalkdatasource.cpp 35577 2016-09-30 23:30:37Z goatbar $");
+
 /************************************************************************/
 /*                         OGRWalkDataSource()                          */
 /************************************************************************/
@@ -37,9 +38,8 @@
 OGRWalkDataSource::OGRWalkDataSource() :
     pszName(NULL),
     papoLayers(NULL),
-    nLayers(0),
-    bDSUpdate(FALSE)
-{ }
+    nLayers(0)
+{}
 
 /************************************************************************/
 /*                        ~OGRWalkDataSource()                          */
@@ -63,17 +63,19 @@ OGRWalkDataSource::~OGRWalkDataSource()
 /*                              Open()                                  */
 /************************************************************************/
 
-int OGRWalkDataSource::Open( const char * pszNewName, int bUpdate )
+int OGRWalkDataSource::Open( const char * pszNewName, int /* bUpdate */ )
 {
 /* -------------------------------------------------------------------- */
 /*      If this is the name of an MDB file, then construct the          */
 /*      appropriate connection string.  Otherwise clip of WALK: to      */
 /*      get the DSN.                                                    */
 /* -------------------------------------------------------------------- */
-    char *pszDSN;
+    char *pszDSN = NULL;
 
     if( STARTS_WITH_CI(pszNewName, "WALK:") )
+    {
         pszDSN = CPLStrdup( pszNewName + 5 );
+    }
     else
     {
         const char *pszDSNStringTemplate = "DRIVER=Microsoft Access Driver (*.mdb);DBQ=%s";
@@ -102,8 +104,6 @@ int OGRWalkDataSource::Open( const char * pszNewName, int bUpdate )
 
     pszName = CPLStrdup( pszNewName );
 
-    bDSUpdate = bUpdate;
-
 /* -------------------------------------------------------------------- */
 /*      Collect list of layers and their attributes.                    */
 /* -------------------------------------------------------------------- */
@@ -135,12 +135,10 @@ int OGRWalkDataSource::Open( const char * pszNewName, int bUpdate )
 /* -------------------------------------------------------------------- */
 /*      Create a layer for each spatial table.                          */
 /* -------------------------------------------------------------------- */
-    unsigned int iTable;
-
     papoLayers = (OGRWalkLayer **) CPLCalloc(apapszGeomColumns.size(),
                                              sizeof( void * ));
 
-    for( iTable = 0; iTable < apapszGeomColumns.size(); iTable++ )
+    for( unsigned int iTable = 0; iTable < apapszGeomColumns.size(); iTable++ )
     {
         char **papszRecord = apapszGeomColumns[iTable];
 
