@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_xls.h 31777 2015-11-26 14:14:41Z rouault $
+ * $Id: ogr_xls.h 36501 2016-11-25 14:09:24Z rouault $
  *
  * Project:  XLS Translator
  * Purpose:  Definition of classes for OGR .xls driver.
@@ -45,7 +45,7 @@ class OGRXLSLayer : public OGRLayer
 
     char              *pszName;
     int                iSheet;
-    int                bFirstLineIsHeaders;
+    bool               bFirstLineIsHeaders;
     int                nRows;
     unsigned short     nCols;
 
@@ -63,22 +63,20 @@ class OGRXLSLayer : public OGRLayer
                                     int iSheetIn,
                                     int nRowsIn,
                                     unsigned short nColsIn);
-                        ~OGRXLSLayer();
+                        virtual ~OGRXLSLayer();
 
+    virtual void                ResetReading() override;
+    virtual OGRFeature *        GetNextFeature() override;
 
-    virtual void                ResetReading();
-    virtual OGRFeature *        GetNextFeature();
+    virtual OGRFeatureDefn *    GetLayerDefn() override;
+    virtual GIntBig             GetFeatureCount( int bForce = TRUE ) override;
 
-    virtual OGRFeatureDefn *    GetLayerDefn();
-    virtual GIntBig             GetFeatureCount( int bForce = TRUE );
+    virtual const char         *GetName() override { return pszName; }
+    virtual OGRwkbGeometryType  GetGeomType() override { return wkbNone; }
 
-    virtual const char         *GetName() { return pszName; }
-    virtual OGRwkbGeometryType  GetGeomType() { return wkbNone; }
-
-    virtual int                 TestCapability( const char * );
-
-    virtual OGRSpatialReference *GetSpatialRef() { return NULL; }
+    virtual int                 TestCapability( const char * ) override;
 
+    virtual OGRSpatialReference *GetSpatialRef() override { return NULL; }
 };
 
 /************************************************************************/
@@ -96,17 +94,17 @@ class OGRXLSDataSource : public OGRDataSource
 
   public:
                         OGRXLSDataSource();
-                        ~OGRXLSDataSource();
+                        virtual ~OGRXLSDataSource();
 
     int                 Open( const char * pszFilename,
                               int bUpdate );
 
-    virtual const char*         GetName() { return pszName; }
+    virtual const char*         GetName() override { return pszName; }
 
-    virtual int                 GetLayerCount() { return nLayers; }
-    virtual OGRLayer*           GetLayer( int );
+    virtual int                 GetLayerCount() override { return nLayers; }
+    virtual OGRLayer*           GetLayer( int ) override;
 
-    virtual int                 TestCapability( const char * );
+    virtual int                 TestCapability( const char * ) override;
 
     const void                 *GetXLSHandle();
 };
@@ -118,12 +116,11 @@ class OGRXLSDataSource : public OGRDataSource
 class OGRXLSDriver : public OGRSFDriver
 {
   public:
-                ~OGRXLSDriver();
+                virtual ~OGRXLSDriver();
 
-    virtual const char*         GetName();
-    virtual OGRDataSource*      Open( const char *, int );
-    virtual int                 TestCapability( const char * );
+    virtual const char*         GetName() override;
+    virtual OGRDataSource*      Open( const char *, int ) override;
+    virtual int                 TestCapability( const char * ) override;
 };
 
-
 #endif /* ndef OGR_XLS_H_INCLUDED */
