@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: cpl_aws.cpp 33758 2016-03-21 09:06:22Z rouault $
  *
  * Name:     cpl_aws.cpp
  * Project:  CPL - Common Portability Library
@@ -28,6 +27,8 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
+//! @cond Doxygen_Suppress
+
 #include "cpl_aws.h"
 #include "cpl_vsi_error.h"
 #include "cpl_sha256.h"
@@ -35,9 +36,9 @@
 #include "cpl_minixml.h"
 #include <algorithm>
 
-CPL_CVSID("$Id: cpl_aws.cpp 33758 2016-03-21 09:06:22Z rouault $");
+CPL_CVSID("$Id: cpl_aws.cpp 37174 2017-01-18 20:00:36Z rouault $");
 
-//#define DEBUG_VERBOSE 1
+// #define DEBUG_VERBOSE 1
 
 /************************************************************************/
 /*                         CPLGetLowerCaseHex()                         */
@@ -69,8 +70,8 @@ static CPLString CPLGetLowerCaseHex( const GByte *pabyData, size_t nBytes )
 
 CPLString CPLGetLowerCaseHexSHA256( const void *pabyData, size_t nBytes )
 {
-    GByte hash[CPL_SHA256_HASH_SIZE];
-    CPL_SHA256((const GByte*)pabyData, nBytes, hash);
+    GByte hash[CPL_SHA256_HASH_SIZE] = {};
+    CPL_SHA256(static_cast<const GByte *>(pabyData), nBytes, hash);
     return CPLGetLowerCaseHex(hash, CPL_SHA256_HASH_SIZE);
 }
 
@@ -87,18 +88,19 @@ CPLString CPLGetLowerCaseHexSHA256( const CPLString& osStr )
 /*                       CPLAWSURLEncode()                              */
 /************************************************************************/
 
-CPLString CPLAWSURLEncode(const CPLString& osURL, bool bEncodeSlash)
+CPLString CPLAWSURLEncode( const CPLString& osURL, bool bEncodeSlash )
 {
     CPLString osRet;
     for( size_t i = 0; i < osURL.size(); i++ )
     {
         char ch = osURL[i];
-        if ((ch >= 'A' && ch <= 'Z') || (ch >= 'a' && ch <= 'z') ||
-            (ch >= '0' && ch <= '9') || ch == '_' || ch == '-' || ch == '~' || ch == '.')
+        if( (ch >= 'A' && ch <= 'Z') || (ch >= 'a' && ch <= 'z') ||
+            (ch >= '0' && ch <= '9') ||
+            ch == '_' || ch == '-' || ch == '~' || ch == '.' )
         {
             osRet += ch;
         }
-        else if (ch == '/')
+        else if( ch == '/' )
         {
             if( bEncodeSlash )
                 osRet += "%2F";
@@ -117,40 +119,42 @@ CPLString CPLAWSURLEncode(const CPLString& osURL, bool bEncodeSlash)
 /*                CPLGetAWS_SIGN4_Authorization()                       */
 /************************************************************************/
 
-/* See http://docs.aws.amazon.com/AmazonS3/latest/API/sig-v4-header-based-auth.html */
-CPLString CPLGetAWS_SIGN4_Authorization(const CPLString& osSecretAccessKey,
-                                           const CPLString& osAccessKeyId,
-                                           const CPLString& osAccessToken,
-                                           const CPLString& osAWSRegion,
-                                           const CPLString& osService,
-                                           const CPLString& osVerb,
-                                           const CPLString& osHost,
-                                           const CPLString& osCanonicalURI,
-                                           const CPLString& osCanonicalQueryString,
-                                           const CPLString& osXAMZContentSHA256,
-                                           const CPLString& osTimestamp)
+// See:
+// http://docs.aws.amazon.com/AmazonS3/latest/API/sig-v4-header-based-auth.html
+CPLString
+CPLGetAWS_SIGN4_Authorization( const CPLString& osSecretAccessKey,
+                               const CPLString& osAccessKeyId,
+                               const CPLString& osAccessToken,
+                               const CPLString& osAWSRegion,
+                               const CPLString& osRequestPayer,
+                               const CPLString& osService,
+                               const CPLString& osVerb,
+                               const CPLString& osHost,
+                               const CPLString& osCanonicalURI,
+                               const CPLString& osCanonicalQueryString,
+                               const CPLString& osXAMZContentSHA256,
+                               const CPLString& osTimestamp )
 {
 /* -------------------------------------------------------------------- */
 /*      Compute canonical request string.                               */
 /* -------------------------------------------------------------------- */
-    CPLString osCanonicalRequest;
-    osCanonicalRequest = osVerb + "\n";
+    CPLString osCanonicalRequest = osVerb + "\n";
 
     osCanonicalRequest += osCanonicalURI + "\n";
 
     osCanonicalRequest += osCanonicalQueryString + "\n";
 
-    CPLString osCanonicalHeaders;
-    osCanonicalHeaders += "host:";
-    osCanonicalHeaders += osHost;
-    osCanonicalHeaders += "\n";
-    osCanonicalHeaders += "x-amz-content-sha256:";
-    osCanonicalHeaders += osXAMZContentSHA256;
-    osCanonicalHeaders += "\n";
-    osCanonicalHeaders += "x-amz-date:";
-    osCanonicalHeaders += osTimestamp;
-    osCanonicalHeaders += "\n";
-    if( osAccessToken.size() )
+    CPLString osCanonicalHeaders =
+        "host:" + osHost + "\n" +
+        "x-amz-content-sha256:" + osXAMZContentSHA256 + "\n" +
+        "x-amz-date:" + osTimestamp + "\n";
+    if( !osRequestPayer.empty() )
+    {
+        osCanonicalHeaders += "x-amz-request-payer:";
+        osCanonicalHeaders += osRequestPayer;
+        osCanonicalHeaders += "\n";
+    }
+    if( !osAccessToken.empty() )
     {
         osCanonicalHeaders += "x-amz-security-token:";
         osCanonicalHeaders += osAccessToken;
@@ -160,14 +164,16 @@ CPLString CPLGetAWS_SIGN4_Authorization(const CPLString& osSecretAccessKey,
     osCanonicalRequest += osCanonicalHeaders + "\n";
 
     CPLString osSignedHeaders = "host;x-amz-content-sha256;x-amz-date";
-    if( osAccessToken.size() )
+    if( !osRequestPayer.empty() )
+        osSignedHeaders += ";x-amz-request-payer";
+    if( !osAccessToken.empty() )
         osSignedHeaders += ";x-amz-security-token";
     osCanonicalRequest += osSignedHeaders + "\n";
 
     osCanonicalRequest += osXAMZContentSHA256;
 
 #ifdef DEBUG_VERBOSE
-    CPLDebug("S3", "osCanonicalRequest='%s'\n", osCanonicalRequest.c_str());
+    CPLDebug("S3", "osCanonicalRequest='%s'", osCanonicalRequest.c_str());
 #endif
 
 /* -------------------------------------------------------------------- */
@@ -188,14 +194,14 @@ CPLString CPLGetAWS_SIGN4_Authorization(const CPLString& osSecretAccessKey,
     osStringToSign += CPLGetLowerCaseHexSHA256(osCanonicalRequest);
 
 #ifdef DEBUG_VERBOSE
-    CPLDebug("S3", "osStringToSign='%s'\n", osStringToSign.c_str());
+    CPLDebug("S3", "osStringToSign='%s'", osStringToSign.c_str());
 #endif
 
 /* -------------------------------------------------------------------- */
 /*      Compute signing key.                                            */
 /* -------------------------------------------------------------------- */
-    GByte abySigningKeyIn[CPL_SHA256_HASH_SIZE];
-    GByte abySigningKeyOut[CPL_SHA256_HASH_SIZE];
+    GByte abySigningKeyIn[CPL_SHA256_HASH_SIZE] = {};
+    GByte abySigningKeyOut[CPL_SHA256_HASH_SIZE] = {};
 
     CPLString osFirstKey(CPLString("AWS4") + osSecretAccessKey);
     CPL_HMAC_SHA256( osFirstKey.c_str(), osFirstKey.size(),
@@ -219,21 +225,23 @@ CPLString CPLGetAWS_SIGN4_Authorization(const CPLString& osSecretAccessKey,
     memcpy(abySigningKeyIn, abySigningKeyOut, CPL_SHA256_HASH_SIZE);
 
 #ifdef DEBUG_VERBOSE
-    CPLString osSigningKey(CPLGetLowerCaseHex(abySigningKeyIn, CPL_SHA256_HASH_SIZE));
-    CPLDebug("S3", "osSigningKey='%s'\n", osSigningKey.c_str());
+    CPLString osSigningKey(CPLGetLowerCaseHex(abySigningKeyIn,
+                                              CPL_SHA256_HASH_SIZE));
+    CPLDebug("S3", "osSigningKey='%s'", osSigningKey.c_str());
 #endif
 
 /* -------------------------------------------------------------------- */
 /*      Compute signature.                                              */
 /* -------------------------------------------------------------------- */
-    GByte abySignature[CPL_SHA256_HASH_SIZE];
+    GByte abySignature[CPL_SHA256_HASH_SIZE] = {};
     CPL_HMAC_SHA256( abySigningKeyIn, CPL_SHA256_HASH_SIZE,
                      osStringToSign, osStringToSign.size(),
                      abySignature);
-    CPLString osSignature(CPLGetLowerCaseHex(abySignature, CPL_SHA256_HASH_SIZE));
+    CPLString osSignature(CPLGetLowerCaseHex(abySignature,
+                                             CPL_SHA256_HASH_SIZE));
 
 #ifdef DEBUG_VERBOSE
-    CPLDebug("S3", "osSignature='%s'\n", osSignature.c_str());
+    CPLDebug("S3", "osSignature='%s'", osSignature.c_str());
 #endif
 
 /* -------------------------------------------------------------------- */
@@ -258,7 +266,7 @@ CPLString CPLGetAWS_SIGN4_Authorization(const CPLString& osSecretAccessKey,
     osAuthorization += osSignature;
 
 #ifdef DEBUG_VERBOSE
-    CPLDebug("S3", "osAuthorization='%s'\n", osAuthorization.c_str());
+    CPLDebug("S3", "osAuthorization='%s'", osAuthorization.c_str());
 #endif
 
     return osAuthorization;
@@ -273,7 +281,7 @@ CPLString CPLGetAWS_SIGN4_Timestamp()
     struct tm brokenDown;
     CPLUnixTimeToYMDHMS(time(NULL), &brokenDown);
 
-    char szTimeStamp[4+2+2+1+2+2+2+1+1];
+    char szTimeStamp[4+2+2+1+2+2+2+1+1] = {};
     snprintf(szTimeStamp, sizeof(szTimeStamp), "%04d%02d%02dT%02d%02d%02dZ",
             brokenDown.tm_year + 1900,
             brokenDown.tm_mon + 1,
@@ -289,27 +297,29 @@ CPLString CPLGetAWS_SIGN4_Timestamp()
 /************************************************************************/
 /*                         VSIS3HandleHelper()                          */
 /************************************************************************/
-
-VSIS3HandleHelper::VSIS3HandleHelper(   const CPLString& osSecretAccessKey,
-                                        const CPLString& osAccessKeyId,
-                                        const CPLString& osSessionToken,
-                                        const CPLString& osAWSS3Endpoint,
-                                        const CPLString& osAWSRegion,
-                                        const CPLString& osBucket,
-                                        const CPLString& osObjectKey,
-                                        bool bUseHTTPS, bool bUseVirtualHosting) :
-        m_osURL(BuildURL(osAWSS3Endpoint, osBucket, osObjectKey, bUseHTTPS, bUseVirtualHosting)),
-        m_osSecretAccessKey(osSecretAccessKey),
-        m_osAccessKeyId(osAccessKeyId),
-        m_osSessionToken(osSessionToken),
-        m_osAWSS3Endpoint(osAWSS3Endpoint),
-        m_osAWSRegion(osAWSRegion),
-        m_osBucket(osBucket),
-        m_osObjectKey(osObjectKey),
-        m_bUseHTTPS(bUseHTTPS),
-        m_bUseVirtualHosting(bUseVirtualHosting)
-{
-}
+VSIS3HandleHelper::VSIS3HandleHelper( const CPLString& osSecretAccessKey,
+                                      const CPLString& osAccessKeyId,
+                                      const CPLString& osSessionToken,
+                                      const CPLString& osAWSS3Endpoint,
+                                      const CPLString& osAWSRegion,
+                                      const CPLString& osRequestPayer,
+                                      const CPLString& osBucket,
+                                      const CPLString& osObjectKey,
+                                      bool bUseHTTPS,
+                                      bool bUseVirtualHosting ) :
+    m_osURL(BuildURL(osAWSS3Endpoint, osBucket, osObjectKey, bUseHTTPS,
+                     bUseVirtualHosting)),
+    m_osSecretAccessKey(osSecretAccessKey),
+    m_osAccessKeyId(osAccessKeyId),
+    m_osSessionToken(osSessionToken),
+    m_osAWSS3Endpoint(osAWSS3Endpoint),
+    m_osAWSRegion(osAWSRegion),
+    m_osRequestPayer(osRequestPayer),
+    m_osBucket(osBucket),
+    m_osObjectKey(osObjectKey),
+    m_bUseHTTPS(bUseHTTPS),
+    m_bUseVirtualHosting(bUseVirtualHosting)
+{}
 
 /************************************************************************/
 /*                        ~VSIS3HandleHelper()                          */
@@ -317,7 +327,7 @@ VSIS3HandleHelper::VSIS3HandleHelper(   const CPLString& osSecretAccessKey,
 
 VSIS3HandleHelper::~VSIS3HandleHelper()
 {
-    for(size_t i=0;i<m_osSecretAccessKey.size();i++)
+    for( size_t i = 0; i < m_osSecretAccessKey.size(); i++ )
         m_osSecretAccessKey[i] = 0;
 }
 
@@ -348,8 +358,10 @@ CPLString VSIS3HandleHelper::BuildURL(const CPLString& osAWSS3Endpoint,
 
 void VSIS3HandleHelper::RebuildURL()
 {
-    m_osURL = BuildURL(m_osAWSS3Endpoint, m_osBucket, m_osObjectKey, m_bUseHTTPS, m_bUseVirtualHosting);
-    std::map<CPLString, CPLString>::iterator oIter = m_oMapQueryParameters.begin();
+    m_osURL = BuildURL(m_osAWSS3Endpoint, m_osBucket, m_osObjectKey,
+                       m_bUseHTTPS, m_bUseVirtualHosting);
+    std::map<CPLString, CPLString>::iterator oIter =
+        m_oMapQueryParameters.begin();
     for( ; oIter != m_oMapQueryParameters.end(); ++oIter )
     {
         if( oIter == m_oMapQueryParameters.begin() )
@@ -357,7 +369,7 @@ void VSIS3HandleHelper::RebuildURL()
         else
             m_osURL += "&";
         m_osURL += oIter->first;
-        if( oIter->second.size() )
+        if( !oIter->second.empty() )
         {
             m_osURL += "=";
             m_osURL += oIter->second;
@@ -369,12 +381,14 @@ void VSIS3HandleHelper::RebuildURL()
 /*                        GetBucketAndObjectKey()                       */
 /************************************************************************/
 
-bool VSIS3HandleHelper::GetBucketAndObjectKey(const char* pszURI, const char* pszFSPrefix,
-                                              bool bAllowNoObject,
-                                              CPLString &osBucket, CPLString &osObjectKey)
+bool VSIS3HandleHelper::GetBucketAndObjectKey( const char* pszURI,
+                                               const char* pszFSPrefix,
+                                               bool bAllowNoObject,
+                                               CPLString &osBucket,
+                                               CPLString &osObjectKey )
 {
     osBucket = pszURI;
-    if( osBucket.size() == 0 )
+    if( osBucket.empty() )
     {
         return false;
     }
@@ -399,40 +413,51 @@ bool VSIS3HandleHelper::GetBucketAndObjectKey(const char* pszURI, const char* ps
 /*                          BuildFromURI()                              */
 /************************************************************************/
 
-VSIS3HandleHelper* VSIS3HandleHelper::BuildFromURI(const char* pszURI,
-                                                   const char* pszFSPrefix,
-                                                   bool bAllowNoObject)
+VSIS3HandleHelper* VSIS3HandleHelper::BuildFromURI( const char* pszURI,
+                                                    const char* pszFSPrefix,
+                                                    bool bAllowNoObject )
 {
-    CPLString osSecretAccessKey = CPLGetConfigOption("AWS_SECRET_ACCESS_KEY", "");
-    if( osSecretAccessKey.size() == 0 )
+    const CPLString osSecretAccessKey =
+        CPLGetConfigOption("AWS_SECRET_ACCESS_KEY", "");
+    if( osSecretAccessKey.empty() )
     {
         VSIError(VSIE_AWSInvalidCredentials,
                  "AWS_SECRET_ACCESS_KEY configuration option not defined");
         return NULL;
     }
-    CPLString osAccessKeyId = CPLGetConfigOption("AWS_ACCESS_KEY_ID", "");
-    if( osAccessKeyId.size() == 0 )
+    const CPLString osAccessKeyId = CPLGetConfigOption("AWS_ACCESS_KEY_ID", "");
+    if( osAccessKeyId.empty() )
     {
         VSIError(VSIE_AWSInvalidCredentials,
                  "AWS_ACCESS_KEY_ID configuration option not defined");
         return NULL;
     }
-    CPLString osSessionToken = CPLGetConfigOption("AWS_SESSION_TOKEN", "");
-    CPLString osAWSS3Endpoint = CPLGetConfigOption("AWS_S3_ENDPOINT", "s3.amazonaws.com");
-    CPLString osAWSRegion = CPLGetConfigOption("AWS_REGION", "us-east-1");
-    CPLString osBucket, osObjectKey;
-    if( !GetBucketAndObjectKey(pszURI, pszFSPrefix, bAllowNoObject, osBucket, osObjectKey) )
+    const CPLString osSessionToken =
+        CPLGetConfigOption("AWS_SESSION_TOKEN", "");
+    const CPLString osAWSS3Endpoint =
+        CPLGetConfigOption("AWS_S3_ENDPOINT", "s3.amazonaws.com");
+    const CPLString osAWSRegion = CPLGetConfigOption("AWS_REGION", "us-east-1");
+    const CPLString osRequestPayer =
+        CPLGetConfigOption("AWS_REQUEST_PAYER", "");
+    CPLString osBucket;
+    CPLString osObjectKey;
+    if( !GetBucketAndObjectKey(pszURI, pszFSPrefix, bAllowNoObject,
+                               osBucket, osObjectKey) )
     {
         return NULL;
     }
-    bool bUseHTTPS = CPL_TO_BOOL(CSLTestBoolean(CPLGetConfigOption("AWS_HTTPS", "YES")));
-    bool bIsValidNameForVirtualHosting = (osBucket.find('.') == std::string::npos);
-    bool bUseVirtualHosting = CPL_TO_BOOL(CSLTestBoolean(
-            CPLGetConfigOption("AWS_VIRTUAL_HOSTING",
-                               bIsValidNameForVirtualHosting ? "TRUE" : "FALSE")));
-    return new VSIS3HandleHelper(osSecretAccessKey, osAccessKeyId, osSessionToken,
-                                    osAWSS3Endpoint, osAWSRegion,
-                                    osBucket, osObjectKey, bUseHTTPS, bUseVirtualHosting);
+    const bool bUseHTTPS = CPLTestBool(CPLGetConfigOption("AWS_HTTPS", "YES"));
+    const bool bIsValidNameForVirtualHosting =
+        osBucket.find('.') == std::string::npos;
+    const bool bUseVirtualHosting = CPLTestBool(
+        CPLGetConfigOption("AWS_VIRTUAL_HOSTING",
+                           bIsValidNameForVirtualHosting ? "TRUE" : "FALSE"));
+    return new VSIS3HandleHelper(osSecretAccessKey, osAccessKeyId,
+                                 osSessionToken,
+                                 osAWSS3Endpoint, osAWSRegion,
+                                 osRequestPayer,
+                                 osBucket, osObjectKey, bUseHTTPS,
+                                 bUseVirtualHosting);
 }
 
 /************************************************************************/
@@ -449,7 +474,8 @@ void VSIS3HandleHelper::ResetQueryParameters()
 /*                         AddQueryParameter()                          */
 /************************************************************************/
 
-void VSIS3HandleHelper::AddQueryParameter(const CPLString& osKey, const CPLString& osValue)
+void VSIS3HandleHelper::AddQueryParameter( const CPLString& osKey,
+                                           const CPLString& osValue )
 {
     m_oMapQueryParameters[osKey] = osValue;
     RebuildURL();
@@ -459,50 +485,64 @@ void VSIS3HandleHelper::AddQueryParameter(const CPLString& osKey, const CPLStrin
 /*                           GetCurlHeaders()                           */
 /************************************************************************/
 
-struct curl_slist* VSIS3HandleHelper::GetCurlHeaders(const CPLString& osVerb,
-                                                     const void *pabyDataContent,
-                                                     size_t nBytesContent)
+struct curl_slist *
+VSIS3HandleHelper::GetCurlHeaders( const CPLString& osVerb,
+                                   const void *pabyDataContent,
+                                   size_t nBytesContent )
 {
     CPLString osXAMZDate = CPLGetConfigOption("AWS_TIMESTAMP", "");
-    if( osXAMZDate.size() == 0 )
+    if( osXAMZDate.empty() )
         osXAMZDate = CPLGetAWS_SIGN4_Timestamp();
 
-    CPLString osXAMZContentSHA256 = CPLGetLowerCaseHexSHA256(pabyDataContent, nBytesContent);
+    const CPLString osXAMZContentSHA256 =
+        CPLGetLowerCaseHexSHA256(pabyDataContent, nBytesContent);
 
     CPLString osCanonicalQueryString;
-    std::map<CPLString, CPLString>::iterator oIter = m_oMapQueryParameters.begin();
+    std::map<CPLString, CPLString>::iterator oIter =
+        m_oMapQueryParameters.begin();
     for( ; oIter != m_oMapQueryParameters.end(); ++oIter )
     {
-        if( osCanonicalQueryString.size() )
+        if( !osCanonicalQueryString.empty() )
             osCanonicalQueryString += "&";
         osCanonicalQueryString += oIter->first;
         osCanonicalQueryString += "=";
         osCanonicalQueryString += CPLAWSURLEncode(oIter->second);
     }
 
-    CPLString osAuthorization = CPLGetAWS_SIGN4_Authorization(
-            m_osSecretAccessKey,
-            m_osAccessKeyId,
-            m_osSessionToken,
-            m_osAWSRegion,
-            "s3",
-            osVerb,
-            (m_bUseVirtualHosting) ? m_osBucket + "." + m_osAWSS3Endpoint : m_osAWSS3Endpoint,
-            (m_bUseVirtualHosting) ? ("/" + m_osObjectKey).c_str() : ("/" + m_osBucket + "/" + m_osObjectKey).c_str(),
-            osCanonicalQueryString,
-            osXAMZContentSHA256,
-            osXAMZDate);
+    const CPLString osHost(m_bUseVirtualHosting
+        ? CPLString(m_osBucket + "." + m_osAWSS3Endpoint) : m_osAWSS3Endpoint);
+    const CPLString osAuthorization = CPLGetAWS_SIGN4_Authorization(
+        m_osSecretAccessKey,
+        m_osAccessKeyId,
+        m_osSessionToken,
+        m_osAWSRegion,
+        m_osRequestPayer,
+        "s3",
+        osVerb,
+        osHost,
+        m_bUseVirtualHosting
+        ? ("/" + m_osObjectKey).c_str() :
+        ("/" + m_osBucket + "/" + m_osObjectKey).c_str(),
+        osCanonicalQueryString,
+        osXAMZContentSHA256,
+        osXAMZDate);
 
     struct curl_slist *headers=NULL;
-    headers = curl_slist_append(headers,
-                                CPLSPrintf("x-amz-date: %s", osXAMZDate.c_str()));
-    headers = curl_slist_append(headers,
-                                CPLSPrintf("x-amz-content-sha256: %s", osXAMZContentSHA256.c_str()));
-    if( m_osSessionToken.size() )
-        headers = curl_slist_append(headers,
-                                CPLSPrintf("X-Amz-Security-Token: %s", m_osSessionToken.c_str()));
-    headers = curl_slist_append(headers,
-                                CPLSPrintf("Authorization: %s", osAuthorization.c_str()));
+    headers = curl_slist_append(
+        headers, CPLSPrintf("x-amz-date: %s", osXAMZDate.c_str()));
+    headers = curl_slist_append(
+        headers, CPLSPrintf("x-amz-content-sha256: %s",
+                            osXAMZContentSHA256.c_str()));
+    if( !m_osSessionToken.empty() )
+        headers = curl_slist_append(
+            headers,
+            CPLSPrintf("X-Amz-Security-Token: %s", m_osSessionToken.c_str()));
+    if( !m_osRequestPayer.empty() )
+        headers = curl_slist_append(
+            headers,
+            CPLSPrintf("x-amz-request-payer: %s", m_osRequestPayer.c_str()));
+    headers = curl_slist_append(
+        headers, CPLSPrintf("Authorization: %s", osAuthorization.c_str()));
     return headers;
 }
 
@@ -510,7 +550,8 @@ struct curl_slist* VSIS3HandleHelper::GetCurlHeaders(const CPLString& osVerb,
 /*                          CanRestartOnError()                         */
 /************************************************************************/
 
-bool VSIS3HandleHelper::CanRestartOnError(const char* pszErrorMsg, bool bSetError)
+bool VSIS3HandleHelper::CanRestartOnError( const char* pszErrorMsg,
+                                           bool bSetError )
 {
 #ifdef DEBUG_VERBOSE
     CPLDebug("S3", "%s", pszErrorMsg);
@@ -518,14 +559,21 @@ bool VSIS3HandleHelper::CanRestartOnError(const char* pszErrorMsg, bool bSetErro
 
     if( !STARTS_WITH(pszErrorMsg, "<?xml") )
     {
-        if(bSetError) { VSIError(VSIE_AWSError, "Invalid AWS response: %s", pszErrorMsg); }
+        if( bSetError )
+        {
+            VSIError(VSIE_AWSError, "Invalid AWS response: %s", pszErrorMsg);
+        }
         return false;
     }
 
     CPLXMLNode* psTree = CPLParseXMLString(pszErrorMsg);
     if( psTree == NULL )
     {
-        if(bSetError) { VSIError(VSIE_AWSError, "Malformed AWS XML repsonse: %s", pszErrorMsg); }
+        if( bSetError )
+        {
+            VSIError(VSIE_AWSError,
+                     "Malformed AWS XML response: %s", pszErrorMsg);
+        }
         return false;
     }
 
@@ -533,7 +581,11 @@ bool VSIS3HandleHelper::CanRestartOnError(const char* pszErrorMsg, bool bSetErro
     if( pszCode == NULL )
     {
         CPLDestroyXMLNode(psTree);
-        if(bSetError) { VSIError(VSIE_AWSError, "Malformed AWS XML repsonse: %s", pszErrorMsg); }
+        if( bSetError )
+        {
+            VSIError(VSIE_AWSError,
+                     "Malformed AWS XML response: %s", pszErrorMsg);
+        }
         return false;
     }
 
@@ -543,7 +595,11 @@ bool VSIS3HandleHelper::CanRestartOnError(const char* pszErrorMsg, bool bSetErro
         if( pszRegion == NULL )
         {
             CPLDestroyXMLNode(psTree);
-            if(bSetError) { VSIError(VSIE_AWSError, "Malformed AWS XML repsonse: %s", pszErrorMsg); }
+            if( bSetError )
+            {
+                VSIError(VSIE_AWSError,
+                         "Malformed AWS XML response: %s", pszErrorMsg);
+            }
             return false;
         }
         SetAWSRegion(pszRegion);
@@ -554,13 +610,20 @@ bool VSIS3HandleHelper::CanRestartOnError(const char* pszErrorMsg, bool bSetErro
 
     if( EQUAL(pszCode, "PermanentRedirect") )
     {
-        const char* pszEndpoint = CPLGetXMLValue(psTree, "=Error.Endpoint", NULL);
+        const char* pszEndpoint =
+            CPLGetXMLValue(psTree, "=Error.Endpoint", NULL);
         if( pszEndpoint == NULL ||
-            (m_bUseVirtualHosting && (strncmp(pszEndpoint, m_osBucket.c_str(), m_osBucket.size()) != 0 ||
-                                    pszEndpoint[m_osBucket.size()] != '.')) )
+            (m_bUseVirtualHosting &&
+             (strncmp(pszEndpoint, m_osBucket.c_str(),
+                      m_osBucket.size()) != 0 ||
+              pszEndpoint[m_osBucket.size()] != '.')) )
         {
             CPLDestroyXMLNode(psTree);
-            if(bSetError) { VSIError(VSIE_AWSError, "Malformed AWS XML repsonse: %s", pszErrorMsg); }
+            if( bSetError )
+            {
+                VSIError(VSIE_AWSError,
+                         "Malformed AWS XML response: %s", pszErrorMsg);
+            }
             return false;
         }
         if( !m_bUseVirtualHosting &&
@@ -570,16 +633,18 @@ bool VSIS3HandleHelper::CanRestartOnError(const char* pszErrorMsg, bool bSetErro
             m_bUseVirtualHosting = true;
             CPLDebug("S3", "Switching to virtual hosting");
         }
-        SetAWSS3Endpoint((m_bUseVirtualHosting) ? pszEndpoint + m_osBucket.size() + 1 : pszEndpoint);
+        SetAWSS3Endpoint(
+            m_bUseVirtualHosting
+            ? pszEndpoint + m_osBucket.size() + 1
+            : pszEndpoint);
         CPLDebug("S3", "Switching to endpoint %s", m_osAWSS3Endpoint.c_str());
         CPLDestroyXMLNode(psTree);
         return true;
     }
 
-    if(bSetError) {
-        /*
-         * Translate AWS errors into VSI errors
-         */
+    if( bSetError )
+    {
+        // Translate AWS errors into VSI errors.
         const char* pszMessage = CPLGetXMLValue(psTree, "=Error.Message", NULL);
 
         if( pszMessage == NULL ) {
@@ -606,7 +671,7 @@ bool VSIS3HandleHelper::CanRestartOnError(const char* pszErrorMsg, bool bSetErro
 /*                          SetAWSS3Endpoint()                          */
 /************************************************************************/
 
-void VSIS3HandleHelper::SetAWSS3Endpoint(const CPLString &osStr)
+void VSIS3HandleHelper::SetAWSS3Endpoint( const CPLString &osStr )
 {
     m_osAWSS3Endpoint = osStr;
     RebuildURL();
@@ -616,15 +681,25 @@ void VSIS3HandleHelper::SetAWSS3Endpoint(const CPLString &osStr)
 /*                           SetAWSRegion()                             */
 /************************************************************************/
 
-void VSIS3HandleHelper::SetAWSRegion(const CPLString &osStr)
+void VSIS3HandleHelper::SetAWSRegion( const CPLString &osStr )
 {
     m_osAWSRegion = osStr;
 }
+
+/************************************************************************/
+/*                           SetRequestPayer()                          */
+/************************************************************************/
+
+void VSIS3HandleHelper::SetRequestPayer( const CPLString &osStr )
+{
+    m_osRequestPayer = osStr;
+}
+
 /************************************************************************/
 /*                         SetVirtualHosting()                          */
 /************************************************************************/
 
-void VSIS3HandleHelper::SetVirtualHosting(bool b)
+void VSIS3HandleHelper::SetVirtualHosting( bool b )
 {
     m_bUseVirtualHosting = b;
     RebuildURL();
@@ -634,10 +709,12 @@ void VSIS3HandleHelper::SetVirtualHosting(bool b)
 /*                           SetObjectKey()                             */
 /************************************************************************/
 
-void VSIS3HandleHelper::SetObjectKey(const CPLString &osStr)
+void VSIS3HandleHelper::SetObjectKey( const CPLString &osStr )
 {
     m_osObjectKey = osStr;
     RebuildURL();
 }
 
 #endif
+
+//! @endcond
