@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id: cpl_vsi_mem.cpp 35728 2016-10-14 16:17:28Z rouault $
  *
  * Project:  VSI Virtual File System
  * Purpose:  Implementation of Memory Buffer virtual IO functions.
@@ -28,14 +27,34 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
+#include "cpl_port.h"
+#include "cpl_vsi.h"
 #include "cpl_vsi_virtual.h"
-#include "cpl_string.h"
-#include "cpl_multiproc.h"
-#include "cpl_atomic_ops.h"
-#include <time.h>
-#include <map>
 
-CPL_CVSID("$Id: cpl_vsi_mem.cpp 35728 2016-10-14 16:17:28Z rouault $");
+#include <cerrno>
+#include <cstddef>
+#include <cstring>
+#include <ctime>
+#if HAVE_FCNTL_H
+#  include <fcntl.h>
+#endif
+#if HAVE_SYS_STAT_H
+#  include <sys/stat.h>
+#endif
+
+#include <map>
+#include <string>
+#include <utility>
+
+#include "cpl_atomic_ops.h"
+#include "cpl_conv.h"
+#include "cpl_error.h"
+#include "cpl_multiproc.h"
+#include "cpl_string.h"
+
+//! @cond Doxygen_Suppress
+
+CPL_CVSID("$Id: cpl_vsi_mem.cpp 36990 2016-12-21 19:42:05Z goatbar $");
 
 /*
 ** Notes on Multithreading:
@@ -76,12 +95,13 @@ public:
     CPLString     osFilename;
     volatile int  nRefCount;
 
-    int           bIsDirectory;
+    bool          bIsDirectory;
 
-    int           bOwnData;
+    bool          bOwnData;
     GByte        *pabyData;
     vsi_l_offset  nLength;
     vsi_l_offset  nAllocLength;
+    vsi_l_offset  nMaxLength;
 
     time_t        mTime;
 
@@ -102,20 +122,27 @@ class VSIMemHandle CPL_FINAL : public VSIVirtualHandle
   public:
     VSIMemFile    *poFile;
     vsi_l_offset  m_nOffset;
-    int           bUpdate;
-    int           bEOF;
-    int           bExtendFileAtNextWrite;
+    bool          bUpdate;
+    bool          bEOF;
+    bool          bExtendFileAtNextWrite;
 
-                      VSIMemHandle() : poFile(NULL), m_nOffset(0), bUpdate(0),
-                                       bEOF(0), bExtendFileAtNextWrite(0) {}
+    VSIMemHandle() :
+        poFile(NULL),
+        m_nOffset(0),
+        bUpdate(false),
+        bEOF(false),
+        bExtendFileAtNextWrite(false) {}
+    virtual ~VSIMemHandle() {}
 
-    virtual int       Seek( vsi_l_offset nOffset, int nWhence );
-    virtual vsi_l_offset Tell();
-    virtual size_t    Read( void *pBuffer, size_t nSize, size_t nMemb );
-    virtual size_t    Write( const void *pBuffer, size_t nSize, size_t nMemb );
-    virtual int       Eof();
-    virtual int       Close();
-    virtual int       Truncate( vsi_l_offset nNewSize );
+    virtual int       Seek( vsi_l_offset nOffset, int nWhence ) override;
+    virtual vsi_l_offset Tell() override;
+    virtual size_t    Read( void *pBuffer, size_t nSize,
+                            size_t nMemb ) override;
+    virtual size_t    Write( const void *pBuffer, size_t nSize,
+                             size_t nMemb ) override;
+    virtual int       Eof() override;
+    virtual int       Close() override;
+    virtual int       Truncate( vsi_l_offset nNewSize ) override;
 };
 
 /************************************************************************/
@@ -126,25 +153,29 @@ class VSIMemHandle CPL_FINAL : public VSIVirtualHandle
 
 class VSIMemFilesystemHandler CPL_FINAL : public VSIFilesystemHandler
 {
-public:
-    std::map<CPLString,VSIMemFile*>   oFileList;
+  public:
+    std::map<CPLString, VSIMemFile*> oFileList;
     CPLMutex        *hMutex;
 
                      VSIMemFilesystemHandler();
     virtual          ~VSIMemFilesystemHandler();
 
+    // TODO(schwehr): Fix VSIFileFromMemBuffer so that using is not needed.
     using VSIFilesystemHandler::Open;
 
     virtual VSIVirtualHandle *Open( const char *pszFilename,
                                     const char *pszAccess,
-                                    bool bSetError );
-    virtual int      Stat( const char *pszFilename, VSIStatBufL *pStatBuf, int nFlags );
-    virtual int      Unlink( const char *pszFilename );
-    virtual int      Mkdir( const char *pszDirname, long nMode );
-    virtual int      Rmdir( const char *pszDirname );
-    virtual char   **ReadDirEx( const char *pszDirname, int nMaxFiles );
-    virtual int      Rename( const char *oldpath, const char *newpath );
-    virtual GIntBig  GetDiskFreeSpace( const char* pszDirname );
+                                    bool bSetError ) override;
+    virtual int      Stat( const char *pszFilename, VSIStatBufL *pStatBuf,
+                           int nFlags ) override;
+    virtual int      Unlink( const char *pszFilename ) override;
+    virtual int      Mkdir( const char *pszDirname, long nMode ) override;
+    virtual int      Rmdir( const char *pszDirname ) override;
+    virtual char   **ReadDirEx( const char *pszDirname,
+                                int nMaxFiles ) override;
+    virtual int      Rename( const char *oldpath,
+                             const char *newpath ) override;
+    virtual GIntBig  GetDiskFreeSpace( const char* pszDirname ) override;
 
     static  void     NormalizePath( CPLString & );
 
@@ -163,11 +194,12 @@ public:
 
 VSIMemFile::VSIMemFile() :
     nRefCount(0),
-    bIsDirectory(FALSE),
-    bOwnData(TRUE),
+    bIsDirectory(false),
+    bOwnData(true),
     pabyData(NULL),
     nLength(0),
-    nAllocLength(0)
+    nAllocLength(0),
+    nMaxLength(GUINTBIG_MAX)
 {
     time(&mTime);
 }
@@ -195,39 +227,52 @@ VSIMemFile::~VSIMemFile()
 bool VSIMemFile::SetLength( vsi_l_offset nNewLength )
 
 {
+    if( nNewLength > nMaxLength )
+    {
+        CPLError(CE_Failure, CPLE_NotSupported,
+                 "Maximum file size reached!");
+        return false;
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Grow underlying array if needed.                                */
 /* -------------------------------------------------------------------- */
     if( nNewLength > nAllocLength )
     {
-        /* If we don't own the buffer, we cannot reallocate it because */
-        /* the return address might be different from the one passed by */
-        /* the caller. Hence, the caller would not be able to free */
-        /* the buffer... */
+        // If we don't own the buffer, we cannot reallocate it because
+        // the return address might be different from the one passed by
+        // the caller. Hence, the caller would not be able to free
+        // the buffer.
         if( !bOwnData )
         {
-            CPLError(CE_Failure, CPLE_NotSupported,
-                     "Cannot extended in-memory file whose ownership was not transferred");
+            CPLError(
+                CE_Failure, CPLE_NotSupported,
+                "Cannot extended in-memory file whose ownership was not "
+                "transferred" );
             return false;
         }
 
-        GByte *pabyNewData;
         const vsi_l_offset nNewAlloc = (nNewLength + nNewLength / 10) + 5000;
-        if( (vsi_l_offset)(size_t)nNewAlloc != nNewAlloc )
-            pabyNewData = NULL;
-        else
-            pabyNewData = (GByte *) VSIRealloc(pabyData, (size_t)nNewAlloc);
+        GByte *pabyNewData = NULL;
+        if( static_cast<vsi_l_offset>(static_cast<size_t>(nNewAlloc))
+            == nNewAlloc )
+        {
+            pabyNewData = static_cast<GByte *>(
+                VSIRealloc(pabyData, static_cast<size_t>(nNewAlloc) ));
+        }
         if( pabyNewData == NULL )
         {
-            CPLError(CE_Failure, CPLE_OutOfMemory,
-                     "Cannot extend in-memory file to " CPL_FRMT_GUIB " bytes due to out-of-memory situation",
-                     nNewAlloc);
+            CPLError(
+                CE_Failure, CPLE_OutOfMemory,
+                "Cannot extend in-memory file to " CPL_FRMT_GUIB
+                " bytes due to out-of-memory situation",
+                nNewAlloc);
             return false;
         }
 
-        /* Clear the new allocated part of the buffer */
+        // Clear the new allocated part of the buffer.
         memset(pabyNewData + nAllocLength, 0,
-               (size_t) (nNewAlloc - nAllocLength));
+               static_cast<size_t>(nNewAlloc - nAllocLength));
 
         pabyData = pabyNewData;
         nAllocLength = nNewAlloc;
@@ -245,7 +290,6 @@ bool VSIMemFile::SetLength( vsi_l_offset nNewLength )
 /* ==================================================================== */
 /************************************************************************/
 
-
 /************************************************************************/
 /*                               Close()                                */
 /************************************************************************/
@@ -268,37 +312,45 @@ int VSIMemHandle::Close()
 int VSIMemHandle::Seek( vsi_l_offset nOffset, int nWhence )
 
 {
-    bExtendFileAtNextWrite = FALSE;
+    bExtendFileAtNextWrite = false;
     if( nWhence == SEEK_CUR )
+    {
         m_nOffset += nOffset;
+    }
     else if( nWhence == SEEK_SET )
+    {
         m_nOffset = nOffset;
+    }
     else if( nWhence == SEEK_END )
+    {
         m_nOffset = poFile->nLength + nOffset;
+    }
     else
     {
         errno = EINVAL;
         return -1;
     }
 
-    bEOF = FALSE;
+    bEOF = false;
 
     if( m_nOffset > poFile->nLength )
     {
-        if( !bUpdate ) // Read-only files cannot be extended by seek.
+        if( !bUpdate )  // Read-only files cannot be extended by seek.
         {
-            CPLDebug( "VSIMemHandle",
-                      "Attempt to extend read-only file '%s' to length " CPL_FRMT_GUIB " from " CPL_FRMT_GUIB ".",
-                      poFile->osFilename.c_str(),
-                      m_nOffset, poFile->nLength );
+            CPLDebug(
+                "VSIMemHandle",
+                "Attempt to extend read-only file '%s' to length " CPL_FRMT_GUIB
+                " from " CPL_FRMT_GUIB ".",
+                poFile->osFilename.c_str(),
+                m_nOffset, poFile->nLength);
 
             m_nOffset = poFile->nLength;
             errno = EACCES;
             return -1;
         }
-        else // Writable files are zero-extended by seek past end.
+        else  // Writable files are zero-extended by seek past end.
         {
-            bExtendFileAtNextWrite = TRUE;
+            bExtendFileAtNextWrite = true;
         }
     }
 
@@ -327,19 +379,20 @@ size_t VSIMemHandle::Read( void * pBuffer, size_t nSize, size_t nCount )
 
     if( nBytesToRead + m_nOffset > poFile->nLength )
     {
-        if (poFile->nLength < m_nOffset)
+        if( poFile->nLength < m_nOffset )
         {
-            bEOF = TRUE;
+            bEOF = true;
             return 0;
         }
 
-        nBytesToRead = (size_t)(poFile->nLength - m_nOffset);
+        nBytesToRead = static_cast<size_t>(poFile->nLength - m_nOffset);
         nCount = nBytesToRead / nSize;
-        bEOF = TRUE;
+        bEOF = true;
     }
 
     if( nBytesToRead )
-        memcpy( pBuffer, poFile->pabyData + m_nOffset, (size_t)nBytesToRead );
+        memcpy( pBuffer, poFile->pabyData + m_nOffset,
+                static_cast<size_t>(nBytesToRead) );
     m_nOffset += nBytesToRead;
 
     return nCount;
@@ -359,7 +412,7 @@ size_t VSIMemHandle::Write( const void * pBuffer, size_t nSize, size_t nCount )
     }
     if( bExtendFileAtNextWrite )
     {
-        bExtendFileAtNextWrite = FALSE;
+        bExtendFileAtNextWrite = false;
         if( !poFile->SetLength( m_nOffset ) )
             return 0;
     }
@@ -404,11 +457,11 @@ int VSIMemHandle::Truncate( vsi_l_offset nNewSize )
         return -1;
     }
 
-    bExtendFileAtNextWrite = FALSE;
-    if (poFile->SetLength( nNewSize ))
+    bExtendFileAtNextWrite = false;
+    if( poFile->SetLength( nNewSize ) )
         return 0;
-    else
-        return -1;
+
+    return -1;
 }
 
 /************************************************************************/
@@ -423,7 +476,7 @@ int VSIMemHandle::Truncate( vsi_l_offset nNewSize )
 
 VSIMemFilesystemHandler::VSIMemFilesystemHandler() :
     hMutex(NULL)
-{ }
+{}
 
 /************************************************************************/
 /*                      ~VSIMemFilesystemHandler()                      */
@@ -432,7 +485,8 @@ VSIMemFilesystemHandler::VSIMemFilesystemHandler() :
 VSIMemFilesystemHandler::~VSIMemFilesystemHandler()
 
 {
-    for( std::map<CPLString,VSIMemFile*>::const_iterator iter = oFileList.begin();
+    for( std::map<CPLString, VSIMemFile*>::const_iterator iter =
+             oFileList.begin();
          iter != oFileList.end();
          ++iter )
     {
@@ -459,37 +513,48 @@ VSIMemFilesystemHandler::Open( const char *pszFilename,
     CPLString osFilename = pszFilename;
     NormalizePath( osFilename );
 
+    vsi_l_offset nMaxLength = GUINTBIG_MAX;
+    const size_t iPos = osFilename.find("||maxlength=");
+    if( iPos != std::string::npos )
+    {
+        nMaxLength = static_cast<vsi_l_offset>(CPLAtoGIntBig(
+                    osFilename.substr(iPos + strlen("||maxlength=")).c_str()));
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Get the filename we are opening, create if needed.              */
 /* -------------------------------------------------------------------- */
-    VSIMemFile *poFile;
-    if( oFileList.find(osFilename) == oFileList.end() )
-        poFile = NULL;
-    else
+    VSIMemFile *poFile = NULL;
+    if( oFileList.find(osFilename) != oFileList.end() )
         poFile = oFileList[osFilename];
 
-    // If no file and opening in read, error out
-    if( strstr(pszAccess,"w") == NULL
+    // If no file and opening in read, error out.
+    if( strstr(pszAccess, "w") == NULL
         && strstr(pszAccess, "a") == NULL
         && poFile == NULL )
     {
-        if(bSetError) { VSIError(VSIE_FileError, "No such file or directory"); }
+        if( bSetError )
+        {
+            VSIError(VSIE_FileError, "No such file or directory");
+        }
         errno = ENOENT;
         return NULL;
     }
 
-    // Create
+    // Create.
     if( poFile == NULL )
     {
         poFile = new VSIMemFile;
         poFile->osFilename = osFilename;
         oFileList[poFile->osFilename] = poFile;
-        CPLAtomicInc(&(poFile->nRefCount)); // for file list
+        CPLAtomicInc(&(poFile->nRefCount));  // For file list.
+        poFile->nMaxLength = nMaxLength;
     }
     // Overwrite
     else if( strstr(pszAccess, "w") )
     {
         poFile->SetLength(0);
+        poFile->nMaxLength = nMaxLength;
     }
 
     if( poFile->bIsDirectory )
@@ -505,16 +570,15 @@ VSIMemFilesystemHandler::Open( const char *pszFilename,
 
     poHandle->poFile = poFile;
     poHandle->m_nOffset = 0;
-    poHandle->bEOF = FALSE;
-    if( strstr(pszAccess,"w") || strstr(pszAccess,"+")
-        || strstr(pszAccess,"a") )
-        poHandle->bUpdate = TRUE;
-    else
-        poHandle->bUpdate = FALSE;
+    poHandle->bEOF = false;
+    poHandle->bUpdate =
+        strstr(pszAccess, "w") ||
+        strstr(pszAccess, "+") ||
+        strstr(pszAccess, "a");
 
     CPLAtomicInc(&(poFile->nRefCount));
 
-    if( strstr(pszAccess,"a") )
+    if( strstr(pszAccess, "a") )
         poHandle->m_nOffset = poFile->nLength;
 
     return poHandle;
@@ -536,7 +600,7 @@ int VSIMemFilesystemHandler::Stat( const char * pszFilename,
 
     memset( pStatBuf, 0, sizeof(VSIStatBufL) );
 
-    if ( osFilename == "/vsimem/" )
+    if( osFilename == "/vsimem/" )
     {
         pStatBuf->st_size = 0;
         pStatBuf->st_mode = S_IFDIR;
@@ -589,7 +653,6 @@ int VSIMemFilesystemHandler::Unlink_unlocked( const char * pszFilename )
     CPLString osFilename = pszFilename;
     NormalizePath( osFilename );
 
-
     if( oFileList.find(osFilename) == oFileList.end() )
     {
         errno = ENOENT;
@@ -629,9 +692,9 @@ int VSIMemFilesystemHandler::Mkdir( const char * pszPathname,
     VSIMemFile *poFile = new VSIMemFile;
 
     poFile->osFilename = osPathname;
-    poFile->bIsDirectory = TRUE;
+    poFile->bIsDirectory = true;
     oFileList[osPathname] = poFile;
-    CPLAtomicInc(&(poFile->nRefCount)); /* referenced by file list */
+    CPLAtomicInc(&(poFile->nRefCount));  // Referenced by file list.
 
     return 0;
 }
@@ -660,35 +723,37 @@ char **VSIMemFilesystemHandler::ReadDirEx( const char *pszPath,
 
     NormalizePath( osPath );
 
-    std::map<CPLString,VSIMemFile*>::const_iterator iter;
     char **papszDir = NULL;
-    size_t nPathLen = strlen(osPath);
+    size_t nPathLen = osPath.size();
 
-    if( nPathLen > 0 && osPath[nPathLen-1] == '/' )
+    if( nPathLen > 0 && osPath.back() == '/' )
         nPathLen--;
 
-    /* In case of really big number of files in the directory, CSLAddString */
-    /* can be slow (see #2158). We then directly build the list. */
-    int nItems=0;
-    int nAllocatedItems=0;
+    // In case of really big number of files in the directory, CSLAddString
+    // can be slow (see #2158). We then directly build the list.
+    int nItems = 0;
+    int nAllocatedItems = 0;
 
-    for( iter = oFileList.begin(); iter != oFileList.end(); ++iter )
+    for( std::map<CPLString, VSIMemFile*>::const_iterator iter =
+             oFileList.begin();
+         iter != oFileList.end();
+         ++iter )
     {
         const char *pszFilePath = iter->second->osFilename.c_str();
-        if( EQUALN(osPath,pszFilePath,nPathLen)
+        if( EQUALN(osPath, pszFilePath, nPathLen)
             && pszFilePath[nPathLen] == '/'
-            && strstr(pszFilePath+nPathLen+1,"/") == NULL )
+            && strstr(pszFilePath+nPathLen+1, "/") == NULL )
         {
-            if (nItems == 0)
+            if( nItems == 0 )
             {
-                papszDir = (char**) CPLCalloc(2,sizeof(char*));
+                papszDir = static_cast<char**>(CPLCalloc(2, sizeof(char*)));
                 nAllocatedItems = 1;
             }
-            else if (nItems >= nAllocatedItems)
+            else if( nItems >= nAllocatedItems )
             {
                 nAllocatedItems = nAllocatedItems * 2;
-                papszDir = (char**)CPLRealloc(papszDir,
-                                              (nAllocatedItems+2)*sizeof(char*));
+                papszDir = static_cast<char**>(
+                    CPLRealloc(papszDir, (nAllocatedItems + 2)*sizeof(char*)) );
             }
 
             papszDir[nItems] = CPLStrdup(pszFilePath+nPathLen+1);
@@ -719,7 +784,7 @@ int VSIMemFilesystemHandler::Rename( const char *pszOldPath,
     NormalizePath( osOldPath );
     NormalizePath( osNewPath );
 
-    if ( osOldPath.compare(osNewPath) == 0 )
+    if( osOldPath.compare(osNewPath) == 0 )
         return 0;
 
     if( oFileList.find(osOldPath) == oFileList.end() )
@@ -728,11 +793,11 @@ int VSIMemFilesystemHandler::Rename( const char *pszOldPath,
         return -1;
     }
 
-    std::map<CPLString,VSIMemFile*>::iterator it = oFileList.find(osOldPath);
-    while (it != oFileList.end() && it->first.ifind(osOldPath) == 0)
+    std::map<CPLString, VSIMemFile*>::iterator it = oFileList.find(osOldPath);
+    while( it != oFileList.end() && it->first.ifind(osOldPath) == 0 )
     {
         const CPLString osRemainder = it->first.substr(osOldPath.size());
-        if (osRemainder.empty() || osRemainder[0] == '/')
+        if( osRemainder.empty() || osRemainder[0] == '/' )
         {
             const CPLString osNewFullPath = osNewPath + osRemainder;
             Unlink_unlocked(osNewFullPath);
@@ -740,7 +805,10 @@ int VSIMemFilesystemHandler::Rename( const char *pszOldPath,
             it->second->osFilename = osNewFullPath;
             oFileList.erase(it++);
         }
-        else ++it;
+        else
+        {
+            ++it;
+        }
     }
 
     return 0;
@@ -760,7 +828,6 @@ void VSIMemFilesystemHandler::NormalizePath( CPLString &oPath )
         if( oPath[i] == '\\' )
             oPath[i] = '/';
     }
-
 }
 
 /************************************************************************/
@@ -769,12 +836,14 @@ void VSIMemFilesystemHandler::NormalizePath( CPLString &oPath )
 
 GIntBig VSIMemFilesystemHandler::GetDiskFreeSpace( const char* /*pszDirname*/ )
 {
-    GIntBig nRet = CPLGetUsablePhysicalRAM();
+    const GIntBig nRet = CPLGetUsablePhysicalRAM();
     if( nRet <= 0 )
-        nRet = -1;
+        return -1;
     return nRet;
 }
 
+//! @endcond
+
 /************************************************************************/
 /*                     VSIInstallLargeFileHandler()                     */
 /************************************************************************/
@@ -863,19 +932,20 @@ void VSIInstallMemFileHandler()
  */
 
 VSILFILE *VSIFileFromMemBuffer( const char *pszFilename,
-                          GByte *pabyData,
-                          vsi_l_offset nDataLength,
-                          int bTakeOwnership )
+                                GByte *pabyData,
+                                vsi_l_offset nDataLength,
+                                int bTakeOwnership )
 
 {
     if( VSIFileManager::GetHandler("")
         == VSIFileManager::GetHandler("/vsimem/") )
         VSIInstallMemFileHandler();
 
-    VSIMemFilesystemHandler *poHandler = (VSIMemFilesystemHandler *)
-        VSIFileManager::GetHandler("/vsimem/");
+    VSIMemFilesystemHandler *poHandler =
+        static_cast<VSIMemFilesystemHandler *>(
+                VSIFileManager::GetHandler("/vsimem/"));
 
-    if (pszFilename == NULL)
+    if( pszFilename == NULL )
         return NULL;
 
     CPLString osFilename = pszFilename;
@@ -884,7 +954,7 @@ VSILFILE *VSIFileFromMemBuffer( const char *pszFilename,
     VSIMemFile *poFile = new VSIMemFile;
 
     poFile->osFilename = osFilename;
-    poFile->bOwnData = bTakeOwnership;
+    poFile->bOwnData = CPL_TO_BOOL(bTakeOwnership);
     poFile->pabyData = pabyData;
     poFile->nLength = nDataLength;
     poFile->nAllocLength = nDataLength;
@@ -896,7 +966,9 @@ VSILFILE *VSIFileFromMemBuffer( const char *pszFilename,
         CPLAtomicInc(&(poFile->nRefCount));
     }
 
-    return (VSILFILE *) poHandler->Open( osFilename, "r+" );
+    // TODO(schwehr): Fix this so that the using statement is not needed.
+    // Will just adding the bool for bSetError be okay?
+    return reinterpret_cast<VSILFILE *>( poHandler->Open( osFilename, "r+" ) );
 }
 
 /************************************************************************/
@@ -923,10 +995,11 @@ GByte *VSIGetMemFileBuffer( const char *pszFilename,
                             int bUnlinkAndSeize )
 
 {
-    VSIMemFilesystemHandler *poHandler = (VSIMemFilesystemHandler *)
-        VSIFileManager::GetHandler("/vsimem/");
+    VSIMemFilesystemHandler *poHandler =
+        static_cast<VSIMemFilesystemHandler *>(
+            VSIFileManager::GetHandler("/vsimem/"));
 
-    if (pszFilename == NULL)
+    if( pszFilename == NULL )
         return NULL;
 
     CPLString osFilename = pszFilename;
@@ -948,7 +1021,7 @@ GByte *VSIGetMemFileBuffer( const char *pszFilename,
             CPLDebug( "VSIMemFile",
                       "File doesn't own data in VSIGetMemFileBuffer!" );
         else
-            poFile->bOwnData = FALSE;
+            poFile->bOwnData = false;
 
         poHandler->oFileList.erase( poHandler->oFileList.find(osFilename) );
         CPLAtomicDec(&(poFile->nRefCount));
