@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id$
  *
  * Project:  VSI Virtual File System
  * Purpose:  Implementation of caching IO layer.
@@ -28,10 +27,27 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
+#include "cpl_port.h"
 #include "cpl_vsi_virtual.h"
-#include <map>
 
-CPL_CVSID("$Id$");
+#include <cstddef>
+#include <cstring>
+#if HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+
+#include <algorithm>
+#include <map>
+#include <utility>
+
+#include "cpl_conv.h"
+#include "cpl_error.h"
+#include "cpl_vsi.h"
+#include "cpl_vsi_virtual.h"
+
+//! @cond Doxygen_Suppress
+
+CPL_CVSID("$Id: cpl_vsil_cache.cpp 36772 2016-12-10 06:29:22Z goatbar $");
 
 /************************************************************************/
 /* ==================================================================== */
@@ -43,13 +59,13 @@ class VSICacheChunk
 {
 public:
   VSICacheChunk() :
-      bDirty(FALSE),
+      bDirty(false),
       iBlock(0),
       poLRUPrev(NULL),
       poLRUNext(NULL),
       nDataFilled(0),
       pabyData(NULL)
-    { }
+    {}
 
     virtual ~VSICacheChunk()
     {
@@ -59,11 +75,11 @@ public:
     bool Allocate( size_t nChunkSize )
     {
         CPLAssert( pabyData == NULL );
-        pabyData = (GByte *)VSIMalloc( nChunkSize );
+        pabyData = static_cast<GByte *>(VSIMalloc( nChunkSize ));
         return (pabyData != NULL);
     }
 
-    int            bDirty;
+    bool           bDirty;  // TODO(schwehr): Not used?  Added in r22564.
     vsi_l_offset   iBlock;
 
     VSICacheChunk *poLRUPrev;
@@ -85,7 +101,7 @@ class VSICachedFile CPL_FINAL : public VSIVirtualHandle
     VSICachedFile( VSIVirtualHandle *poBaseHandle,
                    size_t nChunkSize,
                    size_t nCacheSize );
-    ~VSICachedFile() { Close(); }
+    virtual ~VSICachedFile() { Close(); }
 
     void          FlushLRU();
     int           LoadBlocks( vsi_l_offset nStartBlock, size_t nBlockCount,
@@ -107,43 +123,44 @@ class VSICachedFile CPL_FINAL : public VSIVirtualHandle
 
     std::map<vsi_l_offset, VSICacheChunk*> oMapOffsetToCache;
 
-    int            bEOF;
+    bool           bEOF;
 
-    virtual int       Seek( vsi_l_offset nOffset, int nWhence );
-    virtual vsi_l_offset Tell();
-    virtual size_t    Read( void *pBuffer, size_t nSize, size_t nMemb );
-    virtual size_t    Write( const void *pBuffer, size_t nSize, size_t nMemb );
-    virtual int       Eof();
-    virtual int       Flush();
-    virtual int       Close();
-    virtual void     *GetNativeFileDescriptor() { return poBase->GetNativeFileDescriptor(); }
+    virtual int       Seek( vsi_l_offset nOffset, int nWhence ) override;
+    virtual vsi_l_offset Tell() override;
+    virtual size_t    Read( void *pBuffer, size_t nSize,
+                            size_t nMemb ) override;
+    virtual size_t    Write( const void *pBuffer, size_t nSize,
+                             size_t nMemb ) override;
+    virtual int       Eof() override;
+    virtual int       Flush() override;
+    virtual int       Close() override;
+    virtual void     *GetNativeFileDescriptor() override
+        { return poBase->GetNativeFileDescriptor(); }
 };
 
 /************************************************************************/
 /*                           VSICachedFile()                            */
 /************************************************************************/
 
-VSICachedFile::VSICachedFile( VSIVirtualHandle *poBaseHandle, size_t nChunkSize, size_t nCacheSize )
-
+VSICachedFile::VSICachedFile( VSIVirtualHandle *poBaseHandle, size_t nChunkSize,
+                              size_t nCacheSize ) :
+    poBase(poBaseHandle),
+    nOffset(0),
+    nFileSize(0),  // Set below.
+    nCacheUsed(0),
+    nCacheMax(nCacheSize),
+    poLRUStart(NULL),
+    poLRUEnd(NULL),
+    bEOF(false)
 {
-    poBase = poBaseHandle;
     m_nChunkSize = nChunkSize;
 
-    nCacheUsed = 0;
-    if ( nCacheSize == 0 )
+    if( nCacheSize == 0 )
         nCacheMax = CPLScanUIntBig(
              CPLGetConfigOption( "VSI_CACHE_SIZE", "25000000" ), 40 );
-    else
-        nCacheMax = nCacheSize;
-
-    poLRUStart = NULL;
-    poLRUEnd = NULL;
 
     poBase->Seek( 0, SEEK_END );
     nFileSize = poBase->Tell();
-
-    nOffset = 0;
-    bEOF = FALSE;
 }
 
 /************************************************************************/
@@ -153,8 +170,10 @@ VSICachedFile::VSICachedFile( VSIVirtualHandle *poBaseHandle, size_t nChunkSize,
 int VSICachedFile::Close()
 
 {
-    for( std::map<vsi_l_offset, VSICacheChunk*>::iterator oIter = oMapOffsetToCache.begin();
-         oIter != oMapOffsetToCache.end(); ++oIter )
+    for( std::map<vsi_l_offset, VSICacheChunk*>::iterator oIter =
+             oMapOffsetToCache.begin();
+         oIter != oMapOffsetToCache.end();
+         ++oIter )
     {
         delete oIter->second;
     }
@@ -182,18 +201,16 @@ int VSICachedFile::Close()
 int VSICachedFile::Seek( vsi_l_offset nReqOffset, int nWhence )
 
 {
-    bEOF = FALSE;
+    bEOF = false;
 
     if( nWhence == SEEK_SET )
     {
-        // use offset directly.
+        // Use offset directly.
     }
-
     else if( nWhence == SEEK_CUR )
     {
         nReqOffset += nOffset;
     }
-
     else if( nWhence == SEEK_END )
     {
         nReqOffset += nFileSize;
@@ -254,7 +271,7 @@ void VSICachedFile::FlushLRU()
 void VSICachedFile::Demote( VSICacheChunk *poBlock )
 
 {
-    // already at end?
+    // Already at end?
     if( poLRUEnd == poBlock )
         return;
 
@@ -272,6 +289,7 @@ void VSICachedFile::Demote( VSICacheChunk *poBlock )
 
     if( poLRUEnd != NULL )
         poLRUEnd->poLRUNext = poBlock;
+
     poLRUEnd = poBlock;
 
     if( poLRUStart == NULL )
@@ -283,6 +301,8 @@ void VSICachedFile::Demote( VSICacheChunk *poBlock )
 /*                                                                      */
 /*      Load the desired set of blocks.  Use pBuffer as a temporary     */
 /*      buffer if it would be helpful.                                  */
+/*                                                                      */
+/*  RETURNS: TRUE on success; FALSE on failure.                         */
 /************************************************************************/
 
 int VSICachedFile::LoadBlocks( vsi_l_offset nStartBlock, size_t nBlockCount,
@@ -290,7 +310,7 @@ int VSICachedFile::LoadBlocks( vsi_l_offset nStartBlock, size_t nBlockCount,
 
 {
     if( nBlockCount == 0 )
-        return 1;
+        return TRUE;
 
 /* -------------------------------------------------------------------- */
 /*      When we want to load only one block, we can directly load it    */
@@ -298,25 +318,27 @@ int VSICachedFile::LoadBlocks( vsi_l_offset nStartBlock, size_t nBlockCount,
 /* -------------------------------------------------------------------- */
     if( nBlockCount == 1 )
     {
-        poBase->Seek( (vsi_l_offset)nStartBlock * m_nChunkSize, SEEK_SET );
+        poBase->Seek( static_cast<vsi_l_offset>(nStartBlock) * m_nChunkSize,
+                      SEEK_SET );
 
         VSICacheChunk *poBlock = new VSICacheChunk();
-        if ( !poBlock || !poBlock->Allocate( m_nChunkSize ) )
+        if( !poBlock || !poBlock->Allocate( m_nChunkSize ) )
         {
             delete poBlock;
-            return 0;
+            return FALSE;
         }
 
         oMapOffsetToCache[nStartBlock] = poBlock;
 
         poBlock->iBlock = nStartBlock;
-        poBlock->nDataFilled = poBase->Read( poBlock->pabyData, 1, m_nChunkSize );
+        poBlock->nDataFilled =
+            poBase->Read( poBlock->pabyData, 1, m_nChunkSize );
         nCacheUsed += poBlock->nDataFilled;
 
         // Merges into the LRU list.
         Demote( poBlock );
 
-        return 1;
+        return TRUE;
     }
 
 /* -------------------------------------------------------------------- */
@@ -329,27 +351,30 @@ int VSICachedFile::LoadBlocks( vsi_l_offset nStartBlock, size_t nBlockCount,
         && nBufferSize < nBlockCount * m_nChunkSize )
     {
         if( !LoadBlocks( nStartBlock, 2, pBuffer, nBufferSize ) )
-            return 0;
+            return FALSE;
 
         return LoadBlocks( nStartBlock+2, nBlockCount-2, pBuffer, nBufferSize );
     }
 
-    if( poBase->Seek( (vsi_l_offset)nStartBlock * m_nChunkSize, SEEK_SET ) != 0 )
-        return 0;
+    if( poBase->Seek( static_cast<vsi_l_offset>(nStartBlock) * m_nChunkSize,
+                      SEEK_SET ) != 0 )
+        return FALSE;
 
 /* -------------------------------------------------------------------- */
 /*      Do we need to allocate our own buffer?                          */
 /* -------------------------------------------------------------------- */
-    GByte *pabyWorkBuffer = (GByte *) pBuffer;
+    GByte *pabyWorkBuffer = static_cast<GByte *>(pBuffer);
 
     if( nBufferSize < m_nChunkSize * nBlockCount )
-        pabyWorkBuffer = (GByte *) CPLMalloc(m_nChunkSize * nBlockCount);
+        pabyWorkBuffer =
+            static_cast<GByte *>( CPLMalloc(m_nChunkSize * nBlockCount) );
 
 /* -------------------------------------------------------------------- */
 /*      Read the whole request into the working buffer.                 */
 /* -------------------------------------------------------------------- */
 
-    size_t nDataRead = poBase->Read( pabyWorkBuffer, 1, nBlockCount*m_nChunkSize);
+    const size_t nDataRead =
+        poBase->Read( pabyWorkBuffer, 1, nBlockCount*m_nChunkSize);
 
     if( nBlockCount * m_nChunkSize > nDataRead + m_nChunkSize - 1 )
         nBlockCount = (nDataRead + m_nChunkSize - 1) / m_nChunkSize;
@@ -357,10 +382,10 @@ int VSICachedFile::LoadBlocks( vsi_l_offset nStartBlock, size_t nBlockCount,
     for( size_t i = 0; i < nBlockCount; i++ )
     {
         VSICacheChunk *poBlock = new VSICacheChunk();
-        if ( !poBlock || !poBlock->Allocate( m_nChunkSize ) )
+        if( !poBlock || !poBlock->Allocate( m_nChunkSize ) )
         {
             delete poBlock;
-            return 0;
+            return FALSE;
         }
 
         poBlock->iBlock = nStartBlock + i;
@@ -375,7 +400,7 @@ int VSICachedFile::LoadBlocks( vsi_l_offset nStartBlock, size_t nBlockCount,
             poBlock->nDataFilled = nDataRead - i*m_nChunkSize;
 
         memcpy( poBlock->pabyData, pabyWorkBuffer + i*m_nChunkSize,
-                (size_t) poBlock->nDataFilled );
+                static_cast<size_t>(poBlock->nDataFilled) );
 
         nCacheUsed += poBlock->nDataFilled;
 
@@ -386,7 +411,7 @@ int VSICachedFile::LoadBlocks( vsi_l_offset nStartBlock, size_t nBlockCount,
     if( pabyWorkBuffer != pBuffer )
         CPLFree( pabyWorkBuffer );
 
-    return 1;
+    return TRUE;
 }
 
 /************************************************************************/
@@ -398,15 +423,16 @@ size_t VSICachedFile::Read( void * pBuffer, size_t nSize, size_t nCount )
 {
     if( nOffset >= nFileSize )
     {
-        bEOF = TRUE;
+        bEOF = true;
         return 0;
     }
 
 /* ==================================================================== */
 /*      Make sure the cache is loaded for the whole request region.     */
 /* ==================================================================== */
-    vsi_l_offset nStartBlock = nOffset / m_nChunkSize;
-    vsi_l_offset nEndBlock = (nOffset + nSize * nCount - 1) / m_nChunkSize;
+    const vsi_l_offset nStartBlock = nOffset / m_nChunkSize;
+    const vsi_l_offset nEndBlock =
+        (nOffset + nSize * nCount - 1) / m_nChunkSize;
 
     for( vsi_l_offset iBlock = nStartBlock; iBlock <= nEndBlock; iBlock++ )
     {
@@ -428,22 +454,24 @@ size_t VSICachedFile::Read( void * pBuffer, size_t nSize, size_t nCount )
 
     while( nAmountCopied < nSize * nCount )
     {
-        vsi_l_offset iBlock = (nOffset + nAmountCopied) / m_nChunkSize;
-        VSICacheChunk *poBlock = oMapOffsetToCache[iBlock];
+        const vsi_l_offset iBlock = (nOffset + nAmountCopied) / m_nChunkSize;
+        VSICacheChunk * poBlock = oMapOffsetToCache[iBlock];
         if( poBlock == NULL )
         {
-            /* We can reach that point when the amount to read exceeds */
-            /* the cache size */
-            LoadBlocks( iBlock, 1, ((GByte *) pBuffer) + nAmountCopied,
-                        MIN(nSize * nCount - nAmountCopied, m_nChunkSize) );
+            // We can reach that point when the amount to read exceeds
+            // the cache size.
+            LoadBlocks(iBlock, 1,
+                       static_cast<GByte *>(pBuffer) + nAmountCopied,
+                       std::min(nSize * nCount - nAmountCopied, m_nChunkSize));
             poBlock = oMapOffsetToCache[iBlock];
             CPLAssert(poBlock != NULL);
         }
 
-        vsi_l_offset nStartOffset = (vsi_l_offset)iBlock * m_nChunkSize;
-        size_t nThisCopy = (size_t)
+        const vsi_l_offset nStartOffset =
+            static_cast<vsi_l_offset>(iBlock) * m_nChunkSize;
+        size_t nThisCopy = static_cast<size_t>(
             ((nStartOffset + poBlock->nDataFilled)
-             - nAmountCopied - nOffset);
+             - nAmountCopied - nOffset));
 
         if( nThisCopy > nSize * nCount - nAmountCopied )
             nThisCopy = nSize * nCount - nAmountCopied;
@@ -451,7 +479,7 @@ size_t VSICachedFile::Read( void * pBuffer, size_t nSize, size_t nCount )
         if( nThisCopy == 0 )
             break;
 
-        memcpy( ((GByte *) pBuffer) + nAmountCopied,
+        memcpy( static_cast<GByte *>(pBuffer) + nAmountCopied,
                 poBlock->pabyData
                 + (nOffset + nAmountCopied) - nStartOffset,
                 nThisCopy );
@@ -467,9 +495,9 @@ size_t VSICachedFile::Read( void * pBuffer, size_t nSize, size_t nCount )
     while( nCacheUsed > nCacheMax )
         FlushLRU();
 
-    size_t nRet = nAmountCopied / nSize;
-    if (nRet != nCount)
-        bEOF = TRUE;
+    const size_t nRet = nAmountCopied / nSize;
+    if( nRet != nCount )
+        bEOF = true;
     return nRet;
 }
 
@@ -504,12 +532,15 @@ int VSICachedFile::Flush()
     return 0;
 }
 
+//! @endcond
+
 /************************************************************************/
 /*                        VSICreateCachedFile()                         */
 /************************************************************************/
 
 VSIVirtualHandle *
-VSICreateCachedFile( VSIVirtualHandle *poBaseHandle, size_t nChunkSize, size_t nCacheSize )
+VSICreateCachedFile( VSIVirtualHandle *poBaseHandle,
+                     size_t nChunkSize, size_t nCacheSize )
 
 {
     return new VSICachedFile( poBaseHandle, nChunkSize, nCacheSize );
